#include "StdAfx.h"
#include "MainWindow.h"
#include <js/JSEngineFactory.h>
#include <common/SourceFactory.h>
#include <js/SourceJSCode.h>
#include <ar/ARWindow.h>
#include <video/CaptureDeviceFactory.h>
#include <window/WindowFactory.h>
#include <plugin/PluginManager.h>
#include <commctrl.h>
#include <image/TextLoaderFactory.h>

#define ALERT_CAPTION "ARBrowser beta"

#define DEFAULT_PATH	"data/simple.js"
#define DEFAULT_DEBUG_HEIGHT	100

#define MARGIN_CONTROLS	4
#define WIDTH_PATH_BUTTON	64
#define HEIGHT_CONTROLS	22

#define ID_STATUSBAR	300
#define ID_PATH_EDIT	301
#define ID_PATH_BUTTON	302
#define ID_DEBUG_EDIT	303

// \z܂B
MainWindow::MainWindow(HWND hWnd, JSEngineFactory* pJSEngineFactory, CaptureDeviceFactory* pCaptureDeviceFactory, WindowFactory* pWindowFactory, HRESULT* phr)
{
	m_hWnd = hWnd;
	*phr = S_OK;
	m_pARWindow = NULL;
	m_pGLWindow = NULL;
	m_pJSEngine = NULL;
	m_pJSContext = NULL;
	m_pWindowFactory = pWindowFactory;
	m_pCaptureDeviceFactory = pCaptureDeviceFactory;
	m_pStatusWnd = NULL;
	m_pPathEdit = NULL;
	m_hToolbarBrush = NULL;
	m_hVideoBrush = NULL;
	m_pPluginManager = NULL;
	m_pPathButton = NULL;
	m_pDebugEdit = NULL;

	HRESULT hr = Init(pJSEngineFactory);
	if(FAILED(hr)) {
		*phr = hr;
		return;
	}
}

// j܂B
MainWindow::~MainWindow(void)
{
	DestroyContext();

	if(m_pJSEngine != NULL) {
		m_pJSEngine->Destroy();
		m_pJSEngine = NULL;
	}
	if(m_pWindowFactory != NULL) {
		delete m_pWindowFactory;
		m_pWindowFactory = NULL;
	}
	if(m_pCaptureDeviceFactory != NULL) {
		delete m_pCaptureDeviceFactory;
		m_pCaptureDeviceFactory = NULL;
	}
	if(m_hToolbarBrush != NULL) {
		DeleteObject(m_hToolbarBrush);
		m_hToolbarBrush = NULL;
	}
	if(m_hVideoBrush != NULL) {
		DeleteObject(m_hVideoBrush);
		m_hVideoBrush = NULL;
	}
	if(m_pPathButton != NULL) {
		delete m_pPathButton;
		m_pPathButton = NULL;
	}
	if(m_pPathEdit != NULL) {
		delete m_pPathEdit;
		m_pPathEdit = NULL;
	}
	if(m_pDebugEdit != NULL) {
		delete m_pDebugEdit;
		m_pDebugEdit = NULL;
	}
	if(m_pStatusWnd != NULL) {
		delete m_pStatusWnd;
		m_pStatusWnd = NULL;
	}
}

// XNvgs܂B
HRESULT MainWindow::Execute(const char *path)
{
	if(m_pARWindow != NULL) {
		DestroyContext();
	}

	// ReLXg̏
	IJSContext* pContext = NULL;
	HRESULT hr = m_pJSEngine->CreateContext(& pContext);
	if(FAILED(hr)) {
		return hr;
	}
	m_pJSContext = pContext;
 
	ICaptureDevice* pCaptureDevice = NULL;
	hr = m_pCaptureDeviceFactory->Create(& pCaptureDevice);
	if(FAILED(hr)) {
		return hr;
	}

	IWindow* pGLWindow = NULL;
	hr = m_pWindowFactory->Create(& pGLWindow);
	if(FAILED(hr)) {
		pCaptureDevice->Destroy();
		pCaptureDevice = NULL;
		return hr;
	}

	hr = pGLWindow->SetParent(m_hWnd);
	if(FAILED(hr)) {
		pCaptureDevice->Destroy();
		pCaptureDevice = NULL;
		pGLWindow->Destroy();
		pGLWindow = NULL;
		return hr;
	}

	WindowWrapper* pWindowW = new WindowWrapper(this, pGLWindow);
	m_pGLWindow = pWindowW;
	ARWindow* pWindow = new ARWindow(pWindowW, pCaptureDevice, & hr);
	if(FAILED(hr)) {
		pCaptureDevice->Destroy();
		pCaptureDevice = NULL;
		pGLWindow->Destroy();
		pGLWindow = NULL;
		return hr;
	}
	m_pARWindow = pWindow;

	hr = pContext->SetNotify(pWindowW);
	if(FAILED(hr)) {
		return hr;
	}

	hr = pWindow->Attach(pContext);
	if(FAILED(hr)) {
		return hr;
	}

	hr = S_OK;
	PluginManager* pPluginManager = new PluginManager(".\\", pWindowW, pWindow, & hr);
	if(FAILED(hr)) {
		delete pPluginManager;
		pPluginManager = NULL;
		return hr;
	}

	hr = pPluginManager->Attach(pContext);
	if(FAILED(hr)) {
		delete pPluginManager;
		pPluginManager = NULL;
		return hr;
	}
	m_pPluginManager = pPluginManager;

	// TextLoader
	{
		TextLoaderFactory textLoaderFactory;
		IImageLoader* pImageLoader = NULL;
		hr = textLoaderFactory.Create(m_hWnd, & pImageLoader);
		if(FAILED(hr)) {
			return hr;
		}

		hr = m_pARWindow->AddImageLoader(TEXT_LOADER_ENTRY, pImageLoader);
		if(FAILED(hr)) {
			return hr;
		}
	}

	char filepath[MAX_PATH + 1];
	DWORD dret = GetModuleFileNameA(GetModuleHandle(NULL), filepath, MAX_PATH);
	if(dret == 0) {
		return E_FAIL;
	}
	SourceFactory sourceFactory(filepath, & hr);
	if(FAILED(hr)) {
		return hr;
	}

	Debug("Evaluating system.js...");
	hr = Evaluate(& sourceFactory, "system.js");
	if(FAILED(hr)) {
		return hr;
	}

	string msg = "Evaluating ";
	msg += path;
	msg += "...";
	Debug(msg.c_str());
	hr = Evaluate(& sourceFactory, path);
	if(FAILED(hr)) {
		return hr;
	}

	hr = pWindow->Start();
	if(FAILED(hr)) {
		return hr;
	}

    hr = m_pStatusWnd->SetText(0, path);
	if(FAILED(hr)) {
		return hr;
	}

	return S_OK;
}

// EBhEB
HRESULT MainWindow::WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam, bool* pPreventDefault)
{
	int wmId = 0;
	int wmEvent = 0;
	HRESULT hr = S_OK;
	switch(message)
	{
	case WM_COMMAND:
		wmId    = LOWORD(wParam);
		wmEvent = HIWORD(wParam);
		// Iꂽj[̉:
		switch (wmId)
		{
		case ID_PATH_EDIT:
			break;
		case ID_PATH_BUTTON:
			if(wmEvent == 0) {
				string path;
				hr = m_pPathEdit->GetText(path);
				if(FAILED(hr)) {
					return hr;
				}
				return Execute(path.c_str());
			}
			break;
		}
		break;
	case WM_SIZE:
		if(hWnd == m_hWnd && m_pStatusWnd != NULL) {
			SendMessage(m_pStatusWnd->GetHWnd(), message, wParam, lParam);
		}
		hr = OnSize(LOWORD(lParam), HIWORD(lParam));
		if(FAILED(hr)) {
			return hr;
		}
		break;
	case WM_PAINT:
		if(m_hWnd == hWnd && m_hWnd != NULL) {
			PAINTSTRUCT ps;
			HDC hDC = BeginPaint(m_hWnd, &ps);
			if(hDC != NULL) {
				hr = OnPaint(hDC);
				EndPaint(m_hWnd, &ps);
				return hr;
			}
		}
	}

	// ftHg͌ĂяołȂ
	return S_OK;
}

// \z܂B
MainWindow::WindowWrapper::WindowWrapper(MainWindow* pOwner, IWindow* pBase)
{
	m_pOwner = pOwner;
	m_pBase = pBase;
}

// j܂B
MainWindow::WindowWrapper::~WindowWrapper()
{
	m_pOwner = NULL;
	if(m_pBase != NULL) {
		m_pBase->Destroy();
		m_pBase = NULL;
	}
}

// j܂B
void MainWindow::WindowWrapper::Destroy()
{
	delete this;
}

// eEBhEnhݒ肵܂B
HRESULT MainWindow::WindowWrapper::SetParent(HWND hWnd)
{
	if(m_pBase == NULL) {
		return E_UNEXPECTED;
	}
	return m_pBase->SetParent(hWnd);
}

// ʒmIuWFNgݒ肵܂B
HRESULT MainWindow::WindowWrapper::SetNotify(IWindowNotify* pNotify)
{
	if(m_pBase == NULL) {
		return E_UNEXPECTED;
	}
	return m_pBase->SetNotify(pNotify);
}

// EBhE𐶐܂B
HRESULT MainWindow::WindowWrapper::Create(int width, int height)
{
	if(m_pBase == NULL) {
		return E_UNEXPECTED;
	}
	HRESULT hr = m_pBase->Create(width, height);
	if(FAILED(hr)) {
		return hr;
	}
	return m_pOwner->RefreshSize();
}

// C[vJn܂B
HRESULT MainWindow::WindowWrapper::RunMainLoop()
{
	if(m_pBase == NULL) {
		return E_UNEXPECTED;
	}
	return m_pBase->RunMainLoop();
}

// ĕ`悵܂B
HRESULT MainWindow::WindowWrapper::Redraw()
{
	if(m_pBase == NULL) {
		return E_UNEXPECTED;
	}
	return m_pBase->Redraw();
}

// GLp̃tbNǉ܂B
HRESULT MainWindow::WindowWrapper::AddGLHook(IWindowGLHook* pHook)
{
	if(m_pBase == NULL) {
		return E_UNEXPECTED;
	}
	return m_pBase->AddGLHook(pHook);
}

// GLp̃tbN폜܂B
HRESULT MainWindow::WindowWrapper::RemoveGLHook(IWindowGLHook* pHook)
{
	if(m_pBase == NULL) {
		return E_UNEXPECTED;
	}
	return m_pBase->RemoveGLHook(pHook);
}

// EBhE܂B
HRESULT MainWindow::WindowWrapper::Close()
{
	if(m_pBase == NULL) {
		return E_UNEXPECTED;
	}
	return m_pBase->Close();
}

// R\[ɏoȂ܂B
HRESULT MainWindow::WindowWrapper::PrintLine(const char* line)
{
	if(m_pBase == NULL) {
		return E_UNEXPECTED;
	}
	return m_pOwner->m_pDebugEdit->PrintLine(line);
}

// A[g\Ȃ܂B
HRESULT MainWindow::WindowWrapper::Alert(const char* message)
{
	MessageBoxA(m_pOwner->m_hWnd, message, ALERT_CAPTION, MB_OK);
	return S_OK;
}

// Xe[^XbZ[W\Ȃ܂B
HRESULT MainWindow::WindowWrapper::SetStatus(const char* status)
{
	return m_pOwner->m_pStatusWnd->SetText(1, status);
}

// ݂̏|[g܂B
HRESULT MainWindow::WindowWrapper::ReportStatus(ostream* pOut)
{
	if(m_pBase == NULL) {
		return E_UNEXPECTED;
	}
	return m_pBase->ReportStatus(pOut);
}

// A[g\Ȃ܂B
HRESULT MainWindow::WindowWrapper::OnAlert(const char* message)
{
	return Alert(message);
}

// G[\Ȃ܂B
HRESULT MainWindow::WindowWrapper::OnError(const char* message)
{
	string msg = "ERROR: ";
	msg += message;

	return Alert(msg.c_str());
}
		
// R\[ɏoȂ܂B
HRESULT MainWindow::WindowWrapper::OnPrintLine(const char* line)
{
	return PrintLine(line);
}

// `ݒ肵܂B
HRESULT MainWindow::WindowWrapper::SetRect(int x, int y, int width, int height)
{
	if(m_pBase == NULL) {
		return E_UNEXPECTED;
	}
	return m_pBase->SetRect(x, y, width, height);
}

// `擾܂B
HRESULT MainWindow::WindowWrapper::GetRect(int* pX, int* pY, int* pWidth, int* pHeight)
{
	if(m_pBase == NULL) {
		return E_UNEXPECTED;
	}
	return m_pBase->GetRect(pX, pY, pWidth, pHeight);
}

// ǂݍݗpGLReLXgJn܂B
HRESULT MainWindow::WindowWrapper::BeginGLLoading()
{
	if(m_pBase == NULL) {
		return E_UNEXPECTED;
	}
	return m_pBase->BeginGLLoading();
}

// ǂݍݗpGLReLXgI܂B
HRESULT MainWindow::WindowWrapper::EndGLLoading()
{
	if(m_pBase == NULL) {
		return E_UNEXPECTED;
	}
	return m_pBase->EndGLLoading();
}

// ܂B
HRESULT MainWindow::Init(JSEngineFactory* pJSEngineFactory)
{
	// TODO: 
	m_hToolbarBrush = CreateSolidBrush(RGB(0xcc,0xcc,0xcc));
	if(m_hToolbarBrush == NULL) {
		return E_FAIL;
	}
	m_hVideoBrush = CreateSolidBrush(RGB(0x00,0x00,0x00));
	if(m_hVideoBrush == NULL) {
		return E_FAIL;
	}

	// Rg[̍쐬
	m_pStatusWnd = new StatusWnd(m_hWnd);

    INT Size[]={250, -1};
	HRESULT hr = m_pStatusWnd->Create(Size, 2, ID_STATUSBAR);
	if(FAILED(hr)) {
		return hr;
	}

    hr = m_pStatusWnd->SetText(0, "(Path)");
	if(FAILED(hr)) {
		return hr;
	}
    
	hr = m_pStatusWnd->SetText(1, "(Status)");
	if(FAILED(hr)) {
		return hr;
	}

	m_pPathEdit = new TextWnd(m_hWnd);

	RECT pathEditRect = {0, 0, WIDTH_PATH_BUTTON, 10};
	hr = m_pPathEdit->Create(DEFAULT_PATH, & pathEditRect, ID_PATH_EDIT);
	if(FAILED(hr)) {
		return hr;
	}

	m_pPathButton = new ButtonWnd(m_hWnd);
	if(m_pPathButton == NULL) {
		return E_OUTOFMEMORY;
	}

	RECT buttonRect = {0, 0, WIDTH_PATH_BUTTON, 10};
	hr = m_pPathButton->Create("Execute", & buttonRect, ID_PATH_BUTTON);
	if(FAILED(hr)) {
		return hr;
	}

	m_pDebugEdit = new TextWnd(m_hWnd);

	hr = m_pDebugEdit->SetMultiline(true);
	if(FAILED(hr)) {
		return hr;
	}

	RECT debugEditRect = {0, 0, 10, DEFAULT_DEBUG_HEIGHT};
	hr = m_pDebugEdit->Create("", & debugEditRect, ID_DEBUG_EDIT);
	if(FAILED(hr)) {
		return hr;
	}

	hr = RefreshSize();
	if(FAILED(hr)) {
		return hr;
	}

	// JavaScriptGW̐
	IJSEngine* pEngine = NULL;
	hr = pJSEngineFactory->Create(& pEngine);
	if(FAILED(hr)) {
		return hr;
	}

	m_pJSEngine = pEngine;
	InvalidateRect(m_hWnd, NULL, true);

	return S_OK;
}

// ReLXgj܂B
void MainWindow::DestroyContext()
{
	// IuWFNgn
	if(m_pARWindow != NULL) {
		if(m_pJSContext != NULL) {
			m_pJSContext->SetNotify(NULL);
		}
		delete m_pARWindow;
		m_pARWindow = NULL;
	}
	if(m_pJSContext != NULL) {
		m_pJSContext->Destroy();
		m_pJSContext = NULL;
	}
	if(m_pPluginManager != NULL) {
		delete m_pPluginManager;
		m_pPluginManager = NULL;
	}
	m_pGLWindow = NULL;
}

// TCYύX܂B
HRESULT MainWindow::OnSize(int width, int height)
{
	RECT editRect, buttonRect, debugRect;
	HRESULT hr = GetControlRect(width, height, & editRect, & buttonRect, & debugRect);
	if(FAILED(hr)) {
		return hr;
	}

	hr = m_pPathEdit->Move(& editRect, true);
	if(FAILED(hr)) {
		return hr;
	}

	hr = m_pPathButton->Move(& buttonRect, true);
	if(FAILED(hr)) {
		return hr;
	}

	hr = m_pDebugEdit->Move(& debugRect, true);
	if(FAILED(hr)) {
		return hr;
	}

	if(m_pGLWindow == NULL) {
		return S_FALSE;
	}

	RECT statusRect;
	hr = m_pStatusWnd->GetRect(& statusRect);
	if(FAILED(hr)) {
		return hr;
	}

	int statusHeight = statusRect.bottom - statusRect.top;
	int debugHeight = debugRect.bottom - debugRect.top;

	int sx = 0, sy = 0, swidth = 0, sheight = 0;
	hr = m_pGLWindow->GetRect(& sx, & sy, & swidth, & sheight);
	if(FAILED(hr)) {
		return hr;
	}

	int vheight = (height - (editRect.bottom + MARGIN_CONTROLS) - statusHeight - debugHeight);
	int vwidth = width;

	int dx = 0;
	int dy = (editRect.bottom + MARGIN_CONTROLS);
	double aspect = ((double) swidth) / ((double) sheight);
	int dheight = vheight;
	int dwidth = (int)(aspect * vheight);
	if(dwidth > vwidth) {
		dwidth = vwidth;
		dheight = (int)(1.0 / aspect * vwidth);
		dy = (vheight - dheight) / 2;
	}else{
		dx = (vwidth - dwidth) / 2;
	}
	if(sx != dx || sy != dy || swidth != dwidth || sheight != dheight) {
		hr = m_pGLWindow->SetRect(dx, dy, dwidth, dheight);
		if(FAILED(hr)) {
			return hr;
		}
	}


	return S_OK;
}

// ʂ`悵܂B
HRESULT MainWindow::OnPaint(HDC hDC)
{
	RECT screenRect;
	BOOL bret = GetWindowRect(m_hWnd, & screenRect);
	if(bret == false) {
		return E_FAIL;
	}
	screenRect.right -= screenRect.left;
	screenRect.bottom -= screenRect.top;
	screenRect.left = 0;
	screenRect.top = 0;

	RECT buttonRect;
	HRESULT hr = m_pPathButton->GetRect(& buttonRect);
	if(FAILED(hr)) {
		return hr;
	}

	RECT statusRect;
	hr = m_pDebugEdit->GetRect(& statusRect);
	if(FAILED(hr)) {
		return hr;
	}

	POINT buttonPnt;
	buttonPnt.x = buttonRect.left;
	buttonPnt.y = buttonRect.top;
	buttonRect.right -= buttonRect.left;
	buttonRect.bottom -= buttonRect.top;
	bret = ScreenToClient(m_hWnd, & buttonPnt);
	if(bret == false) {
		return E_FAIL;
	}
	buttonRect.left = buttonPnt.x;
	buttonRect.top = buttonPnt.y;
	buttonRect.right += buttonPnt.x;
	buttonRect.bottom += buttonPnt.y;

	POINT statusPnt;
	statusPnt.x = statusRect.left;
	statusPnt.y = statusRect.top;
	statusRect.right -= statusRect.left;
	statusRect.bottom -= statusRect.top;
	bret = ScreenToClient(m_hWnd, & statusPnt);
	if(bret == false) {
		return E_FAIL;
	}
	statusRect.left = statusPnt.x;
	statusRect.top = statusPnt.y;
	statusRect.right += statusPnt.x;
	statusRect.bottom += statusPnt.y;

	RECT rect;
	rect.left = screenRect.left;
	rect.right = screenRect.right;
	rect.top = screenRect.top;
	rect.bottom = buttonRect.bottom + MARGIN_CONTROLS;
	FillRect(hDC, & rect, m_hToolbarBrush);

	rect.top = buttonRect.bottom + MARGIN_CONTROLS;
	rect.bottom = statusRect.top;
	FillRect(hDC, & rect, m_hVideoBrush);

	return S_OK;
}

// TCYXV܂B
HRESULT MainWindow::RefreshSize()
{
	// TCY
	RECT windowRect;
	BOOL bRet = GetWindowRect(m_hWnd, & windowRect);
	if(bRet == FALSE) {
		return E_FAIL;
	}

	int smEdgeX = GetSystemMetrics(SM_CXFRAME)*2;
	int smEdgeY = GetSystemMetrics(SM_CYFRAME)*2;
	int titleY = GetSystemMetrics(SM_CYCAPTION);
	int menuY = GetSystemMetrics(SM_CYMENU);
	return OnSize(windowRect.right - windowRect.left - smEdgeX, windowRect.bottom - windowRect.top - smEdgeY - titleY - menuY);
}

// Rg[̋`擾܂B
HRESULT MainWindow::GetControlRect(int width, int height, RECT* pPathEditRect, RECT* pPathButtonRect, RECT* pDebugEditRect)
{
	if(pPathEditRect == NULL || pPathButtonRect == NULL || pDebugEditRect == NULL) {
		return E_POINTER;
	}

	RECT buttonRect;
	HRESULT hr = m_pPathButton->GetRect(& buttonRect);
	if(FAILED(hr)) {
		return hr;
	}

	RECT statusRect;
	hr = m_pStatusWnd->GetRect(& statusRect);
	if(FAILED(hr)) {
		return hr;
	}

	int statusHeight = statusRect.bottom - statusRect.top;

	pPathButtonRect->right = width - MARGIN_CONTROLS;
	pPathButtonRect->left = pPathButtonRect->right - (buttonRect.right - buttonRect.left);
	pPathButtonRect->bottom = HEIGHT_CONTROLS + MARGIN_CONTROLS;
	pPathButtonRect->top = MARGIN_CONTROLS;

	pPathEditRect->left = MARGIN_CONTROLS;
	pPathEditRect->top = MARGIN_CONTROLS;
	pPathEditRect->bottom = MARGIN_CONTROLS + HEIGHT_CONTROLS;
	pPathEditRect->right = pPathButtonRect->left - MARGIN_CONTROLS;

	pDebugEditRect->left = 0;
	pDebugEditRect->top = height - DEFAULT_DEBUG_HEIGHT - statusHeight;
	if(pDebugEditRect->top < pPathEditRect->bottom + MARGIN_CONTROLS) {
		pDebugEditRect->top = pPathEditRect->bottom + MARGIN_CONTROLS;
	}
	pDebugEditRect->right = width;
	pDebugEditRect->bottom = height - statusHeight;
	if(pDebugEditRect->bottom < pDebugEditRect->top) {
		pDebugEditRect->bottom = pDebugEditRect->top;
	}

	return S_OK;
}

// XNvgs܂B
HRESULT MainWindow::Evaluate(SourceFactory* pSourceFactory, const char* path)
{
	if(path == NULL) {
		return E_POINTER;
	}
	string spath = path;
	bool value = false;
	HRESULT hr = SourceFactory::IsDiscPath(spath.c_str(), & value);
	if(FAILED(hr)) {
		return hr;
	}
	if(value) {
		hr = SourceFactory::EncodeURL(spath.c_str(), spath);
		if(FAILED(hr)) {
			return hr;
		}
		string url = "file:///";
		url += spath;
		spath = url;
	}

	ISource* pSource = NULL;
	hr = pSourceFactory->Load(spath.c_str(), & pSource);
	if(FAILED(hr)) {
		return hr;
	}

	string baseURL;
	hr = pSource->GetLocation(baseURL);
	if(FAILED(hr)) {
		pSource->Destroy();
		pSource = NULL;
		return hr;
	}

	SourceJSCode code(pSource, & hr);
	pSource = NULL;
	if(FAILED(hr)) {
		return hr;
	}

	hr = m_pJSContext->SetBaseURL(baseURL.c_str());
	if(FAILED(hr)) {
		return hr;
	}

	hr = m_pJSContext->Evaluate(& code);
	if(FAILED(hr)) {
		return hr;
	}

	return S_OK;
}

// fobOo͂Ȃ܂B
void MainWindow::Debug(const char* message)
{
	if(message == NULL || m_pDebugEdit == NULL) {
		return;
	}
	string msg = "[DEBUG] ";
	msg += message;

	m_pDebugEdit->PrintLine(msg.c_str());
}

// \z܂B
MainWindow::AbstractWnd::AbstractWnd(HWND hParentWnd)
{
	m_hParentWnd = hParentWnd;
	m_hWnd = NULL;
}

// j܂B
MainWindow::AbstractWnd::~AbstractWnd()
{
	m_hParentWnd = NULL;
	if(m_hWnd != NULL) {
		DestroyWindow(m_hWnd);
		m_hWnd = NULL;
	}
}

// `擾܂B
HRESULT MainWindow::AbstractWnd::GetRect(RECT* pRect)
{
	if(pRect == NULL) {
		return E_POINTER;
	}
	if(m_hWnd == NULL) {
		return E_UNEXPECTED;
	}
	BOOL br = GetWindowRect(m_hWnd, pRect);
	if(br == false) {
		return E_FAIL;
	}
	return S_OK;
}

// ړ܂B
HRESULT MainWindow::AbstractWnd::Move(const RECT* pRect, bool repaint)
{
	if(pRect == NULL) {
		return E_POINTER;
	}
	if(m_hWnd == NULL) {
		return E_UNEXPECTED;
	}
	MoveWindow(m_hWnd, pRect->left, pRect->top, pRect->right - pRect->left, pRect->bottom - pRect->top, repaint);
	return S_OK;
}

// EBhEnh擾܂B
HWND MainWindow::AbstractWnd::GetHWnd()
{
	return m_hWnd;
}

// ftHgtHgݒ肵܂B
HRESULT MainWindow::AbstractWnd::SetDefaultFont()
{
	if(m_hWnd == NULL) {
		return E_UNEXPECTED;
	}
	HDC hDC = GetDC(m_hParentWnd);

	HFONT hFont = CreateFont(MulDiv(9,-::GetDeviceCaps(hDC,LOGPIXELSY),72),
			0,0,0,FW_NORMAL,0,0,0,SHIFTJIS_CHARSET,
			OUT_STROKE_PRECIS,CLIP_STROKE_PRECIS,DRAFT_QUALITY,
			VARIABLE_PITCH | FF_SWISS,TEXT("lr SVbN"));
	SendMessage(m_hWnd,WM_SETFONT,(WPARAM)hFont,MAKELONG(TRUE,0));

	ReleaseDC(m_hParentWnd,hDC);
	return S_OK;
}

// \z܂B
MainWindow::ButtonWnd::ButtonWnd(HWND hParentWnd)
: AbstractWnd(hParentWnd)
{
	;
}

// j܂B
MainWindow::ButtonWnd::~ButtonWnd()
{
	;
}

// 쐬܂B
HRESULT MainWindow::ButtonWnd::Create(const char* name, const RECT* pRect, int id)
{
	if(name == NULL || pRect == NULL) {
		return E_POINTER;
	}
	HINSTANCE hInst = GetModuleHandle(NULL);
	m_hWnd = CreateWindowExA(0, "BUTTON", name, WS_CHILD|WS_VISIBLE|BS_PUSHBUTTON, 
		                           pRect->left, pRect->top, pRect->right - pRect->left, pRect->bottom - pRect->top,
								   m_hParentWnd,
								   (HMENU)id,
                                   hInst,
								   NULL );
	if(m_hWnd == NULL) {
		return E_FAIL;
	}
	return SetDefaultFont();
}

// \z܂B
MainWindow::TextWnd::TextWnd(HWND hParentWnd)
: AbstractWnd(hParentWnd)
{
	m_bMultiline = false;
	m_iLines = 0;
}

// j܂B
MainWindow::TextWnd::~TextWnd()
{
	;
}

// sǂݒ肵܂B
HRESULT MainWindow::TextWnd::SetMultiline(bool multiline)
{
	if(m_hWnd != NULL) {
		return E_UNEXPECTED;
	}
	m_bMultiline = multiline;
	return S_OK;
}

// 쐬܂B
HRESULT MainWindow::TextWnd::Create(const char* defaultValue, const RECT* pRect, int id)
{
	if(defaultValue == NULL || pRect == NULL) {
		return E_POINTER;
	}
	DWORD style = 0;
	if(m_bMultiline == true) {
		style |= ES_MULTILINE|WS_HSCROLL|WS_VSCROLL;
	}

	HINSTANCE hInst = GetModuleHandle(NULL);
	m_hWnd = CreateWindowExA(WS_EX_CLIENTEDGE, "EDIT", defaultValue, style|ES_LEFT|WS_CHILD|WS_VISIBLE|WS_BORDER, 
		                           pRect->left, pRect->top, pRect->right - pRect->left, pRect->bottom - pRect->top,
								   m_hParentWnd,
								   (HMENU)id,
                                   hInst,
								   NULL );
	if(m_hWnd == NULL) {
		return E_FAIL;
	}
	return SetDefaultFont();
}

// eLXg擾܂B
HRESULT MainWindow::TextWnd::GetText(string& text)
{
	if(m_hWnd == NULL) {
		return E_UNEXPECTED;
	}
	int len = GetWindowTextLengthA(m_hWnd);
	if(len <= 0) {
		return S_FALSE;
	}

	char* pch = new char[len + 1];
	if(pch == NULL) {
		return E_OUTOFMEMORY;
	}
	ZeroMemory(pch, sizeof(char) * (len + 1));

	len = GetWindowTextA(m_hWnd, pch, len + 1);
	if(len <= 0) {
		delete pch;
		pch = NULL;
		return E_FAIL;
	}
	
	text = pch;
	delete pch;
	pch = NULL;
	return S_OK;
}

// eLXgݒ肵܂B
HRESULT MainWindow::TextWnd::SetText(const char* message)
{
	if(message == NULL) {
		return E_POINTER;
	}
	if(m_hWnd == NULL) {
		return E_UNEXPECTED;
	}

	BOOL br = SetWindowTextA(m_hWnd, message);
	if(br == false) {
		return E_FAIL;
	}

	return S_OK;
}

// so͂܂B
HRESULT MainWindow::TextWnd::PrintLine(const char* message)
{
	string current;
	HRESULT hr = GetText(current);
	if(FAILED(hr)) {
		return hr;
	}

	m_iLines ++;
	current += message;
	current += "\r\n";

	hr = SetText(current.c_str());
	if(FAILED(hr)) {
		return hr;
	}

	SendMessage(m_hWnd, EM_LINESCROLL, 0, m_iLines);

	return S_OK;
}

// \z܂B
MainWindow::StatusWnd::StatusWnd(HWND hParentWnd)
: AbstractWnd(hParentWnd)
{
}

// j܂B
MainWindow::StatusWnd::~StatusWnd()
{
}

// 쐬܂B
HRESULT MainWindow::StatusWnd::Create(int size[], int count, int id)
{
	m_hWnd = CreateStatusWindowA(WS_CHILD|WS_VISIBLE, "Test", m_hParentWnd, ID_STATUSBAR);
	if(m_hWnd == NULL) {
		return E_FAIL;
	}
    SendMessage(m_hWnd, SB_SETPARTS, count, (LPARAM)(LPINT)size);
	return S_OK;
}

// Xe[^Xݒ肵܂B
HRESULT MainWindow::StatusWnd::SetText(int index, const char* message)
{
	if(index < 0 || index > 1) {
		return E_INVALIDARG;
	}
	if(message == NULL) {
		return E_POINTER;
	}

	size_t mbsLen = strlen(message);
	size_t statusTextLen = mbsLen * 2 + 1;
	wchar_t* statusText = new wchar_t[statusTextLen];
	errno_t err = mbstowcs_s(& statusTextLen, statusText, statusTextLen, message, mbsLen);
	if(err != 0) {
		delete statusText;
		statusText = NULL;
		return E_FAIL;
	}

    SendMessage(m_hWnd, SB_SETTEXT, (WPARAM)index|0, (LPARAM)statusText);

	delete statusText;
	statusText = NULL;
	return S_OK;
}

