#include "StdAfx.h"
#include "ITunesSyncClock.h"
#include <mmsystem.h>

// [J[̋NԊu`܂B
#define TIME_RESOL 100

// \z܂B
ITunesSyncClock::ITunesSyncClock(IARGPluginEnvironment* pEnv, IiTunes* pIiTunes, HRESULT* phr)
{
	m_pWorker = NULL;
	m_pEnv = pEnv;
	pEnv->AddRef();

	m_pWorker = new ClockWorker();

	m_pTypeInfo = new ARGTypeInfo(IID_IARGObject);

	HRESULT hr = pEnv->InvokeWorker(m_pWorker);
	if(FAILED(hr)) {
		*phr = hr;
		return;
	}
}

// j܂B
ITunesSyncClock::~ITunesSyncClock()
{
	if(m_pWorker != NULL) {
		HANDLE hExitEvent = NULL;
		m_pWorker->Stop(& hExitEvent);
		m_pWorker->Release();
		m_pWorker = NULL;

		ARGBaseWorker::WaitForDestroy(hExitEvent);
		hExitEvent = NULL;
	}
	if(m_pEnv != NULL) {
		m_pEnv->Release();
		m_pEnv = NULL;
	}
}

// ݎ~bŎ擾܂B
STDMETHODIMP ITunesSyncClock::GetCurrentTimeMS(LONGLONG* pMilliseconds)
{
	if(pMilliseconds == NULL) {
		return E_POINTER;
	}
	if(m_pWorker == NULL) {
		return E_UNEXPECTED;
	}
	return m_pWorker->GetCurrentTimeMS(pMilliseconds);
}

// Đǂ𔻒肵܂B
STDMETHODIMP ITunesSyncClock::IsPlaying(bool* pValue)
{
	if(pValue == NULL) {
		return E_POINTER;
	}
	if(m_pWorker == NULL) {
		return E_UNEXPECTED;
	}
	return m_pWorker->IsPlaying(pValue);
}

// IDispatch̎B
STDMETHODIMP ITunesSyncClock::Invoke(DISPID dispIdMember, REFIID riid, LCID  lcid, WORD  wFlags, DISPPARAMS *pDispParams, VARIANT *pVarResult, EXCEPINFO *pExcepInfo, unsigned int *puArgErr)
{
	// Ȃ
	return S_OK;
}

// \z܂B
ITunesSyncClock::ClockWorker::ClockWorker()
{
	m_bPlaying = false;
	m_baseSec = -1;
	m_baseTickCount = 0;
}

// j܂B
ITunesSyncClock::ClockWorker::~ClockWorker()
{
	;
}

// s܂B
STDMETHODIMP ITunesSyncClock::ClockWorker::Invoke()
{
	// iTunesIuWFNg̎擾
	IiTunes* pITunes = NULL;

	HRESULT hr = CoCreateInstance(CLSID_iTunesApp, NULL, CLSCTX_LOCAL_SERVER, IID_IiTunes, (LPVOID*)& pITunes);
	if(FAILED(hr)) {
		return hr;
	}
	int stateCount = 0;
	int positionCount = 0;

	while(IsContinue()) {
		bool bPlaying = m_bPlaying;
		if(bPlaying == false || stateCount >= 20) {
			ITPlayerState state;
			Sleep(TIME_RESOL);
			hr = pITunes->get_PlayerState(& state);
			if(FAILED(hr)) {
				// TODO: xH
				continue;
			}
			if(state == ITPlayerStatePlaying) {
				bPlaying = true;
			}else{
				bPlaying = false;
			}
			stateCount = 0;
		}else{
			stateCount ++;
		}

		long pos = -1L;
		if(bPlaying && (positionCount >= 0 || m_baseSec < 0)) {
			hr = pITunes->get_PlayerPosition(& pos);
			if(FAILED(hr)) {
				// TODO: xH
				continue;
			}
			positionCount = 0;
		}else if(bPlaying){
			positionCount ++;
		}

		ARGCritSection cs(& m_mutex);
		DWORD tickCount = timeGetTime();
		if(bPlaying) {
			m_bPlaying = true;
			if(pos != m_baseSec && pos >= 0L) {
				m_baseSec = pos;
				m_baseTickCount = tickCount;
			}
		}else{
			m_bPlaying = false;
			m_baseSec = -1;
			m_baseTickCount = 0;
		}
	}

	pITunes->Release();
	pITunes = NULL;

	return S_OK;
}

// ݎ擾܂B
HRESULT ITunesSyncClock::ClockWorker::GetCurrentTimeMS(LONGLONG* pMillisec)
{
	if(pMillisec == NULL) {
		return E_POINTER;
	}
	ARGCritSection cs(& m_mutex);
	if(m_baseSec < 0) {
		*pMillisec = 0;
		return S_OK;
	}
	DWORD tickCount = timeGetTime();
	DWORD procCount = tickCount - m_baseTickCount;
	if(procCount < 0) {
		procCount = 0;
	}
	*pMillisec = m_baseSec * 1000 + procCount;
	return S_OK;
}

// ݂̍ĐԂ擾܂B
HRESULT ITunesSyncClock::ClockWorker::IsPlaying(bool* pValue)
{
	if(pValue == NULL) {
		return E_POINTER;
	}
	ARGCritSection cs(& m_mutex);
	*pValue = m_bPlaying;
	return S_OK;
}
