#include "StdAfx.h"
#include "MQOObjectFactory.h"
#include <plugin/ARGUtil.h>
#include "MQOObject.h"

// \z܂B
MQOObjectFactory::MQOObjectFactory(IARGPluginEnvironment* pEnv)
{
	m_pEnv = pEnv;
	pEnv->AddRef();
}

// j܂B
MQOObjectFactory::~MQOObjectFactory(void)
{
	if(m_pEnv != NULL) {
		m_pEnv->Release();
		m_pEnv = NULL;
	}
}

// IuWFNgOpenGLgp邩ǂ擾܂B
STDMETHODIMP MQOObjectFactory::UseGL(bool* pValue)
{
	if(pValue == NULL) {
		return E_POINTER;
	}
	*pValue = true;
	return S_OK;
};

// p[^^Cv̌␔擾܂B
STDMETHODIMP MQOObjectFactory::GetParameterTypeCandidateCount(int* pCount)
{
	if(pCount == NULL) {
		return E_POINTER;
	}
	*pCount = 1;
	return S_OK;
}

// p[^^Cv̌₲Ƃ̈̐擾܂B
STDMETHODIMP MQOObjectFactory::GetParameterTypeCount(int candIndex, int* pCount)
{
	if(pCount == NULL) {
		return E_POINTER;
	}
	if(candIndex == 0) {
		*pCount = 2;
		return S_OK;
	}else{
		return E_INVALIDARG;
	}
}

// p[^^Cv̌₲Ƃ̈^Cv擾܂B
STDMETHODIMP MQOObjectFactory::GetParameterType(int candIndex, int index, VARTYPE* pVarType)
{
	if(pVarType == NULL) {
		return E_POINTER;
	}
	if(candIndex == 0) {
		if(index == 0) {
			*pVarType = VT_BSTR;
			return S_OK;
		}else if(index == 1) {
			*pVarType = VT_R8;
			return S_OK;
		}else{
			return E_INVALIDARG;
		}
	}else{
		return E_INVALIDARG;
	}
}

// IuWFNg𐶐܂B
STDMETHODIMP MQOObjectFactory::Create(VARIANT* pVars, int varCount, IARGObject** ppObject)
{
	if(pVars == NULL || ppObject == NULL) {
		return E_POINTER;
	}
	if(varCount != 2) {
		return E_INVALIDARG;
	}
	VARIANT* pVPath = pVars + 0;
	if(pVPath->vt != VT_BSTR || pVPath->bstrVal == NULL) {
		return E_INVALIDARG;
	}
	string path;
	HRESULT hr = ARGUtil::GetStringFromBSTR(path, pVPath->bstrVal);
	if(FAILED(hr)) {
		return hr;
	}

	VARIANT* pVScale = pVars + 1;
	if(pVScale->vt != VT_R8) {
		return E_INVALIDARG;
	}
	double scale = pVScale->dblVal;

	int maxLen = 0;
	hr = m_pEnv->GetRealPath(path.c_str(), NULL, 0, & maxLen);
	if(FAILED(hr)) {
		return hr;
	}

	// I[m
	char* pAbsolutePath = new char[maxLen + 1];
	ZeroMemory(pAbsolutePath, sizeof(char) * (maxLen + 1));

	hr = m_pEnv->GetRealPath(path.c_str(), pAbsolutePath, maxLen + 1, & maxLen);
	if(FAILED(hr)) {
		delete[] pAbsolutePath;
		pAbsolutePath = NULL;
		return hr;
	}
	pAbsolutePath[maxLen] = '\0';

	MQOObject* pObject = new MQOObject(m_pEnv);
	hr = pObject->Load(pAbsolutePath, scale);
	delete[] pAbsolutePath;
	pAbsolutePath = NULL;
	if(FAILED(hr)) {
		delete pObject;
		return hr;
	}

	*ppObject = pObject;
	return S_OK;
}
