#include "StdAfx.h"
#include "ARGraphics.h"
#include <AR/ar.h>

// \z܂B
ARGraphics::ARGraphics(ARWindow* pOwner)
{
	m_pOwner = pOwner;
	m_visible = false;
	m_pProp = NULL;
	m_pChildList = NULL;
}

// j܂B
ARGraphics::~ARGraphics(void)
{
	for(vector<ARGraphics*>::iterator it = m_children.begin();
		it != m_children.end();
		it ++) {
		ARGraphics* pGraphics = *it;
		pGraphics->Destroy();
		pGraphics = NULL;
	}
	m_children.clear();
	if(m_pProp != NULL) {
		m_pProp->Destroy();
		m_pProp = NULL;
	}
	if(m_pChildList != NULL) {
		delete m_pChildList;
		m_pChildList = NULL;
	}
	m_pOwner = NULL;
}

// qǉ܂B
HRESULT ARGraphics::AddChild(ARGraphics* pGraphics, int* pIndex)
{
	if(pGraphics == NULL || pIndex == NULL) {
		return E_POINTER;
	}
	*pIndex = m_children.size();
	m_children.push_back(pGraphics);

	return S_OK;
}

// q폜܂B
HRESULT ARGraphics::RemoveChild(int index)
{
	int count = 0;
	for(vector<ARGraphics*>::iterator it = m_children.begin();
		it != m_children.end();
		it ++, count ++) {
		if(count == index) {
			m_children.erase(it);
			return S_OK;
		}
	}
	return S_FALSE;
}

// qXg擾܂B
HRESULT ARGraphics::GetChildren(vector<ARGraphics*>& children)
{
	for(vector<ARGraphics*>::iterator it = m_children.begin();
		it != m_children.end();
		it ++) {
			children.push_back(*it);
	}
	return S_OK;
}

// qXg擾܂B
HRESULT ARGraphics::GetChildren(IArrayProperty** ppProp)
{
	if(ppProp == NULL) {
		return E_POINTER;
	}
	if(m_pChildList == NULL) {
		m_pChildList = new ChildList(this);
	}

	*ppProp = m_pChildList;
	return S_OK;
}

// \ׂǂ擾܂B
HRESULT ARGraphics::IsVisible(bool* pVisible)
{
	if(pVisible == NULL) {
		return E_POINTER;
	}
	*pVisible = m_visible;
	return S_OK;
}

// \ׂǂݒ肵܂B
HRESULT ARGraphics::SetVisible(bool visible)
{
	m_visible = visible;
	return S_OK;
}

// vpeB擾܂B
HRESULT ARGraphics::GetProperty(INamedProperty** ppProp)
{
	if(ppProp == NULL) {
		return E_POINTER;
	}
	if(m_pProp != NULL) {
		*ppProp = m_pProp;
		return S_OK;
	}

	HRESULT hr = CreateProperty(& m_pProp);
	if(FAILED(hr)) {
		return hr;
	}

	*ppProp = m_pProp;
	return S_OK;
}

// q`悵܂B
HRESULT ARGraphics::DrawChildren()
{
	if(m_visible == false) {
		return S_FALSE;
	}
	for(vector<ARGraphics*>::iterator it = m_children.begin();
		it != m_children.end();
		it ++) {
		ARGraphics* pGraphics = *it;
		HRESULT hr = pGraphics->Draw();
		if(FAILED(hr)) {
			return hr;
		}
	}

	return S_OK;
}

// qɑ΂ăIuWFNgt@Ng̒ǉʒm܂B
HRESULT ARGraphics::NotifyObjectFactoryAddedForChildren(const char* name, IARGObjectFactory* pFactory)
{
	if(name == NULL || pFactory == NULL) {
		return E_POINTER;
	}
	for(vector<ARGraphics*>::iterator it = m_children.begin();
		it != m_children.end();
		it ++) {
		ARGraphics* pGraphics = *it;
		HRESULT hr = pGraphics->NotifyObjectFactoryAdded(name, pFactory);
		if(FAILED(hr)) {
			return hr;
		}
	}

	return S_OK;
}

// ACh̏sȂ܂B
HRESULT ARGraphics::OnIdleForChildren()
{
	for(vector<ARGraphics*>::iterator it = m_children.begin();
		it != m_children.end();
		it ++) {
		ARGraphics* pGraphics = *it;
		HRESULT hr = pGraphics->OnIdle();
		if(FAILED(hr)) {
			return hr;
		}
	}
	return S_OK;
}

// I[i[EBhE擾܂B
HRESULT ARGraphics::GetOwner(ARWindow** ppOwner)
{
	if(ppOwner == NULL) {
		return E_POINTER;
	}
	*ppOwner = m_pOwner;
	return S_OK;
}

// IuWFNgt@Ng̒ǉʒm܂B
HRESULT ARGraphics::NotifyObjectFactoryAdded(const char* name, IARGObjectFactory* pFactory)
{
	if(name == NULL || pFactory == NULL) {
		return E_POINTER;
	}
	return NotifyObjectFactoryAddedForChildren(name, pFactory);
}

// ACh̏sȂ܂B
HRESULT ARGraphics::OnIdle()
{
	return OnIdleForChildren();
}

// vpeB𐶐܂B
HRESULT ARGraphics::CreateProperty(INamedProperty** ppProp)
{
	if(ppProp == NULL) {
		return E_POINTER;
	}
	GraphicsPropertyImpl* pNewProp = new GraphicsPropertyImpl(this);
	if(pNewProp == NULL) {
		return E_OUTOFMEMORY;
	}
	HRESULT hr = pNewProp->Init();
	if(FAILED(hr)) {
		delete pNewProp;
		pNewProp = NULL;
		return hr;
	}
	*ppProp = pNewProp;
	return S_OK;
}

// \z܂B
ARGraphics::GraphicsPropertyImpl::GraphicsPropertyImpl(ARGraphics* pOwner)
{
	m_pOwner = pOwner;
}

// j܂B
ARGraphics::GraphicsPropertyImpl::~GraphicsPropertyImpl()
{
	m_pOwner = NULL;
}

// j܂B
void ARGraphics::GraphicsPropertyImpl::Destroy()
{
	delete this;
}

// o܂B
HRESULT ARGraphics::GraphicsPropertyImpl::Init()
{
	HRESULT hr = S_OK;

	hr = DefineProperty("visible", PROP_BOOL);
	if(FAILED(hr)) {
		return hr;
	}

	return S_OK;
}

// bool̃vpeB擾܂B
HRESULT ARGraphics::GraphicsPropertyImpl::GetBoolProperty(const char* name, bool* pValue)
{
	if(name == NULL || pValue == NULL) {
		return E_POINTER;
	}
	string sname = name;
	if(sname == "visible") {
		return m_pOwner->IsVisible(pValue);
	}else{
		return AbstractNamedProperty::GetBoolProperty(name, pValue);
	}
}

// bool̃vpeBݒ肵܂B
HRESULT ARGraphics::GraphicsPropertyImpl::SetBoolProperty(const char* name, bool value)
{
	if(name == NULL) {
		return E_POINTER;
	}
	string sname = name;
	if(sname == "visible") {
		return m_pOwner->SetVisible(value);
	}else{
		return AbstractNamedProperty::SetBoolProperty(name, value);
	}
}

// \z܂B
ARGraphics::ChildList::ChildList(ARGraphics* pOwner)
{
	m_pOwner = pOwner;
}

// j܂B
ARGraphics::ChildList::~ChildList()
{
	m_pOwner = NULL;
}

// j܂B
void ARGraphics::ChildList::Destroy()
{
	delete this;
}

// vpeB擾܂B
HRESULT ARGraphics::ChildList::GetPropertyCount(int* pCount)
{
	if(pCount == NULL) {
		return E_POINTER;
	}
	vector<ARGraphics*> graphics;
	HRESULT hr = m_pOwner->GetChildren(graphics);
	if(FAILED(hr)) {
		return hr;
	}
	*pCount = (int)graphics.size();
	return S_OK;
}
	
// vpeB擾܂B
HRESULT ARGraphics::ChildList::GetPropertyDesc(int index, PROPERTY_DESC* pDesc)
{
	if(pDesc == NULL) {
		return E_POINTER;
	}
	pDesc->type = PROP_JSOBJECT;
	return S_OK;
}

// IJSObject̃vpeB擾܂B
HRESULT ARGraphics::ChildList::GetJSObject(int index, IJSObject** ppValue)
{
	if(ppValue == NULL) {
		return E_POINTER;
	}
	vector<ARGraphics*> graphics;
	HRESULT hr = m_pOwner->GetChildren(graphics);
	if(FAILED(hr)) {
		return hr;
	}

	*ppValue = graphics[index]->GetJSObject();
	return S_OK;
}
