#include "StdAfx.h"
#include "ARWindow.h"
#include <GL/gl.h>
//#include <GL/glut.h>
#include <AR/gsub.h>
#include <AR/param.h>
#include <AR/ar.h>
#include <AR/config.h>
#include <AR/gsub_lite.h>
#include <AR/video.h>
#include "ARPattern.h"
#include "../js/IJSContext.h"
#include "js/ARJSWindow.h"
#include "../js/JSEngineContext.h"
#include "../image/PngImageLoader.h"
#include "../image/JpgImageLoader.h"
#include "../common/URLPath.h"

#define VIEW_SCALEFACTOR		1.0 //0.025		// 1.0 ARToolKit unit becomes 0.025 of my OpenGL units.
#define VIEW_SCALEFACTOR_1		1.0			// 1.0 ARToolKit unit becomes 1.0 of my OpenGL units.

// \z܂B
ARWindow::ARWindow(IWindow* pWindow, ICaptureDevice* pCaptureDevice, HRESULT* phr)
{
	m_pJSObject = NULL;
	m_iCount = 0;
	m_pWindow = NULL;
	m_pCaptureDevice = NULL;
	m_pProp = NULL;

	*phr = S_OK;
	if(pCaptureDevice == NULL || pWindow == NULL) {
		*phr = E_POINTER;
		return;
	}

	HRESULT hr = InitImageLoaders();
	if(FAILED(hr)) {
		*phr = hr;
		return;
	}

	hr = m_rootGraphics.SetVisible(true);
	if(FAILED(hr)) {
		*phr = hr;
		return;
	}

	m_pWindow = pWindow;
	m_pCaptureDevice = pCaptureDevice;
	m_pContext = NULL;

	m_pWindow->SetNotify(this);
}

// j܂B
ARWindow::~ARWindow(void)
{
	m_pContext = NULL;
	if(m_pCaptureDevice != NULL) {
		m_pCaptureDevice->Destroy();
		m_pCaptureDevice = NULL;
	}
	if(m_pWindow != NULL) {
		m_pWindow->Destroy();
		m_pWindow = NULL;
	}
	if(m_pProp != NULL) {
		delete m_pProp;
		m_pProp = NULL;
	}

	for(vector<pair<string, IARGObjectFactory*>>::iterator it = m_objectFactories.begin(); it != m_objectFactories.end(); it ++) {
		IARGObjectFactory* pFactory = it->second;
		pFactory->Release();
		pFactory = NULL;
	}
	m_objectFactories.clear();

	CoUninitialize();
    argCleanup();

	DestroyPatterns();
	m_pJSObject = NULL;
	for(map<string,IImageLoader*>::iterator it = m_imageLoaders.begin(); it != m_imageLoaders.end(); it ++) {
		IImageLoader* pImageLoader = it->second;
		pImageLoader->Destroy();
	}
	m_imageLoaders.clear();
}

// rfI܂B
HRESULT ARWindow::InitVideo(const char* config)
{
	if(m_pCaptureDevice == NULL || m_pContext == NULL) {
		return E_UNEXPECTED;
	}

	string url;
	HRESULT hr = m_pContext->GetAbsoluteURL(config, url);
	if(FAILED(hr)) {
		return hr;
	}

	URLPath urlPath(url.c_str(), & hr);
	if(FAILED(hr)) {
		return hr;
	}
	string realFile;
	hr = urlPath.GetRealPath(realFile);
	if(FAILED(hr)) {
		return hr;
	}
	return m_pCaptureDevice->Init(realFile.c_str());
}

// EBhE܂B
HRESULT ARWindow::InitWindow(const char* config)
{
	if(m_pCaptureDevice == NULL) {
		return E_UNEXPECTED;
	}
	string url;
	HRESULT hr = m_pContext->GetAbsoluteURL(config, url);
	if(FAILED(hr)) {
		return hr;
	}

	URLPath urlPath(url.c_str(), & hr);
	if(FAILED(hr)) {
		return hr;
	}
	string realFile;
	hr = urlPath.GetRealPath(realFile);
	if(FAILED(hr)) {
		return hr;
	}
	return m_pCaptureDevice->InitCamera(realFile.c_str());
}

// EBhEJn܂B
HRESULT ARWindow::Start()
{
	if(m_pCaptureDevice == NULL) {
		return E_UNEXPECTED;
	}

	int width = 0, height = 0;
	HRESULT hr = m_pCaptureDevice->GetSize(& width, & height);
	if(FAILED(hr)) {
		return hr;
	}

	return m_pWindow->Create(width, height);
}

// EBhE~܂B
HRESULT ARWindow::Stop()
{
	if(m_pCaptureDevice == NULL) {
		return E_UNEXPECTED;
	}

	// ~邾
	m_pCaptureDevice->Stop();

    argCleanup();
	return S_OK;
}

// p^[ǉ܂B
HRESULT ARWindow::AddPattern(ARPattern* pPattern)
{
	if(pPattern == NULL) {
		return E_POINTER;
	}
	m_patterns.push_back(pPattern);
	return S_OK;
}

// OtBbNǉ܂B
HRESULT ARWindow::AddGraphics(ARGraphics* pGraphics, int* pIndex)
{
	return m_rootGraphics.AddChild(pGraphics, pIndex);
}

// OtBbN폜܂B
HRESULT ARWindow::RemoveGraphics(int index)
{
	return m_rootGraphics.RemoveChild(index);
}

// C[vȂ܂B
HRESULT ARWindow::RunMainLoop()
{
	if(m_pWindow == NULL) {
		return E_UNEXPECTED;
	}
	return m_pWindow->RunMainLoop();
}

// JavaScriptReLXgݒ肵܂B
HRESULT ARWindow::Attach(IJSContext* pContext)
{
	if(pContext == NULL) {
		return E_POINTER;
	}
	if(m_pProp == NULL) {
		HRESULT hr = S_OK;
		m_pProp = new PropertyImpl(this, & hr);
		if(m_pProp == NULL) {
			return E_OUTOFMEMORY;
		}
		if(FAILED(hr)) {
			return hr;
		}
	}

	m_pContext = pContext;
	m_pJSObject = new ARJSWindow(this, m_pProp);
	if(m_pJSObject == NULL) {
		return E_OUTOFMEMORY;
	}

	JSEngineContext* pContextImpl = (JSEngineContext*) pContext;
	HRESULT hr = m_pJSObject->Attach(pContextImpl, NULL);
	if(FAILED(hr)) {
		return hr;
	}
	return pContextImpl->DefineGlobalProperty("window", m_pJSObject);
}

// EBhE擾܂B
HRESULT ARWindow::GetWindow(IWindow** ppWindow)
{
	if(ppWindow == NULL) {
		return E_POINTER;
	}
	*ppWindow = m_pWindow;
	return S_OK;
}

// EBhEB
HRESULT ARWindow::OnCreate()
{
	// Lv`̊Jn
	HRESULT hr = m_pCaptureDevice->Start();
	if(FAILED(hr)) {
		return hr;
	}

	hr = ReportStatus(& cout);
	if(FAILED(hr)) {
		return hr;
	}
	arUtilTimerReset();

	return S_OK;
}

// IdleB
HRESULT ARWindow::OnIdle()
{
	if(m_pCaptureDevice == NULL || m_pWindow == NULL) {
		return E_UNEXPECTED;
	}

	HRESULT hr = m_rootGraphics.OnIdle();
	if(FAILED(hr)) {
		return hr;
	}

	ARUint8* pBuf = NULL;
	m_iCount ++;

	hr = m_pCaptureDevice->GetImage(& pBuf);
	if(hr == S_FALSE) {
		return hr;
	}
	if(FAILED(hr)) {
		return hr;
	}

    /* detect the markers in the video frame */
    ARMarkerInfo    *marker_info = NULL;
    int             marker_num = 0;
	int             thresh = 100;

    if( arDetectMarker(pBuf, thresh, &marker_info, &marker_num) < 0 ) {
		return E_FAIL;
    }

	ARMarkerInfoContainer markerInfoC;
	markerInfoC.pCore = marker_info;
	for(vector<ARPattern*>::iterator it = m_patterns.begin(); it != m_patterns.end(); it ++) {
		ARPattern* pPattern = *it;
		// p^[̏
		HRESULT hr = pPattern->Process(& markerInfoC, marker_num);

		if(FAILED(hr)) {
			return hr;
		}
	}

	// \񂳂Ă񓯊̎s
	if(m_pContext != NULL) {
		hr = m_pContext->ProcessAsync();
		if(FAILED(hr)) {
			return hr;
		}
	}

	// ĕ`
	m_pWindow->Redraw();
	return S_OK;
}

// `揈B
HRESULT ARWindow::OnDraw()
{
	// Select correct buffer for this context.
	glDrawBuffer(GL_BACK);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); // Clear the buffers for new frame.
	
	HRESULT hr = m_pCaptureDevice->Draw();
	if(FAILED(hr)) {
		return hr;
	}

	hr = m_pCaptureDevice->CaptureNext();
	if(FAILED(hr)) {
		return hr;
	}
			
	GLdouble p[16];
	hr = m_pCaptureDevice->GetProjectionMatrix(p);
	if(FAILED(hr)) {
		return hr;
	}

	glMatrixMode(GL_PROJECTION);
	glLoadMatrixd(p);
	glMatrixMode(GL_MODELVIEW);
	
	// Viewing transformation.
	glLoadIdentity();
	// Lighting and geometry that moves with the camera should go here.
	// (I.e. must be specified before viewing transformations.)
	//none
	
	// All other lighting and geometry goes here.
	// Calculate the camera position for each object and draw it.
	/*for (int i = 0; i < gObjectDataCount; i++) {
		if ((gObjectData[i].visible != 0) && (gObjectData[i].vrml_id >= 0)) {
			//fprintf(stderr, "About to draw object %i\n", i);
			GLdouble m[16];
			arglCameraViewRH(gObjectData[i].trans, m, VIEW_SCALEFACTOR_4);
			glLoadMatrixd(m);
		}			
	}*/

    /*argDrawMode3D();
    argDraw3dCamera( 0, 0 );
    glClearDepth( 1.0 );
    glClear(GL_DEPTH_BUFFER_BIT);*/

	hr = m_rootGraphics.Draw();
	
	// Any 2D overlays go here.
	//none

	return hr;
}

// L[Cxg̏B
HRESULT ARWindow::OnKeyEvent(unsigned char key, int x, int y)
{
	bool preventDefault = false;
	if(m_pJSObject != NULL) {
		HRESULT hr = m_pJSObject->OnKeyPress(key, x, y, & preventDefault);
		if(FAILED(hr)) {
			return hr;
		}
	}
	if(preventDefault) {
		return S_OK;
	}
    if( key == 0x1b ) {
        printf("*** %f (frame/sec)\n", (double)m_iCount/arUtilTimer());
		Stop();
		//delete this;
		exit(0);
    }
	return S_OK;
}

// G[B
HRESULT ARWindow::OnError(HRESULT hr)
{
	// TODO: 
	printf("FATAL(Error): %x\n", hr);
	return S_OK;
}

// Xe[^X擾܂B
HRESULT ARWindow::GetStatus(string& status)
{
	status = m_status;

	return S_OK;
}

// Xe[^Xݒ肵܂B
HRESULT ARWindow::SetStatus(const char* status)
{
	if(status == NULL) {
		return E_POINTER;
	}
	if(m_pWindow == NULL) {
		return E_UNEXPECTED;
	}

	m_status = status;
	return m_pWindow->SetStatus(m_status.c_str());
}

// ImageLoaderݒ肵܂B
HRESULT ARWindow::AddImageLoader(const char* name, IImageLoader* pImageLoader)
{
	if(pImageLoader == NULL || name == NULL) {
		return E_POINTER;
	}
	string sname = name;
	m_imageLoaders[sname] = pImageLoader;
	return S_OK;
}

// ImageLoader擾܂B
HRESULT ARWindow::GetImageLoader(const char* name, IImageLoader** ppImageLoader)
{
	if(ppImageLoader == NULL || name == NULL) {
		return E_POINTER;
	}
	string sname = name;

	IImageLoader* pImageLoader = m_imageLoaders[sname];
	if(pImageLoader == NULL) {
		return E_INVALIDARG;
	}
	*ppImageLoader = pImageLoader;
	return S_OK;
}

// p^[̃Xg擾܂B
HRESULT ARWindow::GetPatterns(vector<ARPattern*>& pattern)
{
	for(vector<ARPattern*>::iterator it = m_patterns.begin();
		it != m_patterns.end();
		it ++) {
			pattern.push_back(*it);
	}
	return S_OK;
}

// OtBbÑXg擾܂B
HRESULT ARWindow::GetWorlds(IArrayProperty** ppProp)
{
	return m_rootGraphics.GetChildren(ppProp);
}

// IuWFNgt@Ngǉ܂B
HRESULT ARWindow::AddObjectFactory(const char* name, IARGObjectFactory* pFactory)
{
	if(name == NULL || pFactory == NULL) {
		return E_POINTER;
	}
	pFactory->AddRef();
	string sname = name;
	pair<string, IARGObjectFactory*> p(sname, pFactory);
	m_objectFactories.push_back(p);

	HRESULT hr = m_rootGraphics.NotifyObjectFactoryAdded(name, pFactory);
	if(FAILED(hr)) {
		return hr;
	}

	return S_OK;
}

// IuWFNgt@Ng̈ꗗ擾܂B
HRESULT ARWindow::GetObjectFactories(vector<pair<string, IARGObjectFactory*>>& factories)
{
	factories = m_objectFactories;
	return S_OK;
}

// OpenGL̓ǂݍݗpReLXgJn܂B
HRESULT ARWindow::BeginGLLoading()
{
	if(m_pWindow == NULL) {
		return E_UNEXPECTED;
	}
	return m_pWindow->BeginGLLoading();
}

// OpenGL̓ǂݍݗpReLXgI܂B
HRESULT ARWindow::EndGLLoading()
{
	if(m_pWindow == NULL) {
		return E_UNEXPECTED;
	}
	return m_pWindow->EndGLLoading();
}

// 摜[_܂B
HRESULT ARWindow::InitImageLoaders()
{
	HRESULT hr = S_OK;
	hr = AddImageLoader("png", new PngImageLoader());
	if(FAILED(hr)) {
		return hr;
	}

	hr = AddImageLoader("jpeg", new JpgImageLoader());
	if(FAILED(hr)) {
		return hr;
	}

	return S_OK;
}

// p^[ׂč폜܂B
void ARWindow::DestroyPatterns()
{
	for(vector<ARPattern*>::iterator it = m_patterns.begin(); it != m_patterns.end(); it ++) {
		ARPattern* pPattern = *it;
		delete pPattern;
	}
	m_patterns.clear();
}

// Ԃ񍐂܂B
HRESULT ARWindow::ReportStatus(ostream* pOut)
{
	if(m_pCaptureDevice != NULL) {
		HRESULT hr = m_pCaptureDevice->ReportStatus(pOut);
		if(FAILED(hr)) {
			return hr;
		}
	}

	if(m_pWindow != NULL) {
		HRESULT hr = m_pWindow->ReportStatus(pOut);
		if(FAILED(hr)) {
			return hr;
		}
	}

	if(arFittingMode == AR_FITTING_TO_INPUT ) {
		fprintf(stderr, "FittingMode (Z): INPUT IMAGE\n");
	} else {
		fprintf(stderr, "FittingMode (Z): COMPENSATED IMAGE\n");
	}
	
	if( arImageProcMode == AR_IMAGE_PROC_IN_FULL ) {
		fprintf(stderr, "ProcMode (X)   : FULL IMAGE\n");
	} else {
		fprintf(stderr, "ProcMode (X)   : HALF IMAGE\n");
	}
		
	if( arTemplateMatchingMode == AR_TEMPLATE_MATCHING_COLOR ) {
		fprintf(stderr, "TemplateMatchingMode (M)   : Color Template\n");
	} else {
		fprintf(stderr, "TemplateMatchingMode (M)   : BW Template\n");
	}
	
	if( arMatchingPCAMode == AR_MATCHING_WITHOUT_PCA ) {
		fprintf(stderr, "MatchingPCAMode (P)   : Without PCA\n");
	} else {
		fprintf(stderr, "MatchingPCAMode (P)   : With PCA\n");
	}
	return S_OK;
}

// \z܂B
ARWindow::PropertyImpl::PropertyImpl(ARWindow* pOwner, HRESULT* phr)
{
	m_pOwner = pOwner;
	*phr = S_OK;

	HRESULT hr = Init();
	if(FAILED(hr)) {
		*phr = hr;
		return;
	}
}

// j܂B
ARWindow::PropertyImpl::~PropertyImpl()
{
	m_pOwner = NULL;
}

// j܂B
void ARWindow::PropertyImpl::Destroy()
{
	delete this;
}

// int̃vpeB擾܂B
HRESULT ARWindow::PropertyImpl::GetIntProperty(const char* name, int* pValue)
{
	if(name == NULL || pValue == NULL) {
		return E_POINTER;
	}
	string sname = name;
	int xsize = 0, ysize = 0;
	if(arVideoInqSize(&xsize, &ysize) < 0) {
		xsize = 0;
		ysize = 0;
	}
	if(sname == "width") {
		*pValue = xsize;
		return S_OK;
	}else if(sname == "height") {
		*pValue = ysize;
		return S_OK;
	}else{
		return AbstractNamedProperty::GetIntProperty(name, pValue);
	}
}

// string̃vpeB擾܂B
HRESULT ARWindow::PropertyImpl::GetStringProperty(const char* name, string& value)
{
	if(name == NULL) {
		return E_POINTER;
	}
	string sname = name;
	if(sname == "status") {
		return m_pOwner->GetStatus(value);
	}else if(sname == "location.href") {
		IJSContext* pContext = m_pOwner->m_pContext;
		if(pContext == NULL) {
			return E_UNEXPECTED;
		}
		return pContext->GetBaseURL(value);
	}else{
		return AbstractNamedProperty::GetStringProperty(name, value);
	}
}

// string̃vpeBݒ肵܂B
HRESULT ARWindow::PropertyImpl::SetStringProperty(const char* name, const char* value)
{
	if(name == NULL || value == NULL) {
		return E_POINTER;
	}
	string sname = name;
	if(sname == "status") {
		return m_pOwner->SetStatus(value);
	}else{
		return AbstractNamedProperty::SetStringProperty(name, value);
	}
}

// ܂B
HRESULT ARWindow::PropertyImpl::Init()
{
	HRESULT hr = S_OK;

	hr = DefineProperty("status", PROP_STRING);
	if(FAILED(hr)) {
		return hr;
	}
	hr = DefineProperty("location.href", PROP_STRING);
	if(FAILED(hr)) {
		return hr;
	}
	hr = DefineProperty("width", PROP_INT);
	if(FAILED(hr)) {
		return hr;
	}
	hr = DefineProperty("height", PROP_INT);
	if(FAILED(hr)) {
		return hr;
	}

	return S_OK;
}

// \z܂B
ARWindow::ARRootGraphics::ARRootGraphics()
: ARGraphics(NULL)	// TODO: \K؂łȂ
{
}

// j܂B
ARWindow::ARRootGraphics::~ARRootGraphics()
{
}

// j܂B
void ARWindow::ARRootGraphics::Destroy()
{
	;
}

// `悵܂B
HRESULT ARWindow::ARRootGraphics::Draw()
{
	return DrawChildren();
}

// IJSObject擾܂B
IJSObject* ARWindow::ARRootGraphics::GetJSObject()
{
	return NULL;
}

