#include "StdAfx.h"
#include "TestARPattern.h"
#include <GL/gl.h>
#include <GL/glut.h>
#include <AR/gsub.h>
#include <AR/video.h>
#include <AR/param.h>
#include <AR/ar.h>

// p^[ێ܂B
const char* PATTERN_PATH = "Data/patt.hiro";

// \z܂B
TestARPattern::TestARPattern(HRESULT* phr)
: ARPattern()
{
	m_patternWidth = 80.0;
	m_patternCenter[0] = 0.0;
	m_patternCenter[1] = 0.0;
	m_visible = false;
	ZeroMemory(& m_trans, sizeof(m_trans));
	*phr = S_OK;

	HRESULT hr = LoadPattern(PATTERN_PATH);
	if(FAILED(hr)) {
		*phr = hr;
		return;
	}
}

// j܂B
TestARPattern::~TestARPattern(void)
{
}

// j܂B
void TestARPattern::Destroy()
{
	delete this;
}

// GraphicsIuWFNg擾܂B
ARGraphics* TestARPattern::CreateGraphics(ARWindow* pWindow)
{
	return new GraphicsImpl(pWindow, this);
}

// oԂʒm܂B
HRESULT TestARPattern::Detect(ARMarkerInfoContainer* pDetectedC)
{
	ARMarkerInfo* pDetected = (ARMarkerInfo*) pDetectedC->pCore;
	if(pDetected == NULL) {
		m_visible = false;
		return S_OK;
	}

	m_visible = true;
    arGetTransMat(pDetected, m_patternCenter, m_patternWidth, m_trans);

	return S_OK;
}

// IuWFNg`悵܂B
HRESULT TestARPattern::Draw()
{
	if(m_visible == false) {
		return S_FALSE;
	}
    double    gl_para[16];
    GLfloat   mat_ambient[]     = {0.0, 0.0, 1.0, 1.0};
    GLfloat   mat_flash[]       = {0.0, 0.0, 1.0, 1.0};
    GLfloat   mat_flash_shiny[] = {50.0};
    GLfloat   light_position[]  = {100.0,-200.0,200.0,0.0};
    GLfloat   ambi[]            = {0.1, 0.1, 0.1, 0.1};
    GLfloat   lightZeroColor[]  = {0.9, 0.9, 0.9, 0.1};
    
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LEQUAL);
    
    /* load the camera transformation matrix */
    argConvGlpara(m_trans, gl_para);
    glMatrixMode(GL_MODELVIEW);
    glLoadMatrixd( gl_para );

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glLightfv(GL_LIGHT0, GL_POSITION, light_position);
    glLightfv(GL_LIGHT0, GL_AMBIENT, ambi);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, lightZeroColor);
    glMaterialfv(GL_FRONT, GL_SPECULAR, mat_flash);
    glMaterialfv(GL_FRONT, GL_SHININESS, mat_flash_shiny);	
    glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
    glMatrixMode(GL_MODELVIEW);
    glTranslatef( 0.0, 0.0, 25.0 );
    glutSolidCube(50.0);
    glDisable( GL_LIGHTING );

    glDisable( GL_DEPTH_TEST );

	return S_OK;
}

// \z܂B
TestARPattern::GraphicsImpl::GraphicsImpl(ARWindow* pWindow, TestARPattern* pOwner)
: ARGraphics(pWindow)
{
	m_pOwner = pOwner;
}

// j܂B
TestARPattern::GraphicsImpl::~GraphicsImpl()
{
	;
}

// j܂B
void TestARPattern::GraphicsImpl::Destroy()
{
	delete this;
}

// `悵܂B
HRESULT TestARPattern::GraphicsImpl::Draw()
{
	return m_pOwner->Draw();
}

// IJSObject擾܂B
IJSObject* TestARPattern::GraphicsImpl::GetJSObject()
{
	return NULL;
}
