#include "StdAfx.h"
#include "ARJSGraphicsModel.h"
#include <GL/gl.h>
#include <GL/glut.h>
#include <AR/gsub.h>
#include <AR/video.h>
#include <AR/param.h>
#include <AR/ar.h>
#include "../../js/JSValueWrapper.h"
#include "AbstractARJSObject.h"

// \z܂B
ARJSGraphicsModel::ARJSGraphicsModel(ARWindow* pOwner)
: ARGraphics(pOwner)
{
	ZeroMemory(& m_translate, sizeof(m_translate));
	ZeroMemory(& m_rotateBase, sizeof(m_rotateBase));
	ZeroMemory(& m_scale, sizeof(m_scale));
	m_scale.x = 1.0;
	m_scale.y = 1.0;
	m_scale.z = 1.0;
	m_rotateAngle = 0.0;
	m_pJSObject = NULL;
}

// j܂B
ARJSGraphicsModel::~ARJSGraphicsModel(void)
{
	if(m_pJSObject != NULL) {
		// thisJSObjectImpl͓Ct^C
		m_pJSObject->Destroy();
		m_pJSObject = NULL;
	}
}

// JavaScriptGWݒ肵܂B
HRESULT ARJSGraphicsModel::Attach(JSEngineContext* pContext)
{
	if(pContext == NULL) {
		return E_POINTER;
	}

	m_pJSObject = CreateJSObject();
	if(m_pJSObject == NULL) {
		return E_OUTOFMEMORY;
	}

	return m_pJSObject->Attach(pContext, NULL);
}

// IJSObject擾܂B
IJSObject* ARJSGraphicsModel::GetJSObject()
{
	return m_pJSObject;
}

// `悵܂B
HRESULT ARJSGraphicsModel::Draw()
{
	if(m_visible == false) {
		return S_FALSE;
	}

	glPushMatrix();
    glScalef(m_scale.x, m_scale.y, m_scale.z);
    glTranslatef(m_translate.x, m_translate.y, m_translate.z);
    glRotatef(m_rotateAngle, m_rotateBase.x, m_rotateBase.y, m_rotateBase.z);

	HRESULT hr = DrawModel();

	glPopMatrix();

	return hr;
}

// IuWFNgt@Ng̒ǉʒm܂B
HRESULT ARJSGraphicsModel::NotifyObjectFactoryAdded(const char* name, IARGObjectFactory* pFactory)
{
	if(name == NULL || pFactory == NULL) {
		return E_POINTER;
	}
	if(m_pJSObject != NULL) {
		HRESULT hr = m_pJSObject->NotifyObjectFactoryAdded(name, pFactory);
		if(FAILED(hr)) {
			return hr;
		}
	}

	return ARGraphics::NotifyObjectFactoryAdded(name, pFactory);
}

// IJSObject𐶐܂B
AbstractARJSObject* ARJSGraphicsModel::CreateJSObject()
{
	INamedProperty* pProp = NULL;
	HRESULT hr = GetProperty(& pProp);
	if(FAILED(hr)) {
		return NULL;
	}
	return new AbstractARJSObject(this, pProp);
}

// vpeB𐶐܂B
HRESULT ARJSGraphicsModel::CreateProperty(INamedProperty** ppProp)
{
	if(ppProp == NULL) {
		return E_POINTER;
	}
	ModelPropertyImpl* pNewProp = new ModelPropertyImpl(this);
	if(pNewProp == NULL) {
		return E_OUTOFMEMORY;
	}
	HRESULT hr = pNewProp->Init();
	if(FAILED(hr)) {
		delete pNewProp;
		pNewProp = NULL;
		return hr;
	}
	*ppProp = pNewProp;
	return S_OK;
}


// \z܂B
ARJSGraphicsModel::ModelPropertyImpl::ModelPropertyImpl(ARJSGraphicsModel* pOwner)
: GraphicsPropertyImpl(pOwner)
{
	m_pOwner = pOwner;
}

// j܂B
ARJSGraphicsModel::ModelPropertyImpl::~ModelPropertyImpl()
{
	m_pOwner = NULL;
}

// j܂B
void ARJSGraphicsModel::ModelPropertyImpl::Destroy()
{
	delete this;
}
		
// ܂B
HRESULT ARJSGraphicsModel::ModelPropertyImpl::Init()
{
	HRESULT hr = S_OK;

	hr = DefineProperty("translate.x", PROP_DOUBLE);
	if(FAILED(hr)) {
		return hr;
	}
	hr = DefineProperty("translate.y", PROP_DOUBLE);
	if(FAILED(hr)) {
		return hr;
	}
	hr = DefineProperty("translate.z", PROP_DOUBLE);
	if(FAILED(hr)) {
		return hr;
	}
	hr = DefineProperty("scale.x", PROP_DOUBLE);
	if(FAILED(hr)) {
		return hr;
	}
	hr = DefineProperty("scale.y", PROP_DOUBLE);
	if(FAILED(hr)) {
		return hr;
	}
	hr = DefineProperty("scale.z", PROP_DOUBLE);
	if(FAILED(hr)) {
		return hr;
	}
	hr = DefineProperty("rotate.angle", PROP_DOUBLE);
	if(FAILED(hr)) {
		return hr;
	}
	hr = DefineProperty("rotate.x", PROP_DOUBLE);
	if(FAILED(hr)) {
		return hr;
	}
	hr = DefineProperty("rotate.y", PROP_DOUBLE);
	if(FAILED(hr)) {
		return hr;
	}
	hr = DefineProperty("rotate.z", PROP_DOUBLE);
	if(FAILED(hr)) {
		return hr;
	}

	return GraphicsPropertyImpl::Init();
}

// doublẽvpeB擾܂B
HRESULT ARJSGraphicsModel::ModelPropertyImpl::GetDoubleProperty(const char* name, double* pValue)
{
	if(name == NULL || pValue == NULL) {
		return E_POINTER;
	}
	string categ, prop;
	HRESULT hr = ParseName(name, categ, prop);
	if(FAILED(hr)) {
		return hr;
	}
	if(categ == "translate") {
		GLfloat* pRefValue = NULL;
		hr = GetMember(&(m_pOwner->m_translate), prop.c_str(), & pRefValue);
		if(FAILED(hr)) {
			return hr;
		}

		*pValue = *pRefValue;
		return S_OK;
	}else if(categ == "rotate") {
		GLfloat* pRefValue = NULL;
		if(prop == "angle") {
			pRefValue = &(m_pOwner->m_rotateAngle);
		}else{
			hr = GetMember(&(m_pOwner->m_rotateBase), prop.c_str(), & pRefValue);
			if(FAILED(hr)) {
				return hr;
			}
		}

		*pValue = *pRefValue;
		return S_OK;
	}else if(categ == "scale") {
		GLfloat* pRefValue = NULL;
		hr = GetMember(&(m_pOwner->m_scale), prop.c_str(), & pRefValue);
		if(FAILED(hr)) {
			return hr;
		}

		*pValue = *pRefValue;
		return S_OK;
	}else{
		return GraphicsPropertyImpl::GetDoubleProperty(name, pValue);
	}
}

// doublẽvpeBݒ肵܂B
HRESULT ARJSGraphicsModel::ModelPropertyImpl::SetDoubleProperty(const char* name, double value)
{
	if(name == NULL) {
		return E_POINTER;
	}
	string categ, prop;
	HRESULT hr = ParseName(name, categ, prop);
	if(FAILED(hr)) {
		return hr;
	}
	if(categ == "translate") {
		GLfloat* pRefValue = NULL;
		hr = GetMember(&(m_pOwner->m_translate), prop.c_str(), & pRefValue);
		if(FAILED(hr)) {
			return hr;
		}

		*pRefValue = value;
		return S_OK;
	}else if(categ == "rotate") {
		GLfloat* pRefValue = NULL;
		if(prop == "angle") {
			pRefValue = &(m_pOwner->m_rotateAngle);
		}else{
			hr = GetMember(&(m_pOwner->m_rotateBase), prop.c_str(), & pRefValue);
			if(FAILED(hr)) {
				return hr;
			}
		}

		*pRefValue = value;
		return S_OK;
	}else if(categ == "scale") {
		GLfloat* pRefValue = NULL;
		hr = GetMember(&(m_pOwner->m_scale), prop.c_str(), & pRefValue);
		if(FAILED(hr)) {
			return hr;
		}

		*pRefValue = value;
		return S_OK;
	}else{
		return GraphicsPropertyImpl::SetDoubleProperty(name, value);
	}
}

// O͂܂B
HRESULT ARJSGraphicsModel::ModelPropertyImpl::ParseName(const char* name, string& categoryName, string& propName)
{
	if(name == NULL) {
		return E_POINTER;
	}
	string sname = name;
	string::size_type dotPos = sname.find(".");
	if(dotPos == string::npos) {
		categoryName = "";
		propName = sname;
		return S_OK;
	}

	categoryName = sname.substr(0, dotPos);
	propName = sname.substr(dotPos + 1);
	return S_OK;
}

// o擾܂B
HRESULT ARJSGraphicsModel::ModelPropertyImpl::GetMember(GLVECTOR* pVector, const char* propName, GLfloat** ppValue)
{
	if(pVector == NULL || propName == NULL || ppValue == NULL) {
		return E_POINTER;
	}

	string spropName = propName;
	if(spropName == "x") {
		*ppValue = &(pVector->x);
		return S_OK;
	}else if(spropName == "y") {
		*ppValue = &(pVector->y);
		return S_OK;
	}else if(spropName == "z") {
		*ppValue = &(pVector->z);
		return S_OK;
	}
	return E_FAIL;
}
