#include "StdAfx.h"
#include "ARJSWindow.h"
#include "../ARWindow.h"
#include <GL/gl.h>
#include <GL/glut.h>
#include <AR/gsub.h>
#include <AR/video.h>
#include <AR/param.h>
#include <AR/ar.h>
#include "ARJSPattern.h"
#include "ARJSWorld.h"
#include "../../js/JSValueWrapper.h"
#include "../../common/URLPath.h"

// \z܂B
ARJSWindow::ARJSWindow(ARWindow* pWindow, INamedProperty* pProp)
: JSNamedProperty(pProp)
{
	m_pWindow = pWindow;
	m_pPatternList = NULL;
	m_pDump = NULL;
}

// j܂B
ARJSWindow::~ARJSWindow()
{
	m_pWindow = NULL;

	if(m_pPatternList != NULL) {
		m_pPatternList->Destroy();
		m_pPatternList = NULL;
	}
	// cĂꍇWindowĂ
	m_pDump = NULL;
}

// j܂B
void ARJSWindow::Destroy()
{
	delete this;
}

// L[Cxg܂B
HRESULT ARJSWindow::OnKeyPress(unsigned char key, int x, int y, bool* pPreventDefault)
{
	JSObject* pEvent = NULL;
	HRESULT hr = CreateKeyEvent(key, x, y, & pEvent);
	if(FAILED(hr)) {
		return hr;
	}

	jsval eventVal = OBJECT_TO_JSVAL(pEvent);
	jsval rval = JSVAL_FALSE;

	hr = m_onKeyPress.InvokeSync(& eventVal, 1, & rval);
	if(FAILED(hr)) {
		return hr;
	}

	if(pPreventDefault != NULL) {
		JSBool bdef = JSVAL_TO_BOOLEAN(rval);
		if(bdef == JS_FALSE) {
			*pPreventDefault = true;
		}
	}
	return S_OK;
}

// ܂B
HRESULT ARJSWindow::InitMembers()
{
	HRESULT hr = S_OK;
	hr = DefineProperty("onkeypress", false);
	if(FAILED(hr)) {
		return hr;
	}
	hr = DefineProperty("patterns", true);
	if(FAILED(hr)) {
		return hr;
	}
	hr = DefineProperty("worlds", true);
	if(FAILED(hr)) {
		return hr;
	}

	hr = DefineFunction("initVideo", 1);
	if(FAILED(hr)) {
		return hr;
	}
	hr = DefineFunction("initWindow", 1);
	if(FAILED(hr)) {
		return hr;
	}
	hr = DefineFunction("loadPattern", 1);
	if(FAILED(hr)) {
		return hr;
	}
	hr = DefineFunction("addWorld", 0);
	if(FAILED(hr)) {
		return hr;
	}
	hr = DefineFunction("saveImage", 1);
	if(FAILED(hr)) {
		return hr;
	}
	hr = DefineFunction("startSaveMovie", 3);
	if(FAILED(hr)) {
		return hr;
	}
	hr = DefineFunction("stopSaveMovie", 0);
	if(FAILED(hr)) {
		return hr;
	}

	hr = m_onKeyPress.Init(m_pContext, this);
	if(FAILED(hr)) {
		return hr;
	}

	m_pPatternList = new PatternList(this);
	if(m_pPatternList == NULL) {
		return E_OUTOFMEMORY;
	}

	hr = m_patternList.Attach(m_pPatternList, m_pContext, this);
	if(FAILED(hr)) {
		return hr;
	}

	IArrayProperty* pWorldList = NULL;
	hr = m_pWindow->GetWorlds(& pWorldList);
	if(FAILED(hr)) {
		return hr;
	}

	hr = m_worldList.Attach(pWorldList, m_pContext, this);
	if(FAILED(hr)) {
		return hr;
	}

	return JSNamedProperty::InitMembers();
}

// vpeBݒ肵܂B
HRESULT ARJSWindow::SetProperty(JSContext *cx, JSObject *obj, const char* name, jsval vp)
{
	if(name == NULL) {
		return E_POINTER;
	}
	string sname = name;
	if(sname == "onkeypress") {
		return m_onKeyPress.Set(vp);
	}else{
		return JSNamedProperty::SetProperty(cx, obj, name, vp);
	}
}

// vpeB擾܂B
HRESULT ARJSWindow::GetProperty(JSContext* cx, JSObject* obj, const char* name, jsval *vp)
{
	if(name == NULL) {
		return E_POINTER;
	}
	string sname = name;
	if(sname == "onkeypress") {
		if(vp == NULL) {
			return S_OK;
		}
		return m_onKeyPress.Get(vp);
	}else if(sname == "patterns") {
		if(vp == NULL) {
			return S_OK;
		}
		IJSObject* pObj = NULL;
		HRESULT hr = m_patternList.GetJSObject(& pObj);
		if(FAILED(hr)) {
			return hr;
		}
		*vp = OBJECT_TO_JSVAL(pObj->GetCore());
		return S_OK;
	}else if(sname == "worlds") {
		if(vp == NULL) {
			return S_OK;
		}
		IJSObject* pObj = NULL;
		HRESULT hr = m_worldList.GetJSObject(& pObj);
		if(FAILED(hr)) {
			return hr;
		}
		*vp = OBJECT_TO_JSVAL(pObj->GetCore());
		return S_OK;
	}else{
		return JSNamedProperty::GetProperty(cx, obj, name, vp);
	}
}

// ֐s܂B
HRESULT ARJSWindow::InvokeFunction(JSContext *cx, JSObject *obj, const char* name, uintN argc, jsval *argv, jsval *rval)
{
	if(name == NULL) {
		return E_POINTER;
	}
	string sname = name;
	if(sname == "initVideo") {
		return InitVideoImpl(cx, obj, argc, argv, rval);
	}else if(sname == "initWindow") {
		return InitWindowImpl(cx, obj, argc, argv, rval);
	}else if(sname == "loadPattern") {
		return LoadPatternImpl(cx, obj, argc, argv, rval);
	}else if(sname == "addWorld") {
		return AddWorldImpl(cx, obj, argc, argv, rval);
	}else if(sname == "saveImage") {
		return SaveImageImpl(cx, obj, argc, argv, rval);
	}else if(sname == "startSaveMovie") {
		return StartSaveMovieImpl(cx, obj, argc, argv, rval);
	}else if(sname == "stopSaveMovie") {
		return StopSaveMovieImpl(cx, obj, argc, argv, rval);
	}else{
		return JSNamedProperty::InvokeFunction(cx, obj, name, argc, argv, rval);
	}
}

// L[Cxg𐶐܂B
HRESULT ARJSWindow::CreateKeyEvent(unsigned char key, int x, int y, JSObject** ppObject)
{
	if(ppObject == NULL) {
		return E_POINTER;
	}

	JSObject* pEvent = NULL;
	HRESULT hr = m_pContext->CreateNewObject(& pEvent);
	if(FAILED(hr)) {
		return hr;
	}

	jsval keyCodeVal = INT_TO_JSVAL(key);

	JSBool bret = JS_SetProperty(m_pContext->GetCore(), pEvent, "keyCode", & keyCodeVal);
	if(bret == JS_FALSE) {
		return E_FAIL;
	}

	*ppObject = pEvent;
	return S_OK;
}

// InitVideo֐̒`
HRESULT ARJSWindow::InitVideoImpl(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if(argc < 1) {
		return E_INVALIDARG;
	}

	string config;
	HRESULT hr = GetValueToString(argv[0], config);
	if(FAILED(hr)) {
		return hr;
	}

	return m_pWindow->InitVideo(config.c_str());
}

// InitVideo֐̒`
HRESULT ARJSWindow::InitWindowImpl(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if(argc < 1) {
		return E_INVALIDARG;
	}

	string config;
	HRESULT hr = GetValueToString(argv[0], config);
	if(FAILED(hr)) {
		return hr;
	}

	return m_pWindow->InitWindow(config.c_str());
}

// LoadPattern֐̒`
HRESULT ARJSWindow::LoadPatternImpl(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if(argc < 1) {
		return E_INVALIDARG;
	}

	string path;
	HRESULT hr = GetValueToString(argv[0], path);
	if(FAILED(hr)) {
		return hr;
	}
	string url;
	hr = m_pContext->GetAbsoluteURL(path.c_str(), url);
	if(FAILED(hr)) {
		return hr;
	}

	URLPath urlPath(url.c_str(), & hr);
	if(FAILED(hr)) {
		return hr;
	}
	string realPath;
	hr = urlPath.GetRealPath(realPath);
	if(FAILED(hr)) {
		return hr;
	}

	ARJSPattern* pNewPattern = new ARJSPattern(url.c_str(), realPath.c_str(), & hr);
	if(pNewPattern == NULL) {
		return E_OUTOFMEMORY;
	}
	if(FAILED(hr)) {
		return hr;
	}

	hr = m_pWindow->AddPattern(pNewPattern);
	if(FAILED(hr)) {
		return hr;
	}

	hr = pNewPattern->Attach(m_pContext);
	if(FAILED(hr)) {
		return hr;
	}

	if(rval == NULL) {
		return S_OK;
	}
	*rval = OBJECT_TO_JSVAL(pNewPattern->GetJSObject()->GetCore());
	return S_OK;
}

// AddWorld֐̒`
HRESULT ARJSWindow::AddWorldImpl(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	ARJSWorld* pNewWorld = new ARJSWorld(m_pWindow);
	if(pNewWorld == NULL) {
		return E_OUTOFMEMORY;
	}

	int index = 0;
	HRESULT hr = m_pWindow->AddGraphics(pNewWorld, & index);
	if(FAILED(hr)) {
		return hr;
	}

	hr = pNewWorld->Attach(m_pContext);
	if(FAILED(hr)) {
		return hr;
	}

	if(rval == NULL) {
		return S_OK;
	}
	*rval = OBJECT_TO_JSVAL(pNewWorld->GetJSObject()->GetCore());
	return S_OK;
}

// XN[Lv`ۑ܂B
HRESULT ARJSWindow::SaveImageImpl(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if(argc < 1) {
		return E_INVALIDARG;
	}

	string path;
	HRESULT hr = GetValueToString(argv[0], path);
	if(FAILED(hr)) {
		return hr;
	}
	string url;
	hr = m_pContext->GetAbsoluteURL(path.c_str(), url);
	if(FAILED(hr)) {
		return hr;
	}

	URLPath urlPath(url.c_str(), & hr);
	if(FAILED(hr)) {
		return hr;
	}
	string realPath;
	hr = urlPath.GetRealPath(realPath);
	if(FAILED(hr)) {
		return hr;
	}

	IWindow* pWnd = NULL;
	hr = m_pWindow->GetWindow(& pWnd);
	if(FAILED(hr)) {
		return hr;
	}

	hr = S_OK;
	ImageFileDump* pDump = new ImageFileDump(pWnd, m_pContext, realPath.c_str(), & hr);
	if(FAILED(hr)) {
		delete pDump;
		pDump = NULL;
		return hr;
	}
	hr = pWnd->AddGLHook(pDump);
	if(FAILED(hr)) {
		delete pDump;
		pDump = NULL;
		return hr;
	}

	return pDump->Start();
}

// XN[Lv`Jn܂B
HRESULT ARJSWindow::StartSaveMovieImpl(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if(argc < 3) {
		return E_INVALIDARG;
	}
	if(m_pDump != NULL) {
		return E_UNEXPECTED;
	}

	string prefix;
	HRESULT hr = GetValueToString(argv[0], prefix);
	if(FAILED(hr)) {
		return hr;
	}

	string suffix;
	hr = GetValueToString(argv[1], suffix);
	if(FAILED(hr)) {
		return hr;
	}

	int bufferSize = JSVAL_TO_INT(argv[2]);

	string url;
	hr = m_pContext->GetAbsoluteURL(prefix.c_str(), url);
	if(FAILED(hr)) {
		return hr;
	}

	URLPath urlPath(url.c_str(), & hr);
	if(FAILED(hr)) {
		return hr;
	}
	string realPrefix;
	hr = urlPath.GetRealPath(realPrefix);
	if(FAILED(hr)) {
		return hr;
	}

	IWindow* pWnd = NULL;
	hr = m_pWindow->GetWindow(& pWnd);
	if(FAILED(hr)) {
		return hr;
	}

	hr = S_OK;
	MovieFileDump* pDump = new MovieFileDump(pWnd, m_pContext, realPrefix.c_str(), suffix.c_str(), bufferSize, & hr);
	if(FAILED(hr)) {
		delete pDump;
		pDump = NULL;
		return hr;
	}
	hr = pWnd->AddGLHook(pDump);
	if(FAILED(hr)) {
		delete pDump;
		pDump = NULL;
		return hr;
	}
	m_pDump = pDump;

	return pDump->Start();
}

// XN[Lv`I܂B
HRESULT ARJSWindow::StopSaveMovieImpl(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if(m_pDump != NULL) {
		return S_OK;
	}

	IWindow* pWnd = NULL;
	HRESULT hr = m_pWindow->GetWindow(& pWnd);
	if(FAILED(hr)) {
		return hr;
	}

	hr = pWnd->RemoveGLHook(m_pDump);
	if(FAILED(hr)) {
		return hr;
	}

	m_pDump->Destroy();
	m_pDump = NULL;
	return S_OK;
}

// \z܂B
ARJSWindow::PatternList::PatternList(ARJSWindow* pOwner)
{
	m_pOwner = pOwner;
}

// j܂B
ARJSWindow::PatternList::~PatternList()
{
	m_pOwner = NULL;
}

// j܂B
void ARJSWindow::PatternList::Destroy()
{
	delete this;
}

// vpeB擾܂B
HRESULT ARJSWindow::PatternList::GetPropertyCount(int* pCount)
{
	if(pCount == NULL) {
		return E_POINTER;
	}
	vector<ARPattern*> patterns;
	HRESULT hr = m_pOwner->m_pWindow->GetPatterns(patterns);
	if(FAILED(hr)) {
		return hr;
	}
	*pCount = (int)patterns.size();
	return S_OK;
}
	
// vpeB擾܂B
HRESULT ARJSWindow::PatternList::GetPropertyDesc(int index, PROPERTY_DESC* pDesc)
{
	if(pDesc == NULL) {
		return E_POINTER;
	}
	pDesc->type = PROP_JSOBJECT;
	return S_OK;
}

// IJSObject̃vpeB擾܂B
HRESULT ARJSWindow::PatternList::GetJSObject(int index, IJSObject** ppValue)
{
	if(ppValue == NULL) {
		return E_POINTER;
	}
	vector<ARPattern*> patterns;
	HRESULT hr = m_pOwner->m_pWindow->GetPatterns(patterns);
	if(FAILED(hr)) {
		return hr;
	}

	*ppValue = patterns[index]->GetJSObject();
	return S_OK;
}

// \z܂B
ARJSWindow::ImageFileDump::ImageFileDump(IWindow* pWindow, JSEngineContext* pContext, const char* savePath, HRESULT* phr)
: GLBitmapDump(pWindow, pContext, 1, phr)
{
	m_savePath = savePath;
}

// j܂B
ARJSWindow::ImageFileDump::~ImageFileDump()
{
}

// ۑp̃pX擾܂B
HRESULT ARJSWindow::ImageFileDump::GetSavePath(LONGLONG t, string& path, bool* pExit)
{
	if(pExit == NULL) {
		return E_POINTER;
	}
	path = m_savePath;
	*pExit = true;
	return S_OK;
}

// \z܂B
ARJSWindow::MovieFileDump::MovieFileDump(IWindow* pWindow, JSEngineContext* pContext, const char* prefix, const char* suffix, int bufferSize, HRESULT* phr)
: GLBitmapDump(pWindow, pContext, bufferSize, phr)
{
	m_savePrefix = prefix;
	m_saveSuffix = suffix;
	m_iIndex = 0;
}

// j܂B
ARJSWindow::MovieFileDump::~MovieFileDump()
{
}

// ۑp̃pX擾܂B
HRESULT ARJSWindow::MovieFileDump::GetSavePath(LONGLONG t, string& path, bool* pExit)
{
	if(pExit == NULL) {
		return E_POINTER;
	}
	string s = m_savePrefix;
	char buf[256];
	sprintf_s(buf, 255, "%05u", m_iIndex);
	s += buf;
	s += m_saveSuffix;
	path = s;
	*pExit = false;
	m_iIndex ++;
	return S_OK;
}
