#include "StdAfx.h"
#include "ARJSWorld.h"
#include <GL/gl.h>
#include <GL/glut.h>
#include <AR/gsub.h>
#include <AR/video.h>
#include <AR/param.h>
#include <AR/ar.h>
#include <AR/gsub_lite.h>
#include "ARJSSolidCube.h"
#include "ARJSImage.h"
#include "../ARWindow.h"
#include "../../js/JSValueWrapper.h"

#define VIEW_SCALEFACTOR_4		1.0 //4.0			// 1.0 ARToolKit unit becomes 4.0 of my OpenGL units.

// \z܂B
ARJSWorld::ARJSWorld(ARWindow* pWindow)
: ARGraphics(pWindow)
{
	m_pWindow = pWindow;
	m_pJSObject = NULL;
	ZeroMemory(& m_trans, sizeof(m_trans));
}

// j܂B
ARJSWorld::~ARJSWorld(void)
{
	if(m_pJSObject != NULL) {
		// thisJSObjectImpl͓Ct^C
		m_pJSObject->Destroy();
		m_pJSObject = NULL;
	}
	m_pWindow = NULL;
}

// j܂B
void ARJSWorld::Destroy()
{
	delete this;
}

// JavaScriptGWݒ肵܂B
HRESULT ARJSWorld::Attach(JSEngineContext* pContext)
{
	if(pContext == NULL) {
		return E_POINTER;
	}

	INamedProperty* pProp = NULL;
	HRESULT hr = GetProperty(& pProp);
	if(FAILED(hr)) {
		return hr;
	}

	m_pJSObject = new JSObjectImpl(this, pProp);
	if(m_pJSObject == NULL) {
		return E_OUTOFMEMORY;
	}

	hr = m_pJSObject->Attach(pContext, NULL);
	if(FAILED(hr)) {
		return hr;
	}

	IArrayProperty* pChildNodes = NULL;
	hr = GetChildren(& pChildNodes);
	if(FAILED(hr)) {
		return hr;
	}

	hr = m_childNodes.Attach(pChildNodes, pContext, m_pJSObject);
	if(FAILED(hr)) {
		return hr;
	}

	return S_OK;
}

// IJSObject擾܂B
IJSObject* ARJSWorld::GetJSObject()
{
	return m_pJSObject;
}

// `悵܂B
HRESULT ARJSWorld::Draw()
{
	if(m_visible == false) {
		return S_FALSE;
	}
    double    gl_para[16];
    GLfloat   light_position[]  = {100.0,-200.0,200.0,0.0};
    GLfloat   ambi[]            = {0.1, 0.1, 0.1, 0.1};
    GLfloat   lightZeroColor[]  = {0.9, 0.9, 0.9, 0.1};
    
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LEQUAL);
    
	arglCameraViewRH(m_trans, gl_para, VIEW_SCALEFACTOR_4);
    glMatrixMode(GL_MODELVIEW);
    glLoadMatrixd( gl_para );

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glLightfv(GL_LIGHT0, GL_POSITION, light_position);
    glLightfv(GL_LIGHT0, GL_AMBIENT, ambi);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, lightZeroColor);

	HRESULT hr = DrawChildren();
	
	glDisable( GL_LIGHTING );

    glDisable( GL_DEPTH_TEST );

	return hr;
}

// \z܂B
ARJSWorld::JSObjectImpl::JSObjectImpl(ARJSWorld* pOwner, INamedProperty* pProp)
: AbstractARJSObject(pOwner, pProp)
{
	m_pOwner = pOwner;
}

// j܂B
ARJSWorld::JSObjectImpl::~JSObjectImpl()
{
	m_pOwner = NULL;
}

// j܂B
void ARJSWorld::JSObjectImpl::Destroy()
{
	delete this;
}

// o܂B
HRESULT ARJSWorld::JSObjectImpl::InitMembers()
{
	HRESULT hr = DefineProperty("translate", false);
	if(FAILED(hr)) {
		return hr;
	}

	hr = DefineProperty("childNodes", true);
	if(FAILED(hr)) {
		return hr;
	}

	return AbstractARJSObject::InitMembers();
}

// vpeBݒ肵܂B
HRESULT ARJSWorld::JSObjectImpl::SetProperty(JSContext *cx, JSObject *obj, const char* name, jsval vp)
{
	string sname = name;

	if(sname == "translate") {
		if(vp == NULL) {
			return E_POINTER;
		}

		JSObject* pObj = NULL;
		JSBool bret = JS_ValueToObject(cx, vp , & pObj);
		if(bret == JS_FALSE) {
			return E_INVALIDARG;
		}
		if(JS_IsArrayObject(cx, pObj) == JS_FALSE) {
			return E_INVALIDARG;
		}

		jsuint len = 0;
		if(JS_GetArrayLength(cx, pObj, & len) == JS_FALSE) {
			return E_INVALIDARG;
		}
		if(len != 3) {
			return E_INVALIDARG;
		}

		for(jsuint i = 0; i < len; i ++) {
			jsval elemv = JSVAL_NULL;
			if(JS_GetElement(cx, pObj, i, & elemv) == JS_FALSE) {
				return E_INVALIDARG;
			}

			HRESULT hr = SetTransElement(cx, i, elemv);
			if(FAILED(hr)) {
				return hr;
			}
		}

		return S_OK;
	}else{
		return AbstractARJSObject::SetProperty(cx, obj, name, vp);
	}
}

// vpeB擾܂B
HRESULT ARJSWorld::JSObjectImpl::GetProperty(JSContext* cx, JSObject* obj, const char* name, jsval *vp)
{
	string sname = name;

	if(sname == "translate") {
		if(vp == NULL) {
			return S_OK;
		}
		jsval transValues[3];
		for(int i = 0; i < 3; i ++) {
			JSObject* pArray = NULL;
			HRESULT hr = CreateDoubleArray(m_pOwner->m_trans[i], 4, & pArray);
			if(FAILED(hr)) {
				return hr;
			}
			transValues[i] = OBJECT_TO_JSVAL(pArray);
		}
		JSObject* pTransObj = JS_NewArrayObject(cx, 3, transValues);
		if(pTransObj == NULL) {
			return E_FAIL;
		}
		*vp = OBJECT_TO_JSVAL(pTransObj);
		return S_OK;
	}else if(sname == "childNodes") {
		if(vp == NULL) {
			return S_OK;
		}
		IJSObject* pObj = NULL;
		HRESULT hr = m_pOwner->m_childNodes.GetJSObject(& pObj);
		if(FAILED(hr)) {
			return hr;
		}

		*vp = OBJECT_TO_JSVAL(pObj->GetCore());
		return S_OK;
	}else{
		return AbstractARJSObject::GetProperty(cx, obj, name, vp);
	}
}

// IDݒ肵܂B
HRESULT ARJSWorld::JSObjectImpl::SetTransElement(JSContext *cx, jsuint index, jsval arrv)
{
	JSObject* pObj = NULL;
	JSBool bret = JS_ValueToObject(cx, arrv, & pObj);
	if(bret == JS_FALSE) {
		return E_INVALIDARG;
	}
	if(JS_IsArrayObject(cx, pObj) == JS_FALSE) {
		return E_INVALIDARG;
	}

	jsuint len = 0;
	if(JS_GetArrayLength(cx, pObj, & len) == JS_FALSE) {
		return E_INVALIDARG;
	}
	if(len != 4) {
		return E_INVALIDARG;
	}

	for(jsuint i = 0; i < len; i ++) {
		jsval elemv = JSVAL_NULL;
		if(JS_GetElement(cx, pObj, i, & elemv) == JS_FALSE) {
			return E_INVALIDARG;
		}

		jsdouble v = 0.0;
		bret = JS_ValueToNumber(cx, elemv, & v);
		if(bret == JS_FALSE) {
			return E_INVALIDARG;
		}
		m_pOwner->m_trans[index][i] = v;
	}

	return S_OK;
}
