#include "StdAfx.h"
#include "AbstractARJSObject.h"
#include "ARJSSolidCube.h"
#include "ARJSPluginObject.h"
#include "ARJSImage.h"
#include "ARJSContainer.h"
#include "../../js/JSValueWrapper.h"
#include "../ARWindow.h"
#include "../../image/TextLoaderFactory.h"
#include "../../js/JSDispatch.h"


// \z܂B
AbstractARJSObject::AbstractARJSObject(ARGraphics* pOwner, INamedProperty* pProp)
: JSNamedProperty(pProp)
{
	m_pOwner = pOwner;
}

// j܂B
AbstractARJSObject::~AbstractARJSObject()
{
	m_pOwner = NULL;
}

// j܂B
void AbstractARJSObject::Destroy()
{
	delete this;
}

// IuWFNgt@Ng̒ǉʒm܂B
HRESULT AbstractARJSObject::NotifyObjectFactoryAdded(const char* name, IARGObjectFactory* pFactory)
{
	if(name == NULL || pFactory == NULL) {
		return E_POINTER;
	}
	return RegisterFactory(name, pFactory);
}

// o܂B
HRESULT AbstractARJSObject::InitMembers()
{
	HRESULT hr = DefineFunction("addSolidCube", 0);
	if(FAILED(hr)) {
		return hr;
	}

	hr = DefineFunction("addContainer", 0);
	if(FAILED(hr)) {
		return hr;
	}

	hr = DefineFunction("loadImage", 2);
	if(FAILED(hr)) {
		return hr;
	}

	hr = DefineFunction("addText", 0);
	if(FAILED(hr)) {
		return hr;
	}

	hr = DefineProperty("childNodes", true);
	if(FAILED(hr)) {
		return hr;
	}

	IArrayProperty* pChildNodes = NULL;
	hr = m_pOwner->GetChildren(& pChildNodes);
	if(FAILED(hr)) {
		return hr;
	}

	hr = m_childNodes.Attach(pChildNodes, m_pContext, this);
	if(FAILED(hr)) {
		return hr;
	}

	ARWindow* pWindow = NULL;
	hr = m_pOwner->GetOwner(& pWindow);
	if(FAILED(hr)) {
		return hr;
	}

	vector<pair<string, IARGObjectFactory*>> factories;
	hr = pWindow->GetObjectFactories(factories);
	if(FAILED(hr)) {
		return hr;
	}

	for(vector<pair<string, IARGObjectFactory*>>::iterator it = factories.begin(); it != factories.end(); it ++) {
		hr = RegisterFactory(it->first.c_str(), it->second);
		if(FAILED(hr)) {
			return hr;
		}
	}

	return JSNamedProperty::InitMembers();
}

// vpeB擾܂B
HRESULT AbstractARJSObject::GetProperty(JSContext* cx, JSObject* obj, const char* name, jsval *vp)
{
	if(name == NULL) {
		return E_POINTER;
	}
	string sname = name;
	if(sname == "childNodes") {
		if(vp == NULL) {
			return S_OK;
		}
		IJSObject* pObj = NULL;
		HRESULT hr = m_childNodes.GetJSObject(& pObj);
		if(FAILED(hr)) {
			return hr;
		}

		*vp = OBJECT_TO_JSVAL(pObj->GetCore());
		return S_OK;
	}else{
		return JSNamedProperty::GetProperty(cx, obj, name, vp);
	}
}

// ֐s܂B
HRESULT AbstractARJSObject::InvokeFunction(JSContext *cx, JSObject *obj, const char* name, uintN argc, jsval *argv, jsval *rval)
{
	string sname = name;

	if(sname == "addSolidCube") {
		return AddSolidCubeImpl(cx, obj, argc, argv, rval);
	}else if(sname == "addContainer") {
		return AddContainerImpl(cx, obj, argc, argv, rval);
	}else if(sname == "loadImage") {
		return LoadImageImpl(cx, obj, argc, argv, rval);
	}else if(sname == "addText") {
		return AddTextImpl(cx, obj, argc, argv, rval);
	}else{
		// Plugin֐̎s
		ARWindow* pWindow = NULL;
		HRESULT hr = m_pOwner->GetOwner(& pWindow);
		if(FAILED(hr)) {
			return hr;
		}

		vector<pair<string, IARGObjectFactory*>> factories;
		hr = pWindow->GetObjectFactories(factories);
		if(FAILED(hr)) {
			return hr;
		}

		for(vector<pair<string, IARGObjectFactory*>>::iterator it = factories.begin(); it != factories.end(); it ++) {
			if(sname == it->first) {
				return CreateObject(cx, obj, it->second, argc, argv, rval);
			}
		}

		return JSNamedProperty::InvokeFunction(cx, obj, name, argc, argv, rval);
	}
}

// IuWFNgt@Ngo^܂B
HRESULT AbstractARJSObject::RegisterFactory(const char* name, IARGObjectFactory* pFactory)
{
	if(name == NULL || pFactory == NULL) {
		return E_POINTER;
	}
	int candCount = 0;
	HRESULT hr = pFactory->GetParameterTypeCandidateCount(& candCount);
	if(FAILED(hr)) {
		return hr;
	}
	for(int i = 0; i < candCount; i ++) {
		int minArgs = 0;
		hr = pFactory->GetParameterTypeCount(i, & minArgs);
		if(FAILED(hr)) {
			return hr;
		}

		hr = DefineFunction(name, minArgs + 1);
		if(FAILED(hr)) {
			return hr;
		}
	}
	return S_OK;
}

// AddWorld֐̒`
HRESULT AbstractARJSObject::AddSolidCubeImpl(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	ARWindow* pWindow = NULL;
	HRESULT hr = m_pOwner->GetOwner(& pWindow);
	if(FAILED(hr)) {
		return hr;
	}

	ARJSSolidCube* pNewObj = new ARJSSolidCube(pWindow);
	if(pNewObj == NULL) {
		return E_OUTOFMEMORY;
	}

	int index = 0;
	hr = m_pOwner->AddChild(pNewObj, & index);
	if(FAILED(hr)) {
		return hr;
	}

	hr = pNewObj->Attach(m_pContext);
	if(FAILED(hr)) {
		return hr;
	}

	if(rval == NULL) {
		return S_OK;
	}
	*rval = OBJECT_TO_JSVAL(pNewObj->GetJSObject()->GetCore());
	return S_OK;
}

// AddContainer֐̒`
HRESULT AbstractARJSObject::AddContainerImpl(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	ARWindow* pWindow = NULL;
	HRESULT hr = m_pOwner->GetOwner(& pWindow);
	if(FAILED(hr)) {
		return hr;
	}

	ARJSContainer* pNewObj = new ARJSContainer(pWindow);
	if(pNewObj == NULL) {
		return E_OUTOFMEMORY;
	}

	int index = 0;
	hr = m_pOwner->AddChild(pNewObj, & index);
	if(FAILED(hr)) {
		return hr;
	}

	hr = pNewObj->Attach(m_pContext);
	if(FAILED(hr)) {
		return hr;
	}

	if(rval == NULL) {
		return S_OK;
	}
	*rval = OBJECT_TO_JSVAL(pNewObj->GetJSObject()->GetCore());
	return S_OK;
}

// AddWorld֐̒`
HRESULT AbstractARJSObject::LoadImageImpl(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	string type;
	HRESULT hr = GetValueToString(argv[0], type);
	if(FAILED(hr)) {
		return hr;
	}

	IImageLoader* pImageLoader = NULL;
	ARWindow* pWindow = NULL;
	hr = m_pOwner->GetOwner(& pWindow);
	if(FAILED(hr)) {
		return hr;
	}

	hr = pWindow->GetImageLoader(type.c_str(), & pImageLoader);
	if(FAILED(hr)) {
		return hr;
	}

	string file;
	hr = GetValueToString(argv[1], file);
	if(FAILED(hr)) {
		return hr;
	}
	double scale = 1.0;
	if(argc >= 3) {
		JSValueWrapper value(cx, argv[2]);
		hr = value.GetDouble(& scale);
		if(FAILED(hr)) {
			return hr;
		}
	}
	jsval callback = JSVAL_NULL;
	if(argc >= 4) {
		callback = argv[3];
	}

	ARJSImage* pNewObj = new ARJSImage(pWindow, pImageLoader);
	if(pNewObj == NULL) {
		return E_OUTOFMEMORY;
	}
	if(FAILED(hr)) {
		return hr;
	}

	int index = 0;
	hr = m_pOwner->AddChild(pNewObj, & index);
	if(FAILED(hr)) {
		return hr;
	}

	hr = pNewObj->Attach(m_pContext);
	if(FAILED(hr)) {
		return hr;
	}

	hr = pNewObj->Load(file.c_str(), scale, callback);
	if(FAILED(hr)) {
		return hr;
	}

	if(rval == NULL) {
		return S_OK;
	}
	*rval = OBJECT_TO_JSVAL(pNewObj->GetJSObject()->GetCore());
	return S_OK;
}

// AddText֐̒`
HRESULT AbstractARJSObject::AddTextImpl(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	string type = TEXT_LOADER_ENTRY;
	IImageLoader* pImageLoader = NULL;
	ARWindow* pWindow = NULL;
	HRESULT hr = m_pOwner->GetOwner(& pWindow);
	if(FAILED(hr)) {
		return hr;
	}

	hr = pWindow->GetImageLoader(type.c_str(), & pImageLoader);
	if(FAILED(hr)) {
		return hr;
	}

	double scale = 1.0;
	if(argc >= 1) {
		JSValueWrapper value(cx, argv[0]);
		hr = value.GetDouble(& scale);
		if(FAILED(hr)) {
			return hr;
		}
	}
	jsval callback = JSVAL_NULL;
	if(argc >= 2) {
		callback = argv[1];
	}

	ARJSImage* pNewObj = new ARJSImage(pWindow, pImageLoader);
	if(pNewObj == NULL) {
		return E_OUTOFMEMORY;
	}
	if(FAILED(hr)) {
		return hr;
	}

	int index = 0;
	hr = m_pOwner->AddChild(pNewObj, & index);
	if(FAILED(hr)) {
		return hr;
	}

	hr = pNewObj->Attach(m_pContext);
	if(FAILED(hr)) {
		return hr;
	}

	hr = pNewObj->Load("", scale, callback);
	if(FAILED(hr)) {
		return hr;
	}

	if(rval == NULL) {
		return S_OK;
	}
	*rval = OBJECT_TO_JSVAL(pNewObj->GetJSObject()->GetCore());
	return S_OK;
}

// vOCIuWFNgǉ܂B
HRESULT AbstractARJSObject::CreateObject(JSContext *cx, JSObject *obj, IARGObjectFactory* pFactory, uintN argc, jsval *argv, jsval *rval)
{
	if(cx == NULL || obj == NULL || pFactory == NULL) {
		return E_POINTER;
	}

	ARWindow* pWindow = NULL;
	HRESULT hr = m_pOwner->GetOwner(& pWindow);
	if(FAILED(hr)) {
		return hr;
	}

	int candCount = 0;
	hr = pFactory->GetParameterTypeCandidateCount(& candCount);
	if(FAILED(hr)) {
		return hr;
	}
	for(int i = 0; i < candCount; i ++) {
		int minArgs = 0;
		hr = pFactory->GetParameterTypeCount(i, & minArgs);
		if(FAILED(hr)) {
			return hr;
		}
		if(argc != minArgs + 1) {
			continue;
		}
		VARIANT* pArgs = NULL;
		if(argc > 1) {
			pArgs = new VARIANT[argc - 1];
			for(uintN j = 0; j < argc - 1; j ++) {
				VariantInit(pArgs + j);
			}

			for(uintN j = 0; j < argc - 1; j ++) {
				VARTYPE vt;
				hr = pFactory->GetParameterType(i, j, & vt);
				if(FAILED(hr)) {
					delete[] pArgs;
					pArgs = NULL;
				}

				hr = JSDispatch::ConvertJSVALtoVARIANT(m_pContext, argv[j], pArgs + j, vt);
				if(FAILED(hr)) {
					delete[] pArgs;
					pArgs = NULL;
				}
			}
		}
		hr = S_OK;
		ARJSPluginObject* pNewObj = new ARJSPluginObject(pWindow, pFactory, & hr);

		int index = 0;
		if(hr == S_OK) {
			hr = m_pOwner->AddChild(pNewObj, & index);
		}

		if(hr == S_OK) {
			hr = pNewObj->Attach(m_pContext);
		}

		if(hr == S_OK) {
			hr = pNewObj->Load(pArgs, argc - 1, argv[argc - 1]);
		}

		if(hr == S_OK) {
			*rval = OBJECT_TO_JSVAL(pNewObj->GetJSObject()->GetCore());
		}

		if(FAILED(hr)) {
			if(pArgs != NULL) {
				for(uintN j = 0; j < argc - 1; j ++) {
					VariantClear(pArgs + j);
				}
				delete[] pArgs;
				pArgs = NULL;
			}
		}
		return hr;
	}
	return E_NOTIMPL;
}
