#include "StdAfx.h"
#include "GLBitmapDump.h"
#include "../../common/ImageFactory.h"
#include "../../js/JSEngineContext.h"

// \z܂B
GLBitmapDump::GLBitmapDump(IWindow* pWindow, JSEngineContext* pContext, int bufferSize, HRESULT* phr)
: GLImageDump(pWindow, bufferSize, phr)
{
	m_pWindow = pWindow;
	m_pContext = pContext;
	m_bExit = false;
}

// j܂B
GLBitmapDump::~GLBitmapDump()
{
}

// obt@̓e܂B
HRESULT GLBitmapDump::Process(LONGLONG t, int width, int height, unsigned char* data)
{
	if(m_bExit) {
		return S_OK;
	}
	string savePath = "";
	bool bExit = false;
	HRESULT hr = GetSavePath(t, savePath, & bExit);
	if(FAILED(hr)) {
		return hr;
	}

	if(hr == S_OK) {
		TextureImage* pImage = new TextureImage(width, height, data);
		hr = ImageFactory::DumpBitmap(savePath.c_str(), pImage);
		delete pImage;
		pImage = NULL;
	}

	if(bExit) {
		m_bExit = true;
		ImageDumpRemover* pRemover = new ImageDumpRemover(m_pWindow, this);
		return m_pContext->InvokeAsync(pRemover);
	}
	return hr;
}

// \z܂B
GLBitmapDump::TextureImage::TextureImage(int width, int height, unsigned char* bgr32)
{
	m_pBGR32 = bgr32;
	m_iWidth = width;
	m_iHeight = height;
}

// j܂B
GLBitmapDump::TextureImage::~TextureImage()
{
}

// j܂B
void GLBitmapDump::TextureImage::Destroy()
{
	;
}

// 摜ŐV̏ԂɍXV܂B
HRESULT GLBitmapDump::TextureImage::Refresh()
{
	return E_NOTIMPL;
}

// 擾܂B
int GLBitmapDump::TextureImage::GetWidth()
{
	return m_iWidth;
}

// 擾܂B
int GLBitmapDump::TextureImage::GetHeight()
{
	return m_iHeight;
}

// Materialp̃At@l擾܂B
double GLBitmapDump::TextureImage::GetMaterialAlpha()
{
	return 1.0;
}

// sNZ̏擾܂B
HRESULT GLBitmapDump::TextureImage::GetPixel(int x, int y, COMMON_ARGB* pARGB)
{
	if(pARGB == NULL) {
		return E_POINTER;
	}
	unsigned char* pBGR32 = m_pBGR32 + (m_iHeight - y - 1) * (m_iWidth * 3) + x * 3;
	pARGB->r = pBGR32[0];
	pARGB->g = pBGR32[1];
	pARGB->b = pBGR32[2];
	pARGB->a = 0xff;
	return S_OK;
}

// Dirtyǂ𔻒肵܂B
HRESULT GLBitmapDump::TextureImage::IsDirty(bool* pDirty)
{
	return E_NOTIMPL;
}

// Dirtyǂݒ肵܂B
HRESULT GLBitmapDump::TextureImage::SetDirty(bool dirty)
{
	return E_NOTIMPL;
}

// vpeB擾܂B
HRESULT GLBitmapDump::TextureImage::GetProperty(INamedProperty** ppProp)
{
	return E_NOTIMPL;
}

// \z܂B
GLBitmapDump::ImageDumpRemover::ImageDumpRemover(IWindow* pWindow, GLBitmapDump* pDump)
{
	m_pWindow = pWindow;
	m_pDump = pDump;
}

// ܂B
GLBitmapDump::ImageDumpRemover::~ImageDumpRemover()
{
	m_pWindow = NULL;
	if(m_pDump != NULL) {
		m_pDump->Destroy();
		m_pDump = NULL;
	}
}

// j܂B
void GLBitmapDump::ImageDumpRemover::Destroy()
{
	delete this;
}

// s܂B
HRESULT GLBitmapDump::ImageDumpRemover::Invoke()
{
	return m_pWindow->RemoveGLHook(m_pDump);
}
