#include "StdAfx.h"
#include "SourceFactory.h"
#include "LocalSource.h"
#include "DiscPath.h"
#include "URLPath.h"

// fBXNpXǂ𔻒肵܂B
HRESULT SourceFactory::IsDiscPath(const char* path, bool* pResult)
{
	if(path == NULL || pResult == NULL) {
		return E_POINTER;
	}
	HRESULT hr = S_OK;
	DiscPath dpath(path, & hr);
	if(FAILED(hr)) {
		return hr;
	}
	return dpath.IsAbsolute(pResult);
}

// URLGR[hsȂ܂B
HRESULT SourceFactory::EncodeURL(const char* path, string& epath)
{
	if(path == NULL) {
		return E_POINTER;
	}
	HRESULT hr = S_OK;
	DiscPath dpath(path, & hr);
	if(FAILED(hr)) {
		return hr;
	}
	int count = 0;
	hr = dpath.GetPathElementCount(& count);
	if(FAILED(hr)) {
		return hr;
	}
	string result = "";
	for(int i = 0; i < count;  i++) {
		string elem = "";
		hr = dpath.GetPathElement(i, elem);
		if(FAILED(hr)) {
			return hr;
		}
		if(i > 0) {
			result += "/";
		}
		if(i == 0) {
			result += elem;
		}else{
			string delem;
			hr = URLPath::EncodeURLPath(elem.c_str(), delem);
			if(FAILED(hr)) {
				return hr;
			}
			result += delem;
		}
	}
	epath = result;
	return S_OK;
}

// URLfR[hsȂ܂B
HRESULT SourceFactory::DecodeURL(const char* path, string& epath)
{
	if(path == NULL) {
		return E_POINTER;
	}
	HRESULT hr = S_OK;
	DiscPath dpath(path, & hr);
	if(FAILED(hr)) {
		return hr;
	}
	int count = 0;
	hr = dpath.GetPathElementCount(& count);
	if(FAILED(hr)) {
		return hr;
	}
	string result = "";
	for(int i = 0; i < count;  i++) {
		string elem = "";
		hr = dpath.GetPathElement(i, elem);
		if(FAILED(hr)) {
			return hr;
		}
		if(i > 0) {
			result += "\\";
		}
		string delem;
		hr = URLPath::DecodeURLPath(elem.c_str(), delem);
		if(FAILED(hr)) {
			return hr;
		}
		result += delem;
	}
	epath = result;
	return S_OK;
}

// \z܂B
SourceFactory::SourceFactory(const char* basePath, HRESULT* phr)
{
	m_pDiscPath = NULL;
	*phr = S_OK;

	HRESULT hr = S_OK;
	m_pDiscPath = new DiscPath(basePath, & hr);
	if(FAILED(hr)) {
		*phr = hr;
		return;
	}
}

// j܂B
SourceFactory::~SourceFactory(void)
{
	if(m_pDiscPath != NULL) {
		delete m_pDiscPath;
		m_pDiscPath = NULL;
	}
}

// ܂B
HRESULT SourceFactory::Load(const char* path, ISource** ppSource)
{
	if(ppSource == NULL) {
		return E_POINTER;
	}
	if(m_pDiscPath == NULL) {
		return E_UNEXPECTED;
	}

	HRESULT hr = S_OK;
	string absoluteURL;
	hr = m_pDiscPath->GetURL(absoluteURL);
	if(FAILED(hr)) {
		return hr;
	}

	URLPath discPath(absoluteURL.c_str(), & hr);
	if(FAILED(hr)) {
		return hr;
	}

	string sresolvedPath;
	hr = discPath.GetAbsoluteURL(path, sresolvedPath);
	if(FAILED(hr)) {
		return hr;
	}

	URLPath resolvedPath(sresolvedPath.c_str(), & hr);
	if(FAILED(hr)) {
		return hr;
	}

	string realPath;
	hr = resolvedPath.GetRealPath(realPath);
	if(FAILED(hr)) {
		return hr;
	}

	ISource* pNewSource = new LocalSource(realPath.c_str(), & hr);
	if(pNewSource == NULL) {
		return E_OUTOFMEMORY;
	}
	if(FAILED(hr)) {
		pNewSource->Destroy();
		pNewSource = NULL;
		*ppSource = NULL;
		return hr;
	}

	*ppSource = pNewSource;
	return S_OK;
}