#include "StdAfx.h"
#include "URLPath.h"
#include "DiscPath.h"

// URLGR[hsȂ܂B
HRESULT URLPath::EncodeURLPath(const char* path, string& epath)
{
	if(path == NULL) {
		return E_POINTER;
	}
	string spath = path;
	epath = "";
	for(string::size_type i = 0; i < spath.length(); i++){
        char c = spath[i];

        if((c >= '0' && c <= '9')
        || (c >= 'A' && c <= 'Z')
        || (c >= 'a' && c <= 'z')
        || (c == '\'')
        || (c == '*')
        || (c == ')')
        || (c == '(')
        || (c == '-')
        || (c == '.')
        || (c == '_')){
			epath += c;
        }else if(c == ' '){
            epath += '+';
        }else{
            epath += '%';
			char urlEN[256];
			int value = (c & 0x00ff);
            sprintf_s(urlEN, 128, "%02X", value);
			epath += urlEN;
		}
    }

	return S_OK;
}

// URLfR[hsȂ܂B
HRESULT URLPath::DecodeURLPath(const char* path, string& epath)
{
	if(path == NULL) {
		return E_POINTER;
	}
	string spath = path;
	epath = "";
	while(spath.length() > 0){
		int val = 0;
		HRESULT hr = StartWithPerc(spath.c_str(), & val);
		if(FAILED(hr)) {
			return hr;
		}

        if(hr == S_FALSE){
			if(spath[0] == '+') {
				epath += ' ';
			}else{
				epath += spath[0];
			}
			spath = spath.substr(1);
        }else{
            epath += val;
			spath = spath.substr(3);
		}
    }

	return S_OK;
}

// %ŊJnĂ邩ǂ𔻒肵܂B
HRESULT URLPath::StartWithPerc(const char* path, int* pValue)
{
	if(path == NULL || pValue == NULL) {
		return E_POINTER;
	}
	string spath = path;
	if(spath.length() < 3) {
		return S_FALSE;
	}
	if(spath[0] != '%') {
		return S_FALSE;
	}
	int value = 0;
	for(int i = 0; i < 2; i ++) {
		char ch = spath[i + 1];
		value = value << 4;
		if('0' <= ch && ch <= '9') {
			value += ch - '0';
		}else if('a' <= ch && ch <= 'f') {
			value += ch - 'a' + 10;
		}else if('A' <= ch && ch <= 'F') {
			value += ch - 'A' + 10;
		}else{
			return S_FALSE;
		}
	}
	*pValue = value;
	return S_OK;
}

// \z܂B
URLPath::URLPath(const char* path, HRESULT* phr)
{
	m_iPortNumber = 0;
	*phr = S_OK;

	HRESULT hr = Parse(path);
	if(FAILED(hr)) {
		*phr = hr;
		return;
	}
}

// \z܂B
URLPath::URLPath(const char* scheme, const char* hostName, int portNumber, vector<string> pathElements, HRESULT* phr)
{
	m_iPortNumber = 0;
	*phr = S_OK;

	if(scheme != NULL) {
		m_scheme = scheme;
	}else{
		m_scheme = "";
	}
	if(hostName != NULL) {
		m_hostName = hostName;
	}else{
		m_hostName = "";
	}
	if(portNumber > 0) {
		m_iPortNumber = portNumber;
	}
	for(vector<string>::iterator it = pathElements.begin(); it != pathElements.end(); it++) {
		m_elements.push_back(*it);
	}
}

// j܂B
URLPath::~URLPath()
{
	;
}

// pXvf̐擾܂B
HRESULT URLPath::GetPathElementCount(int* pCount)
{
	if(pCount == NULL) {
		return E_POINTER;
	}
	*pCount = (int) m_elements.size();
	return S_OK;
}

// pXvf擾܂B
HRESULT URLPath::GetPathElement(int index, string& pathElem)
{
	if(index >= (int)m_elements.size()) {
		return E_INVALIDARG;
	}
	pathElem = m_elements[index];
	return S_OK;
}

// pX폜܂B
HRESULT URLPath::RemoveLastPathElement()
{
	if(m_elements.size() == 0) {
		return S_FALSE;
	}
	vector<string>::iterator it = m_elements.begin();
	int count = m_elements.size() - 1;
	m_elements.erase(it + count);
	return S_OK;
}

// pXɒǉ܂B
HRESULT URLPath::AddLastPathElement(const char* element)
{
	if(element == NULL) {
		return E_POINTER;
	}
	m_elements.push_back(element);
	return S_OK;
}

// pXɒǉ܂B
HRESULT URLPath::AddFirstPathElement(const char* element)
{
	if(element == NULL) {
		return E_POINTER;
	}
	m_elements.insert(m_elements.begin(), element);
	return S_OK;
}

// ΃pXǂ擾܂B
HRESULT URLPath::IsAbsolute(bool* pAbsolute)
{
	if(pAbsolute == NULL) {
		return E_POINTER;
	}

	if(m_scheme.size() > 0) {
		*pAbsolute = true;
	}else{
		*pAbsolute = false;
	}
	return S_OK;
}

// pX擾܂B
HRESULT URLPath::GetRealPath(string& path)
{
	if(m_scheme != "file") {
		return E_FAIL;
	}
	vector<string> elements;
	for(vector<string>::size_type i = 1; i < m_elements.size(); i ++) {
		string elem = m_elements[i];
		HRESULT hr = URLPath::DecodeURLPath(elem.c_str(), elem);
		if(FAILED(hr)) {
			return hr;
		}
		elements.push_back(elem);
	}

	HRESULT hr = S_OK;
	DiscPath discPath(elements, & hr);
	if(FAILED(hr)) {
		return hr;
	}

	return discPath.ToString(path);
}

// URL擾܂B
HRESULT URLPath::GetAbsoluteURL(const char* path, string& url)
{
	if(path == NULL) {
		return E_POINTER;
	}

	HRESULT hr = S_OK;
	URLPath targetPath(path, & hr);
	if(FAILED(hr)) {
		return hr;
	}

	bool bAbsolute = false;
	hr = targetPath.IsAbsolute(& bAbsolute);
	if(FAILED(hr)) {
		return hr;
	}
	if(bAbsolute == true) {
		return targetPath.ToString(url);
	}

	// pX̌
	const char* pHostName = NULL;
	if(m_hostName.length() > 0) {
		pHostName = m_hostName.c_str();
	}
	URLPath tempPath(m_scheme.c_str(), pHostName, m_iPortNumber, m_elements, & hr);
	if(FAILED(hr)) {
		return hr;
	}

	hr = tempPath.RemoveLastPathElement();
	if(FAILED(hr)) {
		return hr;
	}

	int pathCount = 0;
	hr = targetPath.GetPathElementCount(& pathCount);
	if(FAILED(hr)) {
		return hr;
	}
	for(int i = 0; i < pathCount; i ++) {
		string pathElem;
		hr = targetPath.GetPathElement(i, pathElem);
		if(FAILED(hr)) {
			return hr;
		}

		hr = tempPath.AddLastPathElement(pathElem.c_str());
		if(FAILED(hr)) {
			return hr;
		}
	}

	return tempPath.ToString(url);
}

// 񉻂܂B
HRESULT URLPath::ToString(string& path)
{
	string buf = "";
	if(m_scheme.size() > 0) {
		// URL
		buf += m_scheme;
		buf += "://";
		buf += m_hostName;
		if(m_iPortNumber > 0) {
			buf += ":";
			char sportNum[MAX_PATH + 1];
			sportNum[0] = '\0';
			sprintf_s(sportNum, MAX_PATH, "%d", m_iPortNumber);
			buf += sportNum;
		}
	}else{
		// pX̂
		;
	}
	int count = 0;

	for(vector<string>::iterator it = m_elements.begin();
		it != m_elements.end();
		it ++) {
			if(count > 0) {
				buf += "/";
			}
			buf += (*it);
			count ++;
	}

	path = buf;
	return S_OK;
}

// ͂܂B
HRESULT URLPath::Parse(const char* path)
{
	if(path == NULL) {
		return E_POINTER;
	}
	string spath = path;

	string::size_type index = spath.find("://");
	if(index != string::npos) {
		m_scheme = spath.substr(0, index);
		return ParseHostAndPath(path + (index + 3));
	}else{
		m_scheme = "";
		m_hostName = "";
		m_iPortNumber = 0;
		return ParsePath(path);
	}
}

// ͂܂B
HRESULT URLPath::ParseHostAndPath(const char* path)
{
	if(path == NULL) {
		return E_POINTER;
	}
	string spath = path;

	string::size_type slIndex = spath.find("/");
	if(slIndex == string::npos) {
		// pXȂ͔F߂Ȃ
		return E_INVALIDARG;
	}
	string hostPart = spath.substr(0, slIndex);

	string::size_type clIndex = hostPart.find(":");
	if(clIndex != string::npos) {
		m_hostName = hostPart.substr(0, clIndex);
		m_iPortNumber = atoi(hostPart.c_str() + (clIndex + 1));
	}else{
		m_hostName = hostPart;
		m_iPortNumber = 0;
	}
	return ParsePath(spath.c_str() + slIndex);
}

// ͂܂B
HRESULT URLPath::ParsePath(const char* path)
{
	if(path == NULL) {
		return E_POINTER;
	}
	string buf = "";
	string spath = path;

	for(string::iterator it = spath.begin(); it != spath.end(); it ++) {
		char ch = *it;

		if(ch == '/') {
			m_elements.push_back(buf);
			buf = "";
		}else{
			buf += ch;
		}
	}

	m_elements.push_back(buf);
	return S_OK;
}
