#include "StdAfx.h"
#include "TextLoader.h"
#include "JpegCore.h"
#include "../common/URLPath.h"

#define DEFAULT_WIDTH	64
#define DEFAULT_HEIGHT	64

// \z܂B
TextLoader::TextLoader(HWND hOwnerWnd)
{
	m_hOwnerWnd = hOwnerWnd;
}

// j܂B
TextLoader::~TextLoader()
{
	m_hOwnerWnd = NULL;
}

// j܂B
void TextLoader::Destroy()
{
	delete this;
}

// ftHgTCYݒ肵܂B
HRESULT TextLoader::SetDefaultSize(int width, int height)
{
	// ɉȂ
	return S_OK;
}

// 摜ǂݍ݂܂B
HRESULT TextLoader::Load(const char* path, IImageLoaderNotify* pNotify)
{
	if(path == NULL || pNotify == NULL) {
		return E_POINTER;
	}

	WorkerImpl* pWorkerImpl = new WorkerImpl(m_hOwnerWnd, pNotify);
	return m_workerThread.Start(pWorkerImpl);
}

// 摜ǂݍ݂܂B
HRESULT TextLoader::Load(ISource* pSource, IImageLoaderNotify* pNotify)
{
	return E_NOTIMPL;
}

// \z܂B
TextLoader::WorkerImpl::WorkerImpl(HWND hOwnerWnd, IImageLoaderNotify* pNotify)
{
	m_hOwnerWnd = hOwnerWnd;
	m_pNotify = pNotify;
}

// j܂B
TextLoader::WorkerImpl::~WorkerImpl()
{
	if(m_pNotify != NULL) {
		m_pNotify->Destroy();
		m_pNotify = NULL;
	}
	m_hOwnerWnd = NULL;
}

// j܂B
void TextLoader::WorkerImpl::Destroy()
{
	delete this;
}

// s܂B
HRESULT TextLoader::WorkerImpl::Invoke()
{
	return m_pNotify->NotifySuccess(new ImageImpl(m_hOwnerWnd));
}

// \z܂B
TextLoader::ImageImpl::ImageImpl(HWND hOwnerWnd)
: AbstractImage(DEFAULT_WIDTH, DEFAULT_HEIGHT)
{
	m_hOwnerWnd = hOwnerWnd;
	m_pBuf = NULL;
	m_iNextWidth = m_iWidth;
	m_iNextHeight = m_iHeight;
	m_bChanged = false;
	m_nextText = "";
	m_hFont = NULL;
	m_fontName = "lr SVbN";
	m_iFontHeightPx = 32;
	m_iFontWeight = FW_NORMAL;
	m_bFontItalic = false;
	ZeroMemory(& m_fontColor, sizeof(m_fontColor));
	ZeroMemory(& m_bgColor, sizeof(m_bgColor));
	m_bgColor.r = 1.0;
	m_bgColor.g = 1.0;
	m_bgColor.b = 1.0;
	m_hBrush = NULL;
}

// j܂B
TextLoader::ImageImpl::~ImageImpl()
{
	if(m_pBuf != NULL) {
		delete m_pBuf;
		m_pBuf = NULL;
	}
	m_hOwnerWnd = NULL;
	if(m_hFont != NULL) {
		DeleteObject(m_hFont);
		m_hFont = NULL;
	}
	if(m_hBrush != NULL) {
		DeleteObject(m_hBrush);
		m_hBrush = NULL;
	}
}

// j܂B
void TextLoader::ImageImpl::Destroy()
{
	delete this;
}

// 摜ŐV̏ԂɍXV܂B
HRESULT TextLoader::ImageImpl::Refresh()
{
	if(m_bChanged) {
		HRESULT hr = RefreshTexture();
		if(FAILED(hr)) {
			return hr;
		}
		m_bChanged = false;
	}
	return S_OK;
}

// sNZ擾܂B
HRESULT TextLoader::ImageImpl::GetPixel(int x, int y, COMMON_ARGB* pARGB)
{
	if(pARGB == NULL) {
		return E_POINTER;
	}
	if(m_pBuf == NULL) {
		// eȂ
		pARGB->a = 0;
		pARGB->r = 0;
		pARGB->g = 0;
		pARGB->b = 0;
		return S_OK;
	}

	int ry = m_iHeight - y - 1;
	unsigned char *pPixel = &(m_pBuf[(ry * m_iWidth + x) * 4]);
	pARGB->a = (unsigned char)(/*pPixel[3]*/ 0xff * m_dAlpha);
	pARGB->r = pPixel[2];
	pARGB->g = pPixel[1];
	pARGB->b = pPixel[0];
	return S_OK;
}

// vpeBIuWFNg𐶐܂B
AbstractImage::PropertyImpl* TextLoader::ImageImpl::CreateProperty()
{
	return new TextPropertyImpl(this);
}

// eNX`XV܂B
HRESULT TextLoader::ImageImpl::RefreshTexture()
{
	HDC hDC = GetDC(m_hOwnerWnd);

	HRESULT hr = RefreshFont(hDC);
	if(FAILED(hr)) {
	    ReleaseDC(m_hOwnerWnd, hDC);
		return S_OK;
	}

	hr = RefreshBrush(hDC);
	if(FAILED(hr)) {
	    ReleaseDC(m_hOwnerWnd, hDC);
		return S_OK;
	}

	RECT rc;
	ZeroMemory(& rc, sizeof(rc));
	rc.left = 0;
	rc.top = 0;
	rc.bottom = m_iNextHeight;
	rc.right = m_iNextWidth;

    void *pbits = NULL;
    BITMAPINFO bmi;
    ZeroMemory(&bmi.bmiHeader, sizeof(BITMAPINFOHEADER));
    bmi.bmiHeader.biSize      = sizeof(BITMAPINFOHEADER);
    bmi.bmiHeader.biWidth     = m_iNextWidth;
    bmi.bmiHeader.biHeight    = m_iNextHeight;
    bmi.bmiHeader.biPlanes    = 1;
    bmi.bmiHeader.biBitCount  = (WORD) GetDeviceCaps(hDC, BITSPIXEL);
    bmi.bmiHeader.biSizeImage = bmi.bmiHeader.biWidth * bmi.bmiHeader.biHeight * 4;

    HBITMAP hBitmap = CreateDIBSection(hDC, &bmi, DIB_RGB_COLORS, &pbits, NULL, 0);

    HDC hMem = CreateCompatibleDC(hDC);
    SelectObject(hMem, hBitmap);

    FillRect(hMem, &rc, m_hBrush);
    SetTextColor(hMem, GetColorRef(& m_fontColor));
    SetBkMode(hMem, TRANSPARENT);
	SelectObject(hMem, m_hFont);
    DrawTextA(hMem, m_nextText.c_str(), -1, &rc, DT_SINGLELINE|DT_CENTER|DT_VCENTER);

	// f[^̃Rs[
	if(m_pBuf != NULL) {
		delete m_pBuf;
		m_pBuf = NULL;
	}

	m_pBuf = new unsigned char[bmi.bmiHeader.biSizeImage];
	if(m_pBuf == NULL) {
		return E_OUTOFMEMORY;
	}

	memcpy_s(m_pBuf, bmi.bmiHeader.biSizeImage, pbits, bmi.bmiHeader.biSizeImage);

    DeleteDC(hMem);
    ReleaseDC(m_hOwnerWnd, hDC);
    DeleteObject(hBitmap);

	m_iWidth = m_iNextWidth;
	m_iHeight = m_iNextHeight;
	m_bChanged = false;

    return S_OK;
}

// tHgXV܂B
HRESULT TextLoader::ImageImpl::RefreshFont(HDC hDC)
{
	if(m_hFont != NULL) {
		DeleteObject(m_hFont);
		m_hFont = NULL;
	}
	DWORD bItalic = 0;
	if(m_bFontItalic) {
		bItalic = 1;
	}
	m_hFont = CreateFontA(MulDiv(m_iFontHeightPx,-::GetDeviceCaps(hDC,LOGPIXELSY),72),
			0,0,0,m_iFontWeight,bItalic,0,0,SHIFTJIS_CHARSET,
			OUT_STROKE_PRECIS,CLIP_STROKE_PRECIS,DRAFT_QUALITY,
			VARIABLE_PITCH | FF_SWISS,m_fontName.c_str());
	if(m_hFont == NULL) {
		return E_FAIL;
	}
	return S_OK;
}

// uVXV܂B
HRESULT TextLoader::ImageImpl::RefreshBrush(HDC hDC)
{
	if(m_hBrush != NULL) {
		DeleteObject(m_hBrush);
		m_hBrush = NULL;
	}
	m_hBrush = CreateSolidBrush(GetColorRef(& m_bgColor));
	if(m_hBrush == NULL) {
		return E_FAIL;
	}
	return S_OK;
}

// Fϊ܂B
COLORREF TextLoader::ImageImpl::GetColorRef(const TEXT_COLOR* pColor)
{
	int r = (int)(0xff * (pColor->r));
	int g = (int)(0xff * (pColor->g));
	int b = (int)(0xff * (pColor->b));

	return RGB(r, g, b);
}

// \z܂B
TextLoader::ImageImpl::TextPropertyImpl::TextPropertyImpl(ImageImpl* pOwner)
: PropertyImpl(pOwner)
{
	m_pOwner = pOwner;
}

// j܂B
TextLoader::ImageImpl::TextPropertyImpl::~TextPropertyImpl()
{
	m_pOwner = NULL;
}

// j܂B
void TextLoader::ImageImpl::TextPropertyImpl::Destroy()
{
	delete this;
}

// ܂B
HRESULT TextLoader::ImageImpl::TextPropertyImpl::Init()
{
	HRESULT hr = S_OK;

	hr = DefineProperty("width", PROP_INT);
	if(FAILED(hr)) {
		return hr;
	}

	hr = DefineProperty("height", PROP_INT);
	if(FAILED(hr)) {
		return hr;
	}

	hr = DefineProperty("text", PROP_STRING);
	if(FAILED(hr)) {
		return hr;
	}

	hr = DefineProperty("font.height", PROP_INT);
	if(FAILED(hr)) {
		return hr;
	}

	hr = DefineProperty("font.name", PROP_STRING);
	if(FAILED(hr)) {
		return hr;
	}

	hr = DefineProperty("font.weight", PROP_STRING);
	if(FAILED(hr)) {
		return hr;
	}

	hr = DefineProperty("font.italic", PROP_BOOL);
	if(FAILED(hr)) {
		return hr;
	}

	hr = DefineProperty("font.color.r", PROP_DOUBLE);
	if(FAILED(hr)) {
		return hr;
	}

	hr = DefineProperty("font.color.g", PROP_DOUBLE);
	if(FAILED(hr)) {
		return hr;
	}

	hr = DefineProperty("font.color.b", PROP_DOUBLE);
	if(FAILED(hr)) {
		return hr;
	}

	hr = DefineProperty("background.color.r", PROP_DOUBLE);
	if(FAILED(hr)) {
		return hr;
	}

	hr = DefineProperty("background.color.g", PROP_DOUBLE);
	if(FAILED(hr)) {
		return hr;
	}

	hr = DefineProperty("background.color.b", PROP_DOUBLE);
	if(FAILED(hr)) {
		return hr;
	}

	return PropertyImpl::Init();
}

// int̃vpeB擾܂B
HRESULT TextLoader::ImageImpl::TextPropertyImpl::GetIntProperty(const char* name, int* pValue)
{
	if(name == NULL || pValue == NULL) {
		return E_POINTER;
	}
	string sname = name;
	if(sname == "width") {
		*pValue = m_pOwner->m_iNextWidth;
		return S_OK;
	}else if(sname == "height") {
		*pValue = m_pOwner->m_iNextHeight;
		return S_OK;
	}else if(sname == "font.height") {
		*pValue = m_pOwner->m_iFontHeightPx;
		return S_OK;
	}else{
		return PropertyImpl::GetIntProperty(name, pValue);
	}
}

// int̃vpeBݒ肵܂B
HRESULT TextLoader::ImageImpl::TextPropertyImpl::SetIntProperty(const char* name, int value)
{
	if(name == NULL) {
		return E_POINTER;
	}
	string sname = name;
	if(sname == "width") {
		m_pOwner->m_iNextWidth = value;
		m_pOwner->SetDirty(true);
		m_pOwner->m_bChanged = true;
		return S_OK;
	}else if(sname == "height") {
		m_pOwner->m_iNextHeight = value;
		m_pOwner->SetDirty(true);
		m_pOwner->m_bChanged = true;
		return S_OK;
	}else if(sname == "font.height") {
		m_pOwner->m_iFontHeightPx = value;
		m_pOwner->SetDirty(true);
		m_pOwner->m_bChanged = true;
		return S_OK;
	}else{
		return PropertyImpl::SetIntProperty(name, value);
	}
}

// string̃vpeB擾܂B
HRESULT TextLoader::ImageImpl::TextPropertyImpl::GetStringProperty(const char* name, string& value)
{
	if(name == NULL) {
		return E_POINTER;
	}
	string sname = name;
	if(sname == "text") {
		value = m_pOwner->m_nextText;
		return S_OK;
	}else if(sname == "font.name") {
		value = m_pOwner->m_fontName;
		return S_OK;
	}else if(sname == "font.weight") {
		if(m_pOwner->m_iFontWeight == FW_NORMAL) {
			value = "normal";
		}else if(m_pOwner->m_iFontWeight == FW_BOLD) {
			value = "bold";
		}
		return S_OK;
	}else{
		return PropertyImpl::GetStringProperty(name, value);
	}
}

// string̃vpeBݒ肵܂B
HRESULT TextLoader::ImageImpl::TextPropertyImpl::SetStringProperty(const char* name, const char* value)
{
	if(name == NULL || value == NULL) {
		return E_POINTER;
	}
	string sname = name;
	if(sname == "text") {
		m_pOwner->m_nextText = value;
		m_pOwner->SetDirty(true);
		m_pOwner->m_bChanged = true;
		return S_OK;
	}else if(sname == "font.name") {
		m_pOwner->m_fontName = value;
		m_pOwner->SetDirty(true);
		m_pOwner->m_bChanged = true;
		return S_OK;
	}else if(sname == "font.weight") {
		if(value == "normal") {
			m_pOwner->m_iFontWeight = FW_NORMAL;
		}else if(value == "bold") {
			m_pOwner->m_iFontWeight = FW_BOLD;
		}
		m_pOwner->SetDirty(true);
		m_pOwner->m_bChanged = true;
		return S_OK;
	}else{
		return PropertyImpl::SetStringProperty(name, value);
	}
}

// ̃vpeB擾܂B
HRESULT TextLoader::ImageImpl::TextPropertyImpl::GetDoubleProperty(const char* name, double* pValue)
{
	if(name == NULL || pValue == NULL) {
		return E_POINTER;
	}
	string sname = name;
	if(sname == "font.color.r") {
		*pValue = m_pOwner->m_fontColor.r;
		return S_OK;
	}else if(sname == "font.color.g") {
		*pValue = m_pOwner->m_fontColor.g;
		return S_OK;
	}else if(sname == "font.color.b") {
		*pValue = m_pOwner->m_fontColor.b;
		return S_OK;
	}else if(sname == "background.color.r") {
		*pValue = m_pOwner->m_bgColor.r;
		return S_OK;
	}else if(sname == "background.color.g") {
		*pValue = m_pOwner->m_bgColor.g;
		return S_OK;
	}else if(sname == "background.color.b") {
		*pValue = m_pOwner->m_bgColor.b;
		return S_OK;
	}else{
		return PropertyImpl::GetDoubleProperty(name, pValue);
	}
}

// ̃vpeBݒ肵܂B
HRESULT TextLoader::ImageImpl::TextPropertyImpl::SetDoubleProperty(const char* name, double value)
{
	if(name == NULL) {
		return E_POINTER;
	}
	string sname = name;
	if(sname == "font.color.r") {
		m_pOwner->m_fontColor.r = value;
		m_pOwner->SetDirty(true);
		m_pOwner->m_bChanged = true;
		return S_OK;
	}else if(sname == "font.color.g") {
		m_pOwner->m_fontColor.g = value;
		m_pOwner->SetDirty(true);
		m_pOwner->m_bChanged = true;
		return S_OK;
	}else if(sname == "font.color.b") {
		m_pOwner->m_fontColor.b = value;
		m_pOwner->SetDirty(true);
		m_pOwner->m_bChanged = true;
		return S_OK;
	}else if(sname == "background.color.r") {
		m_pOwner->m_bgColor.r = value;
		m_pOwner->SetDirty(true);
		m_pOwner->m_bChanged = true;
		return S_OK;
	}else if(sname == "background.color.g") {
		m_pOwner->m_bgColor.g = value;
		m_pOwner->SetDirty(true);
		m_pOwner->m_bChanged = true;
		return S_OK;
	}else if(sname == "background.color.b") {
		m_pOwner->m_bgColor.b = value;
		m_pOwner->SetDirty(true);
		m_pOwner->m_bChanged = true;
		return S_OK;
	}else{
		return PropertyImpl::SetDoubleProperty(name, value);
	}
}

// bool̃vpeB擾܂B
HRESULT TextLoader::ImageImpl::TextPropertyImpl::GetBoolProperty(const char* name, bool* pValue)
{
	if(name == NULL || pValue == NULL) {
		return E_POINTER;
	}
	string sname = name;
	if(sname == "font.italic") {
		*pValue = m_pOwner->m_bFontItalic;
		return S_OK;
	}else{
		return PropertyImpl::GetBoolProperty(name, pValue);
	}
}

// bool̃vpeBݒ肵܂B
HRESULT TextLoader::ImageImpl::TextPropertyImpl::SetBoolProperty(const char* name, bool value)
{
	if(name == NULL) {
		return E_POINTER;
	}
	string sname = name;
	if(sname == "font.italic") {
		m_pOwner->m_bFontItalic = value;
		m_pOwner->SetDirty(true);
		m_pOwner->m_bChanged = true;
		return S_OK;
	}else{
		return PropertyImpl::SetBoolProperty(name, value);
	}
}
