#include "StdAfx.h"
#include "JSEngineContext.h"
#include "IJSCode.h"
#include "IJSObject.h"
#include "IJSFunction.h"
#include "JSGlobalObject.h"
#include "../common/CritSection.h"
#include "../common/URLPath.h"

// O[oIuWFNg̗p
JSClass g_globalClass = {
	"global",0,
	JS_PropertyStub,JS_PropertyStub,JS_PropertyStub,JS_PropertyStub,
	JS_EnumerateStub,JS_ResolveStub,JS_ConvertStub,JS_FinalizeStub
};

// O[oIuWFNg̗p
JSClass g_embeddedClass = {
	"embbeded",JSCLASS_HAS_PRIVATE,
	JS_PropertyStub,JS_PropertyStub,JS_PropertyStub,JS_PropertyStub,
	JS_EnumerateStub,JS_ResolveStub,JS_ConvertStub,JS_FinalizeStub
};

// \z܂B
JSEngineContext::JSEngineContext(JSRuntime* pRuntime, size_t stackChunkSize, HRESULT* phr)
{
	m_pContext = NULL;
	m_pGlobal = NULL;
	m_pBaseURL = NULL;
	*phr = S_OK;

	// 
	*phr = Init(pRuntime, stackChunkSize);
}

// j܂B
JSEngineContext::~JSEngineContext(void)
{
	if(m_pGlobal != NULL) {
		m_pGlobal->Destroy();
		m_pGlobal = NULL;
	}
	if(m_pContext != NULL) {
		JS_DestroyContext(m_pContext);
		m_pContext = NULL;
	}
	DestroyQueue(m_asyncQueue);
	if(m_pBaseURL != NULL) {
		delete m_pBaseURL;
		m_pBaseURL = NULL;
	}
}

// ̃IuWFNgj܂B
void JSEngineContext::Destroy()
{
	delete this;
}

// ʒmIuWFNgݒ肵܂B
HRESULT JSEngineContext::SetNotify(IJSContextNotify* pNotify)
{
	if(m_pGlobal == NULL) {
		return E_UNEXPECTED;
	}
	return m_pGlobal->SetNotify(pNotify);
}

// R[hs܂B
HRESULT JSEngineContext::Evaluate(IJSCode* pCode)
{
	if(pCode == NULL) {
		return E_POINTER;
	}
	if(m_pContext == NULL || m_pGlobal == NULL) {
		return E_UNEXPECTED;
	}

	HRESULT hr = S_OK;
	string code;
	hr = pCode->GetText(code);
	if(FAILED(hr)) {
		return hr;
	}

	string filename;
	hr = pCode->GetFilename(filename);
	if(FAILED(hr)) {
		return hr;
	}

	unsigned int lineNum = pCode->GetLineNumber();

	jsval rval;
	JSBool bret = JS_EvaluateScript(m_pContext, m_pGlobal->GetCore(), code.c_str(), code.size(), filename.c_str(), lineNum, & rval);
	if(bret == JS_FALSE) {
		return E_FAIL;
	}

	JS_GC(m_pContext);

	return S_OK;
}

// VIuWFNg𐶐܂B
HRESULT JSEngineContext::CreateNewObject(JSObject** ppObject)
{
	if(ppObject == NULL) {
		return E_POINTER;
	}
	JSObject* pObj = JS_NewObject(m_pContext, & g_embeddedClass, NULL, NULL);
	if(pObj == NULL) {
		return E_OUTOFMEMORY;
	}
	*ppObject = pObj;
	return S_OK;
}

// SpiderMonkeỹRAIuWFNg擾܂B
JSContext* JSEngineContext::GetCore()
{
	return m_pContext;
}

// O[ovpeB`܂B
HRESULT JSEngineContext::DefineGlobalProperty(const char* name, IJSObject* pObject)
{
	if(m_pGlobal == NULL) {
		return E_UNEXPECTED;
	}

	return m_pGlobal->DefineProperty(name, pObject);
}

// ֐𓯊s܂B
HRESULT JSEngineContext::InvokeSync(jsval function, IJSObject* pThis, jsval* argv, int argc, jsval* rval)
{
	if(argv == NULL) {
		return E_POINTER;
	}

	/*if(JS_TypeOfValue(m_pContext, function) != JSTYPE_FUNCTION) {
		return S_FALSE;
	}

	JSFunction* pFunction = JS_ValueToFunction(m_pContext, function);
	if(pFunction == NULL) {
		return E_FAIL;
	}*/

	JSBool bret = JS_CallFunctionValue(m_pContext, pThis->GetCore(), function, argc, argv, rval);
	if(bret == JS_FALSE) {
		return E_FAIL;
	}

	JS_GC(m_pContext);
	return S_OK;
}

// O[oIuWFNg擾܂B
HRESULT JSEngineContext::GetGlobalObject(IJSObject** ppObject)
{
	if(m_pGlobal == NULL) {
		return E_UNEXPECTED;
	}
	if(ppObject == NULL) {
		return E_POINTER;
	}

	*ppObject = m_pGlobal;
	return S_OK;
}

// \񂵂܂B
HRESULT JSEngineContext::InvokeAsync(IJSFunction* pFunction)
{
	if(pFunction == NULL) {
		return E_POINTER;
	}
	return PushQueue(pFunction);
}

// \񂳂Ă鏈s܂B
HRESULT JSEngineContext::ProcessAsync()
{
	queue<IJSFunction*> exec;

	HRESULT hr = PopQueue(exec);
	if(FAILED(hr)) {
		return hr;
	}

	hr = S_OK;
	while(exec.size() > 0) {
		IJSFunction* pFunc = exec.front();
		exec.pop();

		HRESULT lhr = pFunc->Invoke();
		if(FAILED(lhr)) {
			hr = lhr;
		}

		pFunc->Destroy();
		pFunc = NULL;
	}
	return hr;
}

// URL擾܂B
HRESULT JSEngineContext::GetAbsoluteURL(const char* path, string& url)
{
	if(path == NULL) {
		return E_POINTER;
	}
	if(m_pBaseURL == NULL) {
		return E_UNEXPECTED;
	}

	return m_pBaseURL->GetAbsoluteURL(path, url);
}

// {ƂȂURLݒ肵܂B
HRESULT JSEngineContext::SetBaseURL(const char* baseURL)
{
	if(baseURL == NULL) {
		return E_POINTER;
	}

	HRESULT hr = S_OK;
	URLPath* pBaseURL = new URLPath(baseURL, & hr);
	if(FAILED(hr)) {
		delete pBaseURL;
		pBaseURL = NULL;
		return hr;
	}
	if(m_pBaseURL != NULL) {
		delete m_pBaseURL;
		m_pBaseURL = NULL;
	}
	m_pBaseURL = pBaseURL;

	return S_OK;
}

// {ƂȂURL擾܂B
HRESULT JSEngineContext::GetBaseURL(string& url)
{
	if(m_pBaseURL == NULL) {
		return E_FAIL;
	}
	return m_pBaseURL->ToString(url);
}

// ܂B
HRESULT JSEngineContext::Init(JSRuntime* pRuntime, size_t stackChunkSize)
{
	m_pContext = JS_NewContext(pRuntime, stackChunkSize);
	if(m_pContext == NULL) {
		return E_FAIL;
	}

	JS_SetContextPrivate(m_pContext, this);
	JS_SetErrorReporter(m_pContext, JSEngineContext::ErrorReporter);

	JSObject* pGlobalCore = JS_NewObject(m_pContext, & g_globalClass, NULL, NULL);
	if(pGlobalCore == NULL) {
		return E_OUTOFMEMORY;
	}

	JSBool bret = JS_InitStandardClasses(m_pContext, pGlobalCore);
	if(bret == JS_FALSE) {
		return E_FAIL;
	}

	m_pGlobal = new JSGlobalObject();
	if(m_pGlobal == NULL) {
		return E_OUTOFMEMORY;
	}

	return m_pGlobal->Attach(this, pGlobalCore);
}

// G[ʒm܂B
void JSEngineContext::NotifyError(const char* message, JSErrorReport* pReport)
{
	HRESULT hr = E_FAIL;
	if(m_pGlobal != NULL) {
		hr = m_pGlobal->NotifyError(message);
		if(!(FAILED(hr))) {
			return;
		}
	}

	// G[ʒmɂs
	printf("FATAL(NotifyError): %x\n", hr);
}

// L[o܂B
HRESULT JSEngineContext::PopQueue(queue<IJSFunction*>& queue)
{
	CritSection crit(& m_mutex);
	while(m_asyncQueue.size() > 0) {
		IJSFunction* pFunc = m_asyncQueue.front();
		m_asyncQueue.pop();

		queue.push(pFunc);
	}
	return S_OK;
}

// L[Ɋ֐ǉ܂B
HRESULT JSEngineContext::PushQueue(IJSFunction* pFunction)
{
	CritSection crit(& m_mutex);
	m_asyncQueue.push(pFunction);

	return S_OK;
}

// L[̃IuWFNgׂč폜܂B
HRESULT JSEngineContext::DestroyQueue(queue<IJSFunction*>& queue)
{
	while(queue.size() > 0) {
		IJSFunction* pFunc = queue.front();
		queue.pop();
		pFunc->Destroy();
		pFunc = NULL;
	}
	return S_OK;
}

// G[ʒm܂B
void JSEngineContext::ErrorReporter(JSContext* pContext, const char* message, JSErrorReport* pReport)
{
	JSEngineContext* pThis = GET_JS_ENGINE_CONTEXT(pContext);
	pThis->NotifyError(message, pReport);
}
