#include "StdAfx.h"
#include "JSMemberCollection.h"

// \z܂B
JSMemberCollection::JSMemberCollection(const char* className)
{
	m_className = className;
	m_pJSContext = NULL;
	m_pJSObject = NULL;
}

// j܂B
JSMemberCollection::~JSMemberCollection()
{
	m_pJSContext = NULL;
	m_pJSObject = NULL;
}

// ܂B
HRESULT JSMemberCollection::Init(JSContext* pContext, JSObject* pObject)
{
	if(pContext == NULL || pObject == NULL) {
		return E_POINTER;
	}

	m_pJSContext = pContext;
	m_pJSObject = pObject;
	return InitMembers();
}

// ꏊ擾܂B
void JSMemberCollection::GetLocation(JSContext* cx, const char* className, const char* implName, jsval idval, string& location)
{
	location += m_className;
	location += "::";

	if(implName != NULL) {
		string sid = "(unknown)";
		if(idval != JSVAL_NULL) {
			HRESULT hr = GetValueToString(idval, sid);
			if(hr == S_OK) {
				location += "[id=";
				location += sid;
				location += "]";
			}
		}

		string simplName = implName;
		string funcName = "IFunctionName_";
		if(simplName == "GetPropImpl") {
			location += "Get(";
			location += sid;
			location += ")";
		}else if(simplName == "SetPropImpl") {
			location += "Set(";
			location += sid;
			location += ")";
		}else if(simplName.find(funcName) != string::npos) {
			string sindex = simplName.substr(funcName.size());
			int index = atoi(sindex.c_str());
			if(index < (int)m_functionNames.size() && index >= 0) {
				location += m_functionNames[index];
			}else{
				location += simplName;
			}
		}else{
			location += simplName;
		}
	}else{
		location += "(unknown)";
	}
}

// vpeBݒ肵܂B
HRESULT JSMemberCollection::SetProperty(JSContext *cx, JSObject *obj, const char* name, jsval vp)
{
	return S_FALSE;
}

// vpeB擾܂B
HRESULT JSMemberCollection::GetProperty(JSContext* cx, JSObject* obj, const char* name, jsval *vp)
{
	*vp = JSVAL_NULL;
	return S_FALSE;
}

// ֐s܂B
HRESULT JSMemberCollection::InvokeFunction(JSContext *cx, JSObject *obj, const char* name, uintN argc, jsval *argv, jsval *rval)
{
	*rval = JSVAL_NULL;
	return S_FALSE;
}

// vpeB`܂B
HRESULT JSMemberCollection::DefineProperty(const char* name, bool readonly)
{
	if(name == NULL) {
		return E_POINTER;
	}
	if(m_pJSContext == NULL || m_pJSObject == NULL) {
		return E_UNEXPECTED;
	}

	uintN attrs = 0;
	if(readonly) {
		attrs = JSPROP_READONLY;
	}

	JSBool bret = JS_DefineProperty(m_pJSContext, m_pJSObject, name, JSVAL_NULL, 
									JSOW_PROPERTY_REF(JSMemberCollection,SGetProp), 
									JSOW_PROPERTY_REF(JSMemberCollection,SSetProp), attrs);
	if(bret == JS_FALSE) {
		return E_FAIL;
	}

	return S_OK;
}

// ֐`܂B
HRESULT JSMemberCollection::DefineFunction(const char* name, int minArgs)
{
	if(name == NULL) {
		return E_POINTER;
	}
	if(m_pJSContext == NULL || m_pJSObject == NULL) {
		return E_UNEXPECTED;
	}

	JSNative pFunctionRef = NULL;
	HRESULT hr = GetNativeRef(name, & pFunctionRef);
	if(FAILED(hr)) {
		return hr;
	}

	JSFunction* pFunction = JS_DefineFunction(m_pJSContext, m_pJSObject, name, pFunctionRef, minArgs, 0);
	if(pFunction == NULL) {
		return E_FAIL;
	}
	return S_OK;
}

// vpeB`B
JSOW_PROPERTY_DEF(JSMemberCollection, SSetProp, SetPropImpl, SGetProp, GetPropImpl);

// vpeBݒ肵܂B
HRESULT JSMemberCollection::SetPropImpl(JSContext *cx, JSObject *obj, jsval idval, jsval *vp)
{
	if(vp == NULL) {
		return S_FALSE;
	}

	string name;
	HRESULT hr = GetValueToString(idval, name);
	if(FAILED(hr)) {
		return hr;
	}

	return SetProperty(cx, obj, name.c_str(), *vp);
}

// vpeB擾܂B
HRESULT JSMemberCollection::GetPropImpl(JSContext *cx, JSObject *obj, jsval idval, jsval *vp)
{
	if(vp == NULL) {
		return S_FALSE;
	}

	string name;
	HRESULT hr = GetValueToString(idval, name);
	if(FAILED(hr)) {
		return hr;
	}

	return GetProperty(cx, obj, name.c_str(), vp);
}

// JSNativeւ̎QƂ擾܂B
HRESULT JSMemberCollection::GetNativeRef(const char* name, JSNative* pFuncRef)
{
	if(name == NULL || pFuncRef == NULL) {
		return E_POINTER;
	}
	UINT index = m_functionNames.size();
	if(index >= MAX_NATIVE) {
		return E_UNEXPECTED;
	}

	string sname = name;
	m_functionNames.push_back(sname);

	*pFuncRef = m_nativeRefs[index];
	return S_OK;
}

// ֐s܂B
HRESULT JSMemberCollection::InvokeFunc(JSContext *cx, JSObject *obj, UINT index, uintN argc, jsval *argv, jsval *rval)
{
	if(index >= m_functionNames.size()) {
		return E_INVALIDARG;
	}

	string name = m_functionNames[index];

	return InvokeFunction(cx, obj, name.c_str(), argc, argv, rval);
}

// JSNative`܂B
JSMEM_FUNCTION_DEF(0, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(1, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(2, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(3, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(4, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(5, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(6, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(7, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(8, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(9, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(10, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(11, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(12, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(13, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(14, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(15, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(16, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(17, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(18, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(19, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(20, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(21, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(22, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(23, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(24, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(25, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(26, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(27, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(28, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(29, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(30, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(31, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(32, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(33, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(34, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(35, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(36, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(37, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(38, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(39, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(40, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(41, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(42, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(43, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(44, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(45, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(46, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(47, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(48, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(49, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(50, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(51, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(52, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(53, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(54, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(55, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(56, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(57, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(58, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(59, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(60, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(61, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(62, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(63, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(64, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(65, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(66, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(67, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(68, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(69, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(70, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(71, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(72, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(73, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(74, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(75, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(76, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(77, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(78, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(79, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(80, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(81, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(82, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(83, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(84, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(85, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(86, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(87, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(88, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(89, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(90, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(91, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(92, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(93, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(94, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(95, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(96, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(97, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(98, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);
JSMEM_FUNCTION_DEF(99, JSMemberCollection, SInvokeFunc, InvokeFuncImpl);

// ֐Xg̐錾
JSNative JSMemberCollection::m_nativeRefs[MAX_NATIVE] = {
	JSMEM_FUNCTION_INIT(0, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(1, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(2, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(3, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(4, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(5, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(6, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(7, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(8, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(9, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(10, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(11, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(12, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(13, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(14, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(15, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(16, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(17, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(18, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(19, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(20, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(21, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(22, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(23, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(24, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(25, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(26, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(27, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(28, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(29, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(30, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(31, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(32, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(33, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(34, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(35, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(36, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(37, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(38, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(39, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(40, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(41, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(42, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(43, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(44, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(45, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(46, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(47, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(48, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(49, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(50, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(51, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(52, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(53, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(54, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(55, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(56, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(57, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(58, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(59, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(60, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(61, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(62, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(63, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(64, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(65, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(66, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(67, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(68, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(69, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(70, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(71, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(72, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(73, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(74, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(75, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(76, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(77, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(78, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(79, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(80, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(81, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(82, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(83, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(84, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(85, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(86, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(87, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(88, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(89, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(90, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(91, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(92, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(93, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(94, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(95, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(96, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(97, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(98, JSMemberCollection, SInvokeFunc, InvokeFuncImpl),
	JSMEM_FUNCTION_INIT(99, JSMemberCollection, SInvokeFunc, InvokeFuncImpl)
	};

