#include "StdAfx.h"
#include "JSPropertyWrapper.h"
#include "JSValueWrapper.h"

// \z܂B
JSNamedProperty::JSNamedProperty(INamedProperty* pProp)
: JSMemberCollection("JSNamedProperty")
{
	m_prefix = "";
	m_pProp = pProp;
}

// \z܂B
JSNamedProperty::JSNamedProperty(const char* prefix, INamedProperty* pProp)
: JSMemberCollection("JSNamedProperty")
{
	m_prefix = prefix;
	m_pProp = pProp;
}

// j܂B
JSNamedProperty::~JSNamedProperty()
{
	for(map<string,JSNamedProperty*>::iterator it = m_props.begin();
		it != m_props.end();
		it ++) {
			JSNamedProperty* pProp = it->second;
			pProp->Destroy();
			pProp = NULL;
	}
	m_props.clear();
	m_pProp = NULL;
}

// j܂B
void JSNamedProperty::Destroy()
{
	delete this;
}

// v~eBuȒlݒ肵܂B
HRESULT JSNamedProperty::SetPrimitiveValue(JSContext* cx, JSObject* obj, IPropertyValue* pTarget, jsval val)
{
	if(cx == NULL || obj == NULL || pTarget == NULL) {
		return E_POINTER;
	}

	PROPERTY_DESC desc;
	ZeroMemory(& desc, sizeof(desc));

	HRESULT hr = pTarget->GetPropertyDesc(& desc);
	if(FAILED(hr)) {
		return hr;
	}

	if(desc.type == PROP_DOUBLE) {
		JSValueWrapper valuew(cx, val);

		double value = 0.0;
		hr = valuew.GetDouble(& value);
		if(FAILED(hr)) {
			return hr;
		}

		return pTarget->SetDoubleProperty(value);
	}else if(desc.type == PROP_BOOL) {
		JSValueWrapper valuew(cx, val);

		bool value = false;
		hr = valuew.GetBool(& value);
		if(FAILED(hr)) {
			return hr;
		}

		return pTarget->SetBoolProperty(value);
	}else if(desc.type == PROP_INT) {
		JSValueWrapper valuew(cx, val);

		int value = 0;
		hr = valuew.GetInt(& value);
		if(FAILED(hr)) {
			return hr;
		}

		return pTarget->SetIntProperty(value);
	}else if(desc.type == PROP_STRING) {
		JSValueWrapper valuew(cx, val);

		string value;
		hr = valuew.GetString(value);
		if(FAILED(hr)) {
			return hr;
		}

		return pTarget->SetStringProperty(value.c_str());
	}else if(desc.type == PROP_JSOBJECT) {
		return E_NOTIMPL;
	}else{
		return E_NOTIMPL;
	}
}

// v~eBuȒl擾܂B
HRESULT JSNamedProperty::GetPrimitiveValue(JSContext* cx, JSObject* obj, IPropertyValue* pTarget, jsval* pval)
{
	if(cx == NULL || obj == NULL || pTarget == NULL || pval == NULL) {
		return E_POINTER;
	}
	PROPERTY_DESC desc;
	ZeroMemory(& desc, sizeof(desc));

	HRESULT hr = pTarget->GetPropertyDesc(& desc);
	if(FAILED(hr)) {
		return hr;
	}

	if(desc.type == PROP_DOUBLE) {
		double value = 0.0;
		hr = pTarget->GetDoubleProperty(& value);
		if(FAILED(hr)) {
			return hr;
		}

		JSValueWrapper valuew(cx, JSVAL_NULL);
		hr = valuew.SetDouble(value);
		if(FAILED(hr)) {
			return hr;
		}

		return valuew.Get(pval);
	}else if(desc.type == PROP_BOOL) {
		bool value = false;
		hr = pTarget->GetBoolProperty(& value);
		if(FAILED(hr)) {
			return hr;
		}

		JSValueWrapper valuew(cx, JSVAL_NULL);
		hr = valuew.SetBool(value);
		if(FAILED(hr)) {
			return hr;
		}

		return valuew.Get(pval);
	}else if(desc.type == PROP_INT) {
		int value = 0;
		hr = pTarget->GetIntProperty(& value);
		if(FAILED(hr)) {
			return hr;
		}

		JSValueWrapper valuew(cx, JSVAL_NULL);
		hr = valuew.SetInt(value);
		if(FAILED(hr)) {
			return hr;
		}

		return valuew.Get(pval);
	}else if(desc.type == PROP_STRING) {
		string value;
		hr = pTarget->GetStringProperty(value);
		if(FAILED(hr)) {
			return hr;
		}

		JSValueWrapper valuew(cx, JSVAL_NULL);
		hr = valuew.SetString(value.c_str());
		if(FAILED(hr)) {
			return hr;
		}

		return valuew.Get(pval);
	}else if(desc.type == PROP_JSOBJECT) {
		IJSObject* pValue = NULL;
		hr = pTarget->GetJSObject(& pValue);
		if(FAILED(hr)) {
			return hr;
		}

		if(pValue == NULL) {
			*pval = JSVAL_NULL;
		}else{
			*pval = OBJECT_TO_JSVAL(pValue->GetCore());
		}

		return S_OK;
	}else{
		return E_NOTIMPL;
	}
}

// o܂B
HRESULT JSNamedProperty::InitMembers()
{
	if(m_pProp == NULL) {
		return E_UNEXPECTED;
	}
	int count = 0;
	HRESULT hr = m_pProp->GetPropertyCount(& count);
	if(FAILED(hr)) {
		return hr;
	}

	for(int i = 0; i < count; i ++) {
		string name;
		hr = m_pProp->GetPropertyName(i, name);
		if(FAILED(hr)) {
			return hr;
		}

		string propName;
		bool bPrimitive = false;
		hr = ParseName(name.c_str(), propName, & bPrimitive);
		if(FAILED(hr)) {
			return hr;
		}
		if(hr == S_FALSE) {
			continue;
		}
		if(bPrimitive == false) {
			// IuWFNg^Cv
			if(m_props.count(propName) > 0) {
				continue;
			}

			string propFullName = m_prefix;
			if(propFullName.size() > 0) {
				propFullName += ".";
			}
			propFullName += propName;
			JSNamedProperty* pChild = new JSNamedProperty(propFullName.c_str(), m_pProp);

			m_props[propName] = pChild;

			hr = pChild->Attach(m_pContext, NULL);
			if(FAILED(hr)) {
				return hr;
			}
		}

		hr = DefineProperty(propName.c_str(), false);
		if(FAILED(hr)) {
			return hr;
		}
	}
	return S_OK;
}

// vpeBݒ肵܂B
HRESULT JSNamedProperty::SetProperty(JSContext *cx, JSObject *obj, const char* name, jsval vp)
{
	if(name == NULL) {
		return E_POINTER;
	}
	if(m_pProp == NULL) {
		return E_UNEXPECTED;
	}
	if(m_props.count(name) > 0) {
		JSNamedProperty* pProp = m_props[name];
		JSObject* pObj = NULL;
		JSBool bret = JS_ValueToObject(cx, vp, & pObj);
		if(bret == JS_FALSE) {
			return E_FAIL;
		}
		return pProp->SetValue(cx, pObj);
	}else{
		string fullName = m_prefix;
		if(fullName.size() > 0) {
			fullName += ".";
		}
		fullName += name;

		IPropertyValue* pValue = NULL;
		HRESULT hr = m_pProp->GetPropertyValue(fullName.c_str(), & pValue);
		if(FAILED(hr)) {
			return hr;
		}
		if(pValue == NULL) {
			return E_FAIL;
		}

		hr = JSNamedProperty::SetPrimitiveValue(cx, obj, pValue, vp);
		pValue->Release();
		pValue = NULL;

		return hr;
	}
}

// vpeB擾܂B
HRESULT JSNamedProperty::GetProperty(JSContext* cx, JSObject* obj, const char* name, jsval *vp)
{
	if(name == NULL || vp == NULL) {
		return E_POINTER;
	}
	if(m_pProp == NULL) {
		return E_UNEXPECTED;
	}

	if(m_props.count(name) > 0) {
		JSNamedProperty* pProp = m_props[name];
		*vp = OBJECT_TO_JSVAL(pProp->GetCore());
		return S_OK;
	}else{
		string fullName = m_prefix;
		if(fullName.size() > 0) {
			fullName += ".";
		}
		fullName += name;

		IPropertyValue* pValue = NULL;
		HRESULT hr = m_pProp->GetPropertyValue(fullName.c_str(), & pValue);
		if(FAILED(hr)) {
			return hr;
		}
		if(pValue == NULL) {
			return E_FAIL;
		}

		hr = JSNamedProperty::GetPrimitiveValue(cx, obj, pValue, vp);
		pValue->Release();
		pValue = NULL;

		return hr;
	}
}

// O͂܂B
HRESULT JSNamedProperty::ParseName(const char* fullName, string& propName, bool* pPrimitive)
{
	if(fullName == NULL || pPrimitive == NULL) {
		return E_POINTER;
	}

	string sfullName = fullName;
	if(m_prefix.size() > 0) {
		string prefix = m_prefix + ".";
		string::size_type prefixPos = sfullName.find(prefix);
		if(prefixPos != 0) {
			return S_FALSE;
		}
		sfullName = sfullName.substr(prefix.size());
	}
	string::size_type dotPos = sfullName.find(".");
	if(dotPos == string::npos) {
		*pPrimitive = true;
		propName = sfullName;
	}else{
		*pPrimitive = false;
		propName = sfullName.substr(0, dotPos);
	}
	return S_OK;
}

// IuWFNgݒ肵܂B
HRESULT JSNamedProperty::SetValue(JSContext* cx, JSObject* pObj)
{
	if(cx == NULL || pObj == NULL) {
		return E_POINTER;
	}

	JSIdArray* pIds = JS_Enumerate(cx, pObj);
	if(pIds == NULL) {
		return E_FAIL;
	}

	for(int i = 0; i < pIds->length; i ++) {
		jsid id = pIds->vector[i];
		jsval idv = JSVAL_NULL;

		JSBool bret = JS_IdToValue(cx, id, & idv);
		if(bret == JS_FALSE) {
			JS_DestroyIdArray(cx, pIds);
			pIds = NULL;
			return E_FAIL;
		}

		string sid;
		HRESULT hr = GetValueToString(idv, sid);
		if(FAILED(hr)) {
			JS_DestroyIdArray(cx, pIds);
			pIds = NULL;
			return hr;
		}

		jsval propv = JSVAL_NULL;
		bret = JS_GetProperty(cx, pObj, sid.c_str(), & propv);
		if(bret == JS_FALSE) {
			JS_DestroyIdArray(cx, pIds);
			pIds = NULL;
			return E_FAIL;
		}

		hr = SetProperty(cx, GetCore(), sid.c_str(), propv);
		if(FAILED(hr)) {
			JS_DestroyIdArray(cx, pIds);
			pIds = NULL;
			return hr;
		}
	}
	JS_DestroyIdArray(cx, pIds);
	pIds = NULL;
	return S_OK;
}

// O[oIuWFNg̗p
JSClass g_arrayClass = {
	"arArray",JSCLASS_HAS_PRIVATE,
	JS_PropertyStub,JS_PropertyStub,JSOW_PROPERTY_REF(JSArrayProperty, SGetProp),JSOW_PROPERTY_REF(JSArrayProperty, SSetProp),
	JS_EnumerateStub,JS_ResolveStub,JS_ConvertStub,JS_FinalizeStub
};

// \z܂B
JSArrayProperty::JSArrayProperty(IArrayProperty* pProp)
{
	m_pProp = pProp;
}

// j܂B
JSArrayProperty::~JSArrayProperty()
{
	m_pProp = NULL;
}

// j܂B
void JSArrayProperty::Destroy()
{
	delete this;
}

// JavaScriptGWƃIuWFNgݒ肵܂B
HRESULT JSArrayProperty::Attach(JSEngineContext* pContext, JSObject* pObject)
{
	if(pContext == NULL) {
		return E_POINTER;
	}

	if(pObject == NULL) {
		JSObject* pNewObj = JS_NewObject(pContext->GetCore(), & g_arrayClass, NULL, NULL);
		if(pNewObj == NULL) {
			return E_FAIL;
		}
		pObject = pNewObj;
	}

	return JSObjectWrapper::Attach(pContext, pObject, true);
}

/**
* vpeB擾p̃\bh錾B
*/
JSOW_PROPERTY_DEF(JSArrayProperty, SSetProp, SetPropImpl, SGetProp, GetPropImpl);

// vpeBݒ肵܂B
HRESULT JSArrayProperty::SetPropImpl(JSContext *cx, JSObject *obj, jsval idval, jsval* vp)
{
	if(vp == NULL) {
		return S_OK;
	}
	if(m_pProp == NULL) {
		return E_UNEXPECTED;
	}
	int target = 0;
	HRESULT hr = GetTarget(cx, idval, & target);
	if(FAILED(hr)) {
		return hr;
	}

	if(target < 0) {
		// length
		return E_FAIL;
	}else{
		IPropertyValue* pValue = NULL;
		HRESULT hr = m_pProp->GetPropertyValue(target, & pValue);
		if(FAILED(hr)) {
			return hr;
		}
		if(pValue == NULL) {
			return E_FAIL;
		}

		hr = JSNamedProperty::SetPrimitiveValue(cx, obj, pValue, *vp);
		pValue->Release();
		pValue = NULL;

		return hr;
	}
}

// vpeB擾܂B
HRESULT JSArrayProperty::GetPropImpl(JSContext* cx, JSObject* obj, jsval idval, jsval *vp)
{
	if(vp == NULL) {
		return S_OK;
	}
	if(m_pProp == NULL) {
		return E_UNEXPECTED;
	}

	int target = 0;
	HRESULT hr = GetTarget(cx, idval, & target);
	if(FAILED(hr)) {
		return hr;
	}

	if(target < 0) {
		int count = 0;
		HRESULT hr = m_pProp->GetPropertyCount(& count);
		if(FAILED(hr)) {
			return hr;
		}

		JSValueWrapper valuew(cx, JSVAL_NULL);

		hr = valuew.SetInt(count);
		if(FAILED(hr)) {
			return hr;
		}

		return valuew.Get(vp);
	}else{
		IPropertyValue* pValue = NULL;
		HRESULT hr = m_pProp->GetPropertyValue(target, & pValue);
		if(FAILED(hr)) {
			return hr;
		}
		if(pValue == NULL) {
			return E_FAIL;
		}

		hr = JSNamedProperty::GetPrimitiveValue(cx, obj, pValue, vp);
		pValue->Release();
		pValue = NULL;

		return hr;
	}
}

// o܂B
HRESULT JSArrayProperty::Init(JSContext* cx, JSObject* obj)
{
	return S_OK;
}

// Ώۂ擾܂B
HRESULT JSArrayProperty::GetTarget(JSContext* cx, jsval idval, int* pTarget)
{
	if(pTarget == NULL) {
		return E_POINTER;
	}

	string sid;
	HRESULT hr = GetValueToString(idval, sid);
	if(hr == S_OK && sid == "length") {
		*pTarget = -1;
		return S_OK;
	}

	JSValueWrapper valuew(cx, idval);

	return valuew.GetInt(pTarget);
}

// \z܂B
JSArrayRef::JSArrayRef()
{
	m_pProp = NULL;
	m_pJSProp = NULL;
}

// j܂B
JSArrayRef::~JSArrayRef()
{
	if(m_pJSProp != NULL) {
		m_pJSProp->Destroy();
		m_pJSProp = NULL;
	}
	m_pProp = NULL;
}

// ݒ肵܂B
HRESULT JSArrayRef::Attach(IArrayProperty* pProp, JSEngineContext* pContext, IJSObject* pOwner)
{
	m_pProp = pProp;
	m_pJSProp = new JSArrayProperty(pProp);
	if(m_pJSProp == NULL) {
		return E_OUTOFMEMORY;
	}

	HRESULT hr = m_pJSProp->Attach(pContext, NULL);
	if(FAILED(hr)) {
		return hr;
	}

	return S_OK;
}

// JSIuWFNg擾܂B
HRESULT JSArrayRef::GetJSObject(IJSObject** ppObj)
{
	if(ppObj == NULL) {
		return E_POINTER;
	}
	if(m_pJSProp == NULL) {
		return E_UNEXPECTED;
	}

	*ppObj = m_pJSProp;
	return S_OK;
}
