#include "StdAfx.h"
#include "CalcUtil.h"
#include <math.h>

// [ɏ܂B
HRESULT CalcUtil::ToZero(MMD_MATRIX* pMatrix)
{
	if(pMatrix == NULL) {
		return E_POINTER;
	}
	for(int j = 0; j < 4; j ++) {
		for(int i = 0; i < 4; i ++) {
			pMatrix->values[j][i] = 0.0f;
		}
	}
	return S_OK;
}

// [ɏ܂B
HRESULT CalcUtil::ToZero(MMD_VECTOR3* pVector)
{
	if(pVector == NULL) {
		return E_POINTER;
	}
	pVector->x = pVector->y = pVector->z = 0.0f;
	return S_OK;
}

// [ɏ܂B
HRESULT CalcUtil::ToZero(MMD_VECTOR4* pVector)
{
	if(pVector == NULL) {
		return E_POINTER;
	}
	pVector->x = pVector->y = pVector->z = pVector->w = 0.0f;
	return S_OK;
}

// Pʍs𐶐܂B
HRESULT CalcUtil::GenerateIdentity(MMD_MATRIX* pMatrix)
{
	if(pMatrix == NULL) {
		return E_POINTER;
	}
	ToZero(pMatrix);
	pMatrix->values[0][0] = 1.0f;
	pMatrix->values[1][1] = 1.0f;
	pMatrix->values[2][2] = 1.0f;
	pMatrix->values[3][3] = 1.0f;
	return S_OK;
}

// Z܂B
HRESULT CalcUtil::Subtract(MMD_VECTOR3* pDest, const MMD_VECTOR3* pValue1, const MMD_VECTOR3* pValue2)
{
	if(pDest == NULL || pValue1 == NULL || pValue2 == NULL) {
		return E_POINTER;
	}
	MMD_VECTOR3 temp;
	temp.x = pValue1->x - pValue2->x;
	temp.y = pValue1->y - pValue2->y;
	temp.z = pValue1->z - pValue2->z;

	pDest->x = temp.x;
	pDest->y = temp.y;
	pDest->z = temp.z;
	return S_OK;
}

// Z܂B
HRESULT CalcUtil::Add(MMD_VECTOR3* pDest, const MMD_VECTOR3* pValue1, const MMD_VECTOR3* pValue2)
{
	if(pDest == NULL || pValue1 == NULL || pValue2 == NULL) {
		return E_POINTER;
	}
	MMD_VECTOR3 temp;
	temp.x = pValue1->x + pValue2->x;
	temp.y = pValue1->y + pValue2->y;
	temp.z = pValue1->z + pValue2->z;

	pDest->x = temp.x;
	pDest->y = temp.y;
	pDest->z = temp.z;
	return S_OK;
}

// Z܂B
HRESULT CalcUtil::Multiply(MMD_MATRIX* pDest, const MMD_MATRIX* pValue1, const MMD_MATRIX* pValue2)
{
	if(pDest == NULL || pValue1 == NULL || pValue2 == NULL) {
		return E_POINTER;
	}
	GLfloat matTemp[4][4];
	
	for(int i = 0; i < 4; i ++)
	{
		matTemp[i][0] =	pValue1->values[i][0] * pValue2->values[0][0] + pValue1->values[i][1] * pValue2->values[1][0] + pValue1->values[i][2] * pValue2->values[2][0] + pValue1->values[i][3] * pValue2->values[3][0];
		matTemp[i][1] =	pValue1->values[i][0] * pValue2->values[0][1] + pValue1->values[i][1] * pValue2->values[1][1] + pValue1->values[i][2] * pValue2->values[2][1] + pValue1->values[i][3] * pValue2->values[3][1];
		matTemp[i][2] =	pValue1->values[i][0] * pValue2->values[0][2] + pValue1->values[i][1] * pValue2->values[1][2] + pValue1->values[i][2] * pValue2->values[2][2] + pValue1->values[i][3] * pValue2->values[3][2];
		matTemp[i][3] =	pValue1->values[i][0] * pValue2->values[0][3] + pValue1->values[i][1] * pValue2->values[1][3] + pValue1->values[i][2] * pValue2->values[2][3] + pValue1->values[i][3] * pValue2->values[3][3];
	}

	for(int i = 0; i < 4; i ++)
	{
		pDest->values[i][0] = matTemp[i][0];
		pDest->values[i][1] = matTemp[i][1];
		pDest->values[i][2] = matTemp[i][2];
		pDest->values[i][3] = matTemp[i][3];
	}
	return S_OK;
}

// Z܂B
HRESULT CalcUtil::Multiply(MMD_VECTOR4* pDest, const MMD_VECTOR4* pValue1, const MMD_VECTOR4* pValue2)
{
	if(pDest == NULL || pValue1 == NULL || pValue2 == NULL) {
		return E_POINTER;
	}
	float	px, py, pz, pw;
	float	qx, qy, qz, qw;

	px = pValue1->x; py = pValue1->y; pz = pValue1->z; pw = pValue1->w;
	qx = pValue2->x; qy = pValue2->y; qz = pValue2->z; qw = pValue2->w;

	pDest->x = pw * qx + px * qw + py * qz - pz * qy;
	pDest->y = pw * qy - px * qz + py * qw + pz * qx;
	pDest->z = pw * qz + px * qy - py * qx + pz * qw;
	pDest->w = pw * qw - px * qx - py * qy - pz * qz;
	return S_OK;
}

// K܂B
HRESULT CalcUtil::Normalize(MMD_VECTOR3* pDest, const MMD_VECTOR3* pSource)
{
	if(pDest == NULL || pSource == NULL) {
		return E_POINTER;
	}
	float fSqr = 1.0f / sqrtf( pSource->x * pSource->x + pSource->y * pSource->y + pSource->z * pSource->z );

	pDest->x = pSource->x * fSqr;
	pDest->y = pSource->y * fSqr;
	pDest->z = pSource->z * fSqr;
	return S_OK;
}

// K܂B
HRESULT CalcUtil::Normalize(MMD_VECTOR4* pDest, const MMD_VECTOR4* pSource)
{
	if(pDest == NULL || pSource == NULL) {
		return E_POINTER;
	}

	float fSqr = 1.0f / sqrtf( pSource->x * pSource->x + pSource->y * pSource->y + pSource->z * pSource->z + pSource->w * pSource->w );

	pDest->x = pSource->x * fSqr;
	pDest->y = pSource->y * fSqr;
	pDest->z = pSource->z * fSqr;
	pDest->w = pSource->w * fSqr;
	return S_OK;
}

// Ԃ܂B
HRESULT CalcUtil::Lerp(MMD_MATRIX* pDest, const MMD_MATRIX* pValue1, const MMD_MATRIX* pValue2, float weight)
{
	if(pDest == NULL || pValue1 == NULL || pValue2 == NULL) {
		return E_POINTER;
	}
	float *fOut = (float *)(pDest->values);
	float *fSrc1 = (float *)(pValue1->values);
	float *fSrc2 = (float *)(pValue2->values);
	float rev = 1.0f - weight;

	for(int i = 0; i < 4*4; i ++) {
		*fOut++ = *fSrc1++ * weight + *fSrc2++ * rev;
	}
	return S_OK;
}

// Ԃ܂B
HRESULT CalcUtil::Lerp(MMD_VECTOR4* pDest, const MMD_VECTOR4* pValue1, const MMD_VECTOR4* pValue2, float weight)
{
	if(pDest == NULL || pValue1 == NULL || pValue2 == NULL) {
		return E_POINTER;
	}

	MMD_VECTOR4 temp;
	float	qr = pValue1->x * pValue2->x + pValue1->y * pValue2->y + pValue1->z * pValue2->z + pValue1->w * pValue2->w;
	float	t0 = 1.0f - weight;

	if( qr < 0 )
	{
		temp.x = pValue1->x * t0 - pValue2->x * weight;
		temp.y = pValue1->y * t0 - pValue2->y * weight;
		temp.z = pValue1->z * t0 - pValue2->z * weight;
		temp.w = pValue1->w * t0 - pValue2->w * weight;
	}
	else
	{
		temp.x = pValue1->x * t0 + pValue2->x * weight;
		temp.y = pValue1->y * t0 + pValue2->y * weight;
		temp.z = pValue1->z * t0 + pValue2->z * weight;
		temp.w = pValue1->w * t0 + pValue2->w * weight;
	}

	return Normalize(pDest, & temp);
}

// Ԃ܂B
HRESULT CalcUtil::Lerp(MMD_VECTOR3* pDest, const MMD_VECTOR3* pValue1, const MMD_VECTOR3* pValue2, float weight)
{
	if(pDest == NULL || pValue1 == NULL || pValue2 == NULL) {
		return E_POINTER;
	}
	float	t0 = 1.0f - weight;

	pDest->x = pValue1->x * t0 + pValue2->x * weight;
	pDest->y = pValue1->y * t0 + pValue2->y * weight;
	pDest->z = pValue1->z * t0 + pValue2->z * weight;
	return S_OK;
}

/**
* tsvZ܂B
*/
HRESULT CalcUtil::Inverse(MMD_MATRIX* pDest, const MMD_MATRIX* pSource)
{
	if(pDest == NULL || pSource == NULL) {
		return E_POINTER;
	}
	MMD_MATRIX matTemp;
	memcpy(& matTemp, pSource, sizeof(MMD_MATRIX));

	HRESULT hr = GenerateIdentity(pDest);
	if(FAILED(hr)) {
		return hr;
	}

	for( int i = 0 ; i < 4 ; i++ )
	{
		float	buf = 1 / matTemp.values[i][i];
		for( int j = 0 ; j < 4 ; j++ )
		{
			matTemp.values[i][j] *= buf;
			pDest->values[i][j] *= buf;
		}
		for( int j = 0 ; j < 4 ; j++ )
		{
			if( i != j )
			{
				buf = matTemp.values[j][i];
				for( int k = 0 ; k < 4 ; k++ )
				{
					matTemp.values[j][k] -= matTemp.values[i][k] * buf;
					pDest->values[j][k] -= pDest->values[i][k] * buf;
				}
			}
		}
	}
	return S_OK;
}

// ςvZ܂B
HRESULT CalcUtil::DotProduct(float* dp, const MMD_VECTOR3 *pValue1, const MMD_VECTOR3 *pValue2)
{
	if(dp == NULL || pValue1 == NULL || pValue2 == NULL) {
		return E_POINTER;
	}
	*dp = (pValue1->x * pValue2->x + pValue1->y * pValue2->y + pValue1->z * pValue2->z);
	return S_OK;
}

// OςvZ܂B
HRESULT CalcUtil::CrossProduct(MMD_VECTOR3* pDest, const MMD_VECTOR3* pValue1, const MMD_VECTOR3* pValue2)
{
	if(pDest == NULL || pValue1 == NULL || pValue2 == NULL) {
		return E_POINTER;
	}
	pDest->x = pValue1->y * pValue2->z - pValue1->z * pValue2->y;
	pDest->y = pValue1->z * pValue2->x - pValue1->x * pValue2->z;
	pDest->z = pValue1->x * pValue2->y - pValue1->y * pValue2->x;
	return S_OK;
}

// NH[^jIsɕϊ܂B
HRESULT CalcUtil::QuaternionToMatrix(MMD_MATRIX* pDest, const MMD_VECTOR4* pQuat)
{
	if(pDest == NULL || pQuat == NULL) {
		return E_POINTER;
	}
	float	x2 = pQuat->x * pQuat->x * 2.0f;
	float	y2 = pQuat->y * pQuat->y * 2.0f;
	float	z2 = pQuat->z * pQuat->z * 2.0f;
	float	xy = pQuat->x * pQuat->y * 2.0f;
	float	yz = pQuat->y * pQuat->z * 2.0f;
	float	zx = pQuat->z * pQuat->x * 2.0f;
	float	xw = pQuat->x * pQuat->w * 2.0f;
	float	yw = pQuat->y * pQuat->w * 2.0f;
	float	zw = pQuat->z * pQuat->w * 2.0f;

	pDest->values[0][0] = 1.0f - y2 - z2;
	pDest->values[0][1] = xy + zw;
	pDest->values[0][2] = zx - yw;
	pDest->values[1][0] = xy - zw;
	pDest->values[1][1] = 1.0f - z2 - x2;
	pDest->values[1][2] = yz + xw;
	pDest->values[2][0] = zx + yw;
	pDest->values[2][1] = yz - xw;
	pDest->values[2][2] = 1.0f - x2 - y2;

	pDest->values[0][3] = pDest->values[1][3] = pDest->values[2][3] = pDest->values[3][0] = pDest->values[3][1] = pDest->values[3][2] = 0.0f;
	pDest->values[3][3] = 1.0f;
	return S_OK;
}

// sɂϊsȂ܂B
HRESULT CalcUtil::Transform(MMD_VECTOR3* pDest, const MMD_VECTOR3* pSource, const MMD_MATRIX* pMatrix)
{
	if(pDest == NULL || pSource == NULL || pMatrix == NULL) {
		return E_POINTER;
	}
	MMD_VECTOR3 temp;
	HRESULT hr = Rotate(& temp, pSource, pMatrix);
	if(FAILED(hr)) {
		return hr;
	}

	pDest->x = temp.x + pMatrix->values[3][0];
	pDest->y = temp.y + pMatrix->values[3][1];
	pDest->z = temp.z + pMatrix->values[3][2];
	return S_OK;
}

// sɂ]sȂ܂B
HRESULT CalcUtil::Rotate(MMD_VECTOR3* pDest, const MMD_VECTOR3* pSource, const MMD_MATRIX* pMatrix)
{
	if(pDest == NULL || pSource == NULL || pMatrix == NULL) {
		return E_POINTER;
	}
	MMD_VECTOR3 temp;
	temp.x = pSource->x * pMatrix->values[0][0] + pSource->y * pMatrix->values[1][0] + pSource->z * pMatrix->values[2][0];
	temp.y = pSource->x * pMatrix->values[0][1] + pSource->y * pMatrix->values[1][1] + pSource->z * pMatrix->values[2][1];
	temp.z = pSource->x * pMatrix->values[0][2] + pSource->y * pMatrix->values[1][2] + pSource->z * pMatrix->values[2][2];

	pDest->x = temp.x;
	pDest->y = temp.y;
	pDest->z = temp.z;

	return S_OK;
}

// NH[^jI𐶐܂B
HRESULT CalcUtil::CreateAxis(MMD_VECTOR4 *pDest, const MMD_VECTOR3 *pAxis, float rotAngle)
{
	if(pDest == NULL || pAxis == NULL) {
		return E_POINTER;
	}
	if( fabsf(rotAngle) < 0.0001f ) {
		pDest->x = pDest->y = pDest->z = 0.0f;
		pDest->w = 1.0f;
	} else {
		rotAngle *= 0.5f;
		float	fTemp = sinf(rotAngle);

		pDest->x = pAxis->x * fTemp;
		pDest->y = pAxis->y * fTemp;
		pDest->z = pAxis->z * fTemp;
		pDest->w = cosf(rotAngle);
	}
	return S_OK;
}

// ]̎擾sȂ܂B
HRESULT CalcUtil::ToEuler(MMD_VECTOR3* pAngle, const MMD_VECTOR4* pQuat)
{
	if(pAngle == NULL || pQuat == NULL) {
		return E_POINTER;
	}
	float	x2 = pQuat->x + pQuat->x;
	float	y2 = pQuat->y + pQuat->y;
	float	z2 = pQuat->z + pQuat->z;
	float	xz2 = pQuat->x * z2;
	float	wy2 = pQuat->w * y2;
	float	temp = -(xz2 - wy2);

	if( temp >= 1.f ){
		temp = 1.f;
	}else if( temp <= -1.f ){
		temp = -1.f;
	}

	float	yRadian = asinf(temp);

	float	xx2 = pQuat->x * x2;
	float	xy2 = pQuat->x * y2;
	float	zz2 = pQuat->z * z2;
	float	wz2 = pQuat->w * z2;

	if( yRadian < 3.1415926f * 0.5f )
	{
		if( yRadian > -3.1415926f * 0.5f ) {
			float	yz2 = pQuat->y * z2;
			float	wx2 = pQuat->w * x2;
			float	yy2 = pQuat->y * y2;
			pAngle->x = atan2f( (yz2 + wx2), (1.f - (xx2 + yy2)) );
			pAngle->y = yRadian;
			pAngle->z = atan2f( (xy2 + wz2), (1.f - (yy2 + zz2)) );
		} else {
			pAngle->x = -atan2f( (xy2 - wz2), (1.f - (xx2 + zz2)) );
			pAngle->y = yRadian;
			pAngle->z = 0.f;
		}
	} else {
		pAngle->x = atan2f( (xy2 - wz2), (1.f - (xx2 + zz2)) );
		pAngle->y = yRadian;
		pAngle->z = 0.f;
	}
	return S_OK;
}

// NH[^jI𐶐܂B
HRESULT CalcUtil::CreateEuler(MMD_VECTOR4* pQuat, const MMD_VECTOR3* pAngle)
{
	if(pAngle == NULL || pQuat == NULL) {
		return E_POINTER;
	}
	float	xRadian = pAngle->x * 0.5f;
	float	yRadian = pAngle->y * 0.5f;
	float	zRadian = pAngle->z * 0.5f;
	float	sinX = sinf( xRadian );
	float	cosX = cosf( xRadian );
	float	sinY = sinf( yRadian );
	float	cosY = cosf( yRadian );
	float	sinZ = sinf( zRadian );
	float	cosZ = cosf( zRadian );

	pQuat->x = sinX * cosY * cosZ - cosX * sinY * sinZ;
	pQuat->y = cosX * sinY * cosZ + sinX * cosY * sinZ;
	pQuat->z = cosX * cosY * sinZ - sinX * sinY * cosZ;
	pQuat->w = cosX * cosY * cosZ + sinX * sinY * sinZ;
	return S_OK;
}

// pxsȂ܂B
HRESULT CalcUtil::LimitAngle(MMD_VECTOR4* pDest, const MMD_VECTOR4* pSource)
{
	if(pDest == NULL || pSource == NULL) {
		return E_POINTER;
	}
	MMD_VECTOR3	angle;

	HRESULT hr = ToEuler(&angle, pSource);
	if(FAILED(hr)) {
		return hr;
	}

	if( angle.x < -3.14159f ) {
		angle.x = -3.14159f;
	}
	if( -0.002f < angle.x )	{
		angle.x = -0.002f;
	}
	angle.y = 0.0f;
	angle.z = 0.0f;

	return CreateEuler(pDest, &angle);
}

CalcUtil::CalcUtil(void)
{
}

CalcUtil::~CalcUtil(void)
{
}
