#include "StdAfx.h"
#include "MMDBone.h"
#include "../CalcUtil.h"
#include "../BezierCurve.h"

/**
* Motionpad̑`܂B
*/
typedef struct {
	char	cInterpolationX[16];
	char	cInterpolationY[16];
	char	cInterpolationZ[16];
	char	cInterpolationRot[16];
}MMD_MOTION_PAD;

// [Vmr܂B
bool MMDBone::Less(const MMDBone::Motion* pLeft, const MMDBone::Motion* pRight)
{
	if(pLeft == NULL || pRight == NULL) {
		return false;
	}
	int f1 = 0;
	int f2 = 0;
	HRESULT hr = pLeft->GetFrameNo(& f1);
	if(FAILED(hr)) {
		return false;
	}
	
	hr = pRight->GetFrameNo(& f2);
	if(FAILED(hr)) {
		return false;
	}

	if(f1 < f2) {
		return true;
	}
	return false;
}

// \z܂B
MMDBone::MMDBone(const PMD_BONE_RECORD* pBone, HRESULT* phr)
{
	m_pParent = NULL;
	m_pChild = NULL;
	m_bIKLimitAngle = false;

	if(pBone == NULL) {
		*phr = E_POINTER;
		return;
	}
	m_bone = *pBone;

	HRESULT hr = CalcUtil::ToZero(& m_offset);
	if(FAILED(hr)) {
		*phr = hr;
		return;
	}

	hr = CalcUtil::ToZero(& m_invTransform);
	if(FAILED(hr)) {
		*phr = hr;
		return;
	}

	hr = CalcUtil::ToZero(& m_effectPosition);
	if(FAILED(hr)) {
		*phr = hr;
		return;
	}

	hr = CalcUtil::ToZero(& m_effectRotation);
	if(FAILED(hr)) {
		*phr = hr;
		return;
	}

	hr = CalcUtil::ToZero(& m_effectLocal);
	if(FAILED(hr)) {
		*phr = hr;
		return;
	}

	hr = CalcUtil::ToZero(& m_effectSkinning);
	if(FAILED(hr)) {
		*phr = hr;
		return;
	}
}

// j܂B
MMDBone::~MMDBone()
{
	ClearMotion();
}

// ܂B
HRESULT MMDBone::Init(vector<MMDBone*>& bones)
{
	if((m_bone.parent & 0x8000) == 0) {
		if(m_bone.parent >= bones.size()) {
			return E_UNEXPECTED;
		}
		m_pParent = bones[m_bone.parent];

		m_offset.x = m_bone.pos[0] - m_pParent->m_bone.pos[0];
		m_offset.y = m_bone.pos[1] - m_pParent->m_bone.pos[1];
		m_offset.z = m_bone.pos[2] - m_pParent->m_bone.pos[2];
	}else{
		m_offset.x = m_bone.pos[0];
		m_offset.y = m_bone.pos[1];
		m_offset.z = m_bone.pos[2];
	}

	if((m_bone.to & 0x8000) == 0) {
		if(m_bone.to >= bones.size()) {
			return E_UNEXPECTED;
		}
		m_pChild = bones[m_bone.to];
	}

	HRESULT hr = CalcUtil::GenerateIdentity(& m_invTransform);
	if(FAILED(hr)) {
		return hr;
	}

	m_invTransform.values[3][0] = -m_bone.pos[0];
	m_invTransform.values[3][1] = -m_bone.pos[1];
	m_invTransform.values[3][2] = -m_bone.pos[2];

	return Reset();
}

// vZp̍sȂǂZbg܂B
HRESULT MMDBone::Reset()
{
	HRESULT hr = CalcUtil::ToZero(& m_effectPosition);
	if(FAILED(hr)) {
		return hr;
	}
	hr = CalcUtil::ToZero(& m_effectRotation);
	if(FAILED(hr)) {
		return hr;
	}
	m_effectRotation.w = 1.0f;

	hr = CalcUtil::GenerateIdentity(& m_effectLocal);
	if(FAILED(hr)) {
		return hr;
	}
	m_effectLocal.values[3][0] = m_bone.pos[0];
	m_effectLocal.values[3][1] = m_bone.pos[1];
	m_effectLocal.values[3][2] = m_bone.pos[2];

	return S_OK;
}

// [Vj܂B
HRESULT MMDBone::ClearMotion()
{
	for(vector<Motion*>::size_type i = 0; i < m_motions.size(); i ++) {
		Motion* pMotion = m_motions[i];
		delete pMotion;
		m_motions[i] = NULL;
	}
	m_motions.clear();
	return S_OK;
}

// O擾܂B
HRESULT MMDBone::GetName(string& name)
{
	char buf[21];
	strncpy(buf, m_bone.name, 20);
	buf[20] = '\0';
	name = buf;
	return S_OK;
}

// ǉΏۂǂ𔻒肵܂B
HRESULT MMDBone::IsTarget(const VMD_MOTION_RECORD* pMotion)
{
	if(pMotion == NULL) {
		return E_POINTER;
	}
	if(strncmp(m_bone.name, pMotion->name, 15) == 0) {
		return S_OK;
	}
	return S_FALSE;
}

// [Vǉ܂B
HRESULT MMDBone::AddMotion(const VMD_MOTION_RECORD* pMotion)
{
	if(pMotion == NULL) {
		return E_POINTER;
	}
	HRESULT hr = S_OK;
	Motion* pMot = new Motion(pMotion, & hr);
	if(FAILED(hr)) {
		delete pMot;
		pMot = NULL;
		return hr;
	}
	m_motions.push_back(pMot);
	return S_OK;
}

// XLjOsXV܂B
HRESULT MMDBone::UpdateSkinning()
{
	HRESULT hr = CalcUtil::Multiply(& m_effectSkinning, & m_invTransform, & m_effectLocal);
	if(FAILED(hr)) {
		return hr;
	}
	return S_OK;
}

// XLjOs𒸓_ɓKp܂B(1_)
HRESULT MMDBone::ApplySkinning(MMD_VERTEX_TEXUSE* pDest, const MMD_VERTEX_TEXUSE* pOriginal)
{
	if(pDest == NULL || pOriginal == NULL) {
		return E_POINTER;
	}
	HRESULT hr = CalcUtil::Transform(& (pDest->point), & (pOriginal->point), & m_effectSkinning);
	if(FAILED(hr)) {
		return hr;
	}

	return CalcUtil::Rotate(& (pDest->normal), &(pOriginal->normal), & m_effectSkinning);
}

// XLjOs𒸓_ɓKp܂B(2_)
HRESULT MMDBone::ApplySkinning(MMD_VERTEX_TEXUSE* pDest, const MMD_VERTEX_TEXUSE* pOriginal, MMDBone* pBone, float bweight)
{
	if(pDest == NULL || pOriginal == NULL || pBone == NULL) {
		return E_POINTER;
	}

	MMD_MATRIX skinning;
	HRESULT hr = CalcUtil::ToZero(& skinning);
	if(FAILED(hr)) {
		return hr;
	}

	hr = CalcUtil::Lerp(& skinning, & m_effectSkinning, & (pBone->m_effectSkinning), bweight);
	if(FAILED(hr)) {
		return hr;
	}

	hr = CalcUtil::Transform(& (pDest->point), & (pOriginal->point), & skinning);
	if(FAILED(hr)) {
		return hr;
	}

	return CalcUtil::Rotate(& (pDest->normal), &(pOriginal->normal), & skinning);
}

// [V̂߂̏sȂ܂B
HRESULT MMDBone::PrepareMotion()
{
	std::sort(m_motions.begin(), m_motions.end(), MMDBone::Less);
	return S_OK;
}

// [VXV܂B
HRESULT MMDBone::UpdateMotion(float elapsedFrame)
{
	Motion* pBegin = NULL;
	Motion* pEnd = NULL;
	float offset = 0.0f;
	HRESULT hr = FindMotion(elapsedFrame, & pBegin, & pEnd, & offset);
	if(FAILED(hr)) {
		return hr;
	}
	if(hr == S_FALSE) {
		return S_OK;
	}

	if(pEnd == NULL) {
		hr = pBegin->GetVectors(& m_effectPosition, & m_effectRotation);
	}else{
		MMD_VECTOR3 beginPos, endPos;
		MMD_VECTOR4 beginQt, endQt;

		hr = pBegin->GetVectors(& beginPos, & beginQt);
		if(FAILED(hr)) {
			return hr;
		}

		hr = pEnd->GetVectors(& endPos, & endQt);
		if(FAILED(hr)) {
			return hr;
		}

		hr = pEnd->Lerp(& m_effectPosition, & beginPos, & endPos, offset);
		if(FAILED(hr)) {
			return hr;
		}

		hr = pEnd->Lerp(& m_effectRotation, & beginQt, & endQt, offset);
	}

	if(FAILED(hr)) {
		return hr;
	}
	return S_OK;
}

// ϊpsXV܂B
HRESULT MMDBone::UpdateMatrix()
{
	HRESULT hr = CalcUtil::QuaternionToMatrix(& m_effectLocal, & m_effectRotation);
	if(FAILED(hr)) {
		return hr;
	}

	m_effectLocal.values[3][0] = m_effectPosition.x + m_offset.x;
	m_effectLocal.values[3][1] = m_effectPosition.y + m_offset.y;
	m_effectLocal.values[3][2] = m_effectPosition.z + m_offset.z;

	if(m_pParent != NULL) {
		hr = CalcUtil::Multiply(& m_effectLocal, & m_effectLocal, &(m_pParent->m_effectLocal));
		if(FAILED(hr)) {
			return hr;
		}
	}

	return S_OK;
}

// őt[擾܂B
HRESULT MMDBone::GetMaxFrame(int* pFrameNo)
{
	if(pFrameNo == NULL) {
		return E_POINTER;
	}
	if(m_motions.size() == 0) {
		return S_FALSE;
	}
	return m_motions[m_motions.size() - 1]->GetFrameNo(pFrameNo);
}

// IK֘Ãp[^XV܂B
HRESULT MMDBone::UpdateIKLimitAngle()
{
	if(strncmp(m_bone.name, "Ђ", 20) == 0 || strncmp(m_bone.name, "EЂ", 20) == 0) {
		m_bIKLimitAngle = true;
	}
	return S_OK;
}

// IKLimitAnglesǂ𔻒肵܂B
HRESULT MMDBone::IsIKLimitAngle(bool* pValue)
{
	if(pValue == NULL) {
		return E_POINTER;
	}
	*pValue = m_bIKLimitAngle;
	return S_OK;
}

// ϊs񂩂xNg擾܂B
HRESULT MMDBone::GetPositionFromLocal(MMD_VECTOR3* pVector)
{
	if(pVector == NULL) {
		return E_POINTER;
	}
	pVector->x = m_effectLocal.values[3][0];
	pVector->y = m_effectLocal.values[3][1];
	pVector->z = m_effectLocal.values[3][2];
	return S_OK;
}

// ϊs擾܂B
HRESULT MMDBone::GetLocal(MMD_MATRIX* pMatrix)
{
	if(pMatrix == NULL) {
		return E_POINTER;
	}
	memcpy(pMatrix, & m_effectLocal, sizeof(MMD_MATRIX));
	return S_OK;
}

// sړE]擾܂B
HRESULT MMDBone::GetVectors(MMD_VECTOR3* pPos, MMD_VECTOR4* pQt)
{
	if(pPos == NULL || pQt == NULL) {
		return E_POINTER;
	}
	memcpy(pPos, & m_effectPosition, sizeof(MMD_VECTOR3));
	memcpy(pQt, & m_effectRotation, sizeof(MMD_VECTOR4));
	return S_OK;
}

// sړE]ݒ肵܂B
HRESULT MMDBone::SetVectors(const MMD_VECTOR3* pPos, const MMD_VECTOR4* pQt)
{
	if(pPos == NULL || pQt == NULL) {
		return E_POINTER;
	}
	memcpy(& m_effectPosition, pPos, sizeof(MMD_VECTOR3));
	memcpy(& m_effectRotation, pQt, sizeof(MMD_VECTOR4));
	return S_OK;
}

// ɑΉ郂[V擾܂B
HRESULT MMDBone::FindMotion(float elapsedTime, Motion** ppBegin, Motion** ppEnd, float* pOffset)
{
	if(ppBegin == NULL || ppEnd == NULL || pOffset == NULL) {
		return E_POINTER;
	}
	if(m_motions.size() == 0) {
		return S_FALSE;
	}
	int fr = 0;
	HRESULT hr = m_motions[0]->GetFrameNo(& fr);
	if(elapsedTime <= fr) {
		*ppBegin = m_motions[0];
		return S_OK;
	}
	
	hr = m_motions[m_motions.size() - 1]->GetFrameNo(& fr);
	if(elapsedTime >= fr) {
		*ppBegin = m_motions[m_motions.size() - 1];
		return S_OK;
	}
	for(vector<Motion>::size_type i = 0; i < m_motions.size() - 1; i ++) {
		Motion* pSMotion = m_motions[i];
		Motion* pEMotion = m_motions[i + 1];

		int fr1 = 0;
		int fr2 = 0;
		hr = pSMotion->GetFrameNo(& fr1);
		if(FAILED(hr)) {
			return hr;
		}
		hr = pEMotion->GetFrameNo(& fr2);
		if(FAILED(hr)) {
			return hr;
		}

		if(fr1 <= elapsedTime && elapsedTime <= fr2) {
			*ppBegin = pSMotion;
			*ppEnd = pEMotion;
			*pOffset = (elapsedTime - fr1) / (fr2 - fr1);
			return S_OK;
		}
	}
	return S_FALSE;
}

// \z܂B
MMDBone::Motion::Motion(const VMD_MOTION_RECORD* pMotion, HRESULT* phr)
{
	m_pXBez = NULL;
	m_pYBez = NULL;
	m_pZBez = NULL;
	m_pRotBez = NULL;

	if(pMotion == NULL) {
		*phr = E_POINTER;
		return;
	}
	m_motion = *pMotion;
	m_pos.x = m_motion.pos[0];
	m_pos.y = m_motion.pos[1];
	m_pos.z = m_motion.pos[2];
	MMD_VECTOR4 qt;
	qt.x = m_motion.qt[0];
	qt.y = m_motion.qt[1];
	qt.z = m_motion.qt[2];
	qt.w = m_motion.qt[3];
	HRESULT hr = CalcUtil::Normalize(& m_qt, & qt);
	if(FAILED(hr)) {
		*phr = hr;
		return;
	}

	MMD_MOTION_PAD* pad = (MMD_MOTION_PAD*) m_motion.pad;
	MMD_VECTOR2 p1, p2;

	p1.x = pad->cInterpolationX[0];
	p1.y = pad->cInterpolationX[4];
	p2.x = pad->cInterpolationX[8];
	p2.y = pad->cInterpolationX[12];
	m_pXBez = new BezierCurve(& p1, & p2, & hr);
	if(FAILED(hr)) {
		*phr = hr;
		return;
	}
	p1.x = pad->cInterpolationY[0];
	p1.y = pad->cInterpolationY[4];
	p2.x = pad->cInterpolationY[8];
	p2.y = pad->cInterpolationY[12];
	m_pYBez = new BezierCurve(& p1, & p2, & hr);
	if(FAILED(hr)) {
		*phr = hr;
		return;
	}
	p1.x = pad->cInterpolationZ[0];
	p1.y = pad->cInterpolationZ[4];
	p2.x = pad->cInterpolationZ[8];
	p2.y = pad->cInterpolationZ[12];
	m_pZBez = new BezierCurve(& p1, & p2, & hr);
	if(FAILED(hr)) {
		*phr = hr;
		return;
	}
	p1.x = pad->cInterpolationRot[0];
	p1.y = pad->cInterpolationRot[4];
	p2.x = pad->cInterpolationRot[8];
	p2.y = pad->cInterpolationRot[12];
	m_pRotBez = new BezierCurve(& p1, & p2, & hr);
	if(FAILED(hr)) {
		*phr = hr;
		return;
	}
}

// j܂B
MMDBone::Motion::~Motion()
{
	if(m_pXBez != NULL) {
		delete m_pXBez;
		m_pXBez = NULL;
	}
	if(m_pYBez != NULL) {
		delete m_pYBez;
		m_pYBez = NULL;
	}
	if(m_pZBez != NULL) {
		delete m_pZBez;
		m_pZBez = NULL;
	}
	if(m_pRotBez != NULL) {
		delete m_pRotBez;
		m_pRotBez = NULL;
	}
}

// t[ԍ擾܂B
HRESULT MMDBone::Motion::GetFrameNo(int* pFrameNo) const
{
	if(pFrameNo == NULL) {
		return E_POINTER;
	}
	*pFrameNo = m_motion.frame_no;
	return S_OK;
}

// ړxNgƃNH[^jI擾܂B
HRESULT MMDBone::Motion::GetVectors(MMD_VECTOR3* pPos, MMD_VECTOR4* pQt)
{
	if(pPos == NULL || pQt == NULL) {
		return E_POINTER;
	}
	*pPos = m_pos;
	*pQt = m_qt;
	return S_OK;
}

// ԏsȂ܂B
HRESULT MMDBone::Motion::Lerp(MMD_VECTOR3* pDest, const MMD_VECTOR3* pValue1, const MMD_VECTOR3* pValue2, float weight)
{
	if(pDest == NULL || pValue1 == NULL || pValue2 == NULL) {
		return E_POINTER;
	}

	float posLerp = 0.0f;
	HRESULT hr = m_pXBez->GetValue(weight, & posLerp);
	if(FAILED(hr)) {
		return hr;
	}
	pDest->x = pValue1->x * (1.0f - posLerp) + pValue2->x * posLerp;

	hr = m_pYBez->GetValue(weight, & posLerp);
	if(FAILED(hr)) {
		return hr;
	}
	pDest->y = pValue1->y * (1.0f - posLerp) + pValue2->y * posLerp;

	hr = m_pZBez->GetValue(weight, & posLerp);
	if(FAILED(hr)) {
		return hr;
	}
	pDest->z = pValue1->z * (1.0f - posLerp) + pValue2->z * posLerp;

	return S_OK;
}

// ԏsȂ܂B
HRESULT MMDBone::Motion::Lerp(MMD_VECTOR4* pDest, const MMD_VECTOR4* pValue1, const MMD_VECTOR4* pValue2, float weight)
{
	if(pDest == NULL || pValue1 == NULL || pValue2 == NULL) {
		return E_POINTER;
	}

	float rotLerp = 0.0f;
	HRESULT hr = m_pRotBez->GetValue(weight, & rotLerp);
	if(FAILED(hr)) {
		return hr;
	}

	return CalcUtil::Lerp(pDest, pValue1, pValue2, rotLerp);
}
