#include "StdAfx.h"
#include "MMDMaterial.h"

// \z܂B
MMDMaterial::MMDMaterial(const PMD_MATERIAL_RECORD* pMaterial, HRESULT* phr)
{
	ZeroMemory(& m_texture, sizeof(m_texture));
	m_pCurrentVertexes = NULL;
	m_pVertexes = NULL;

	if(pMaterial == NULL) {
		*phr = E_POINTER;
		return;
	}
	m_material = *pMaterial;
}

// j܂B
MMDMaterial::~MMDMaterial(void)
{
	// eNX`̉̓voC_s
	if(m_pVertexes != NULL) {
		delete[] m_pVertexes;
		m_pVertexes = NULL;
	}
	if(m_pCurrentVertexes != NULL) {
		delete[] m_pCurrentVertexes;
		m_pCurrentVertexes = NULL;
	}
}

// ܂B
HRESULT MMDMaterial::Init(MMDVertexList* pVertexList, vector<MMDBone*> bones, int offset, int* pNextOffset)
{
	if(pVertexList == NULL || pNextOffset == NULL) {
		return E_POINTER;
	}

	m_pCurrentVertexes = new MMD_VERTEX_TEXUSE[m_material.nEdges];
	if(m_pCurrentVertexes == NULL) {
		return E_OUTOFMEMORY;
	}

	m_pVertexes = new MMD_VERTEX_UNIT[m_material.nEdges];
	if(m_pVertexes == NULL) {
		return E_OUTOFMEMORY;
	}

	ZeroMemory(m_pCurrentVertexes, sizeof(MMD_VERTEX_TEXUSE) * m_material.nEdges);
	ZeroMemory(m_pVertexes, sizeof(MMD_VERTEX_UNIT) * m_material.nEdges);

	for(size_t i = 0; i < m_material.nEdges; i ++) {
		MMD_VERTEX_DESC* pOriginalVert = NULL;
		HRESULT hr = pVertexList->GetVertexDesc(i + offset, & pOriginalVert);
		if(FAILED(hr)) {
			return hr;
		}

		m_pVertexes[i].pCurrentVert = m_pCurrentVertexes + i;
		m_pVertexes[i].pOriginalVert = pOriginalVert;
		*(m_pVertexes[i].pCurrentVert) = m_pVertexes[i].pOriginalVert->current;

		PMD_VERTEX_RECORD vert;
		hr = pVertexList->GetVertex(i + offset, & vert);
		if(FAILED(hr)) {
			return hr;
		}

		if(vert.b1 >= bones.size()) {
			return E_UNEXPECTED;
		}
		m_pVertexes[i].pOriginalVert->pBones[0] = bones[vert.b1];
		if(vert.b2 >= bones.size()) {
			return E_UNEXPECTED;
		}
		m_pVertexes[i].pOriginalVert->pBones[1] = bones[vert.b2];
		m_pVertexes[i].pOriginalVert->bweight = (vert.bw / 100.0f);
	}

	*pNextOffset = offset + m_material.nEdges;
	return S_OK;
}

// eNX`̏sȂ܂B
HRESULT MMDMaterial::Prepare(IMMDTextureProvider* pTextureProvider)
{
	if(pTextureProvider == NULL) {
		return E_POINTER;
	}
	if(m_material.textureFileName[0] == '\0') {
		// eNX`w
		return S_OK;
	}
	return pTextureProvider->Load(m_material.textureFileName, & m_texture);
}

// _obt@XV܂B
HRESULT MMDMaterial::UpdateVertexBuffer(float scale)
{
	if(m_pVertexes == NULL) {
		return E_UNEXPECTED;
	}
	for(DWORD i = 0; i < m_material.nEdges; i ++) {
		*(m_pVertexes[i].pCurrentVert) = m_pVertexes[i].pOriginalVert->current;
		m_pVertexes[i].pCurrentVert->point.x *= scale;
		m_pVertexes[i].pCurrentVert->point.y *= scale;
		m_pVertexes[i].pCurrentVert->point.z *= -1.0f*scale;
		m_pVertexes[i].pCurrentVert->normal.z *= -1.0f;
	}
	return S_OK;
}

// `悵܂B
HRESULT MMDMaterial::Draw()
{
	GLint				intFrontFace;

	glPushMatrix();
		//^ZR͒_̕т\ʂ݂ĉE
		glGetIntegerv(GL_FRONT_FACE,&intFrontFace);
		glFrontFace(GL_CW);

	// `
	//glShadeModel(((obj->isShadingFlat))?GL_FLAT:GL_SMOOTH);
	float dalpha = 1.0;
	GLint				bindGL_TEXTURE_2D	= 0;
	GLboolean			isGL_TEXTURE_2D		= GL_FALSE;
	GLboolean			isGL_BLEND			= GL_FALSE;
	GLint				blendGL_SRC_ALPHA	= 0;

	GLfloat				matenv[4];
	matenv[0] = m_material.diffuse.r;
	matenv[1] = m_material.diffuse.g;
	matenv[2] = m_material.diffuse.b;
	matenv[3] = m_material.diffuse.a * dalpha;
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, matenv);

	matenv[0] = m_material.ambient.r;
	matenv[1] = m_material.ambient.g;
	matenv[2] = m_material.ambient.b;
	matenv[3] = dalpha;
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, matenv);

	matenv[0] = m_material.specular.r;
	matenv[1] = m_material.specular.g;
	matenv[2] = m_material.specular.b;
	matenv[3] = dalpha;
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, matenv);

	matenv[0] = m_material.ambient.r;
	matenv[1] = m_material.ambient.g;
	matenv[2] = m_material.ambient.b;
	matenv[3] = dalpha;
	glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, matenv);

	glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, m_material.shininess);

				/*if ( mat->isUseTexture) {	// eNX`ꍇ*/
					glEnableClientState( GL_VERTEX_ARRAY );
					glEnableClientState( GL_NORMAL_ARRAY );
					if(m_texture.tex_mem_height > 0 && m_texture.tex_mem_width > 0) {
						glEnableClientState( GL_TEXTURE_COORD_ARRAY );
					}

					isGL_TEXTURE_2D = glIsEnabled(GL_TEXTURE_2D);
					isGL_BLEND = glIsEnabled(GL_BLEND);
					glGetIntegerv(GL_TEXTURE_BINDING_2D,&bindGL_TEXTURE_2D);
//					glGetIntegerv(GL_BLEND_SRC_ALPHA,&blendGL_SRC_ALPHA);

					glEnable(GL_TEXTURE_2D);
					glEnable(GL_BLEND);
					glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

					if(m_texture.tex_mem_height > 0 && m_texture.tex_mem_width > 0) {
						glBindTexture(GL_TEXTURE_2D,m_texture.texture_id);
					}

					/*if ( g_isVBOSupported ) {	// _obt@gp
						base = (char *)NULL;	// AhXNULL擪
						glBindBufferARB( GL_ARRAY_BUFFER_ARB, mat->VBO_id ); // _obt@т
					}
					else {*/
						// _z̎́AAhX̂܂ܓ
					MMD_VERTEX_TEXUSE* pVert = m_pVertexes[0].pCurrentVert;
					char* base = (char *)(& pVert->point);
					//}

					// _zݒ
					int offset = (int)( (char *)(& pVert->point) - (char *)(& pVert->point) );
					glVertexPointer( 3, GL_FLOAT, sizeof(MMD_VERTEX_TEXUSE) , base + offset );

					if(m_texture.tex_mem_height > 0 && m_texture.tex_mem_width > 0) {
						// eNX`Wzݒ
						offset = (int)((char *)(& pVert->uv)-(char *)(& pVert->point));
						glTexCoordPointer( 2, GL_FLOAT, sizeof(MMD_VERTEX_TEXUSE) , base + offset );
					}

					// @zݒ
					offset = (int)((char *)(& pVert->normal)-(char *)(& pVert->point));
					glNormalPointer( GL_FLOAT, sizeof(MMD_VERTEX_TEXUSE) , base + offset );

					// Fݒ
					glColor4f(1.0f,1.0f,1.0f,1.0f);

					// `s
					glDrawArrays( GL_TRIANGLES, 0, m_material.nEdges );

					glBindTexture(GL_TEXTURE_2D,bindGL_TEXTURE_2D);
					if( isGL_BLEND == GL_FALSE ) glDisable(GL_BLEND);
					if( isGL_TEXTURE_2D == GL_FALSE ) glDisable(GL_TEXTURE_2D);

					/*if ( g_isVBOSupported ) {						// _obt@gp
						glBindBufferARB( GL_ARRAY_BUFFER_ARB, 0 );	// _obt@ftHg
					}*/

					glDisableClientState( GL_VERTEX_ARRAY );
					glDisableClientState( GL_NORMAL_ARRAY );
					if(m_texture.tex_mem_height > 0 && m_texture.tex_mem_width > 0) {
						glDisableClientState( GL_TEXTURE_COORD_ARRAY );
					}
				/*}
				else {	// eNX`Ȃꍇ

					glEnableClientState( GL_VERTEX_ARRAY );
					glEnableClientState( GL_NORMAL_ARRAY );
				//	glEnableClientState( GL_COLOR_ARRAY );

					isGL_BLEND = glIsEnabled(GL_BLEND);
					glEnable(GL_BLEND);
					glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

					if ( g_isVBOSupported ) {	// _obt@gp
						base = (char *)NULL;
						glBindBufferARB( GL_ARRAY_BUFFER_ARB, mat->VBO_id );
					}
					else {
						base = (char *)mat->vertex_p[0].point;
					}

					// _zݒ
					offset = (int)((char *)mat->vertex_p[0].point-(char *)mat->vertex_p[0].point);
					glVertexPointer( 3, GL_FLOAT, sizeof(VERTEX_NOTEX) , base+offset );

					// @zݒ
					offset = (int)((char *)mat->vertex_p[0].normal-(char *)mat->vertex_p[0].point);
					glNormalPointer( GL_FLOAT, sizeof(VERTEX_NOTEX) , base+offset );

					// Fݒ
					glColor4f(mat->color[0],mat->color[1],mat->color[2],mat->color[3]);
				//	offset = (int)((char *)mat->vertex_p[0].color-(char *)mat->vertex_p[0].point);
				//	glColorPointer(4,GL_FLOAT,sizeof(VERTEX_NOTEX),base+offset);

					// `s
					glDrawArrays( GL_TRIANGLES, 0, mat->datanum );

					if( isGL_BLEND == GL_FALSE ) glDisable(GL_BLEND);
					if ( g_isVBOSupported ) {						// _obt@gp
						glBindBufferARB( GL_ARRAY_BUFFER_ARB, 0 );	// _obt@ftHg
					}

				//	glDisableClientState( GL_COLOR_ARRAY );
					glDisableClientState( GL_VERTEX_ARRAY );
					glDisableClientState( GL_NORMAL_ARRAY );

				}
			}*/

		//^ZR͒_̕т\ʂ݂ĉEi̐ݒɂǂj
		glFrontFace(intFrontFace);
	glPopMatrix();

	// `
	return S_OK;
}
