#include "StdAfx.h"
#include "MMDVertexList.h"
#include "MMDBone.h"

// \z܂B
MMDVertexList::MMDVertexList(const PMD_VERTEX_CHUNK* pVertexes, const PMD_INDEX_CHUNK* pIndices, HRESULT* phr)
{
	m_pVertexes = NULL;

	if(pVertexes == NULL || pIndices == NULL) {
		*phr = E_POINTER;
		return;
	}
	m_vertexes = *pVertexes;
	m_indices = *pIndices;

	HRESULT hr = CreateVertexDesc();
	if(FAILED(hr)) {
		*phr = hr;
		return;
	}
}

// j܂B
MMDVertexList::~MMDVertexList()
{
	if(m_pVertexes != NULL) {
		for(PMD_VERTEX_CHUNK::size_type i = 0; i < m_vertexes.size(); i ++) {
			MMD_VERTEX_DESC* pDesc = m_pVertexes[i];
			delete pDesc;
			m_pVertexes[i] = NULL;
		}
		delete[] m_pVertexes;
		m_pVertexes = NULL;
	}
}

// _擾܂B
HRESULT MMDVertexList::GetVertexDescs(MMD_VERTEX_DESC*** pppDescs)
{
	if(pppDescs == NULL) {
		return E_POINTER;
	}
	*pppDescs = m_pVertexes;
	return S_OK;
}

// _擾܂B
HRESULT MMDVertexList::GetVertexDesc(int index, MMD_VERTEX_DESC** ppDescs)
{
	if(ppDescs == NULL) {
		return E_POINTER;
	}
	if(m_indices.size() <= (PMD_INDEX_CHUNK::size_type) index) {
		return E_INVALIDARG;
	}
	int vindex = m_indices[index];
	*ppDescs = m_pVertexes[vindex];
	return S_OK;
}

// CfbNXɑΉ钸_擾܂B
HRESULT MMDVertexList::GetVertex(int index, PMD_VERTEX_RECORD* pVertex)
{
	if(pVertex == NULL) {
		return E_POINTER;
	}
	if(m_indices.size() <=  (PMD_INDEX_CHUNK::size_type) index) {
		return E_INVALIDARG;
	}
	int vindex = m_indices[index];
	*pVertex = m_vertexes[vindex];
	return S_OK;
}

// XLjOsXV܂B
HRESULT MMDVertexList::UpdateSkinning()
{
	for(PMD_VERTEX_CHUNK::size_type i = 0; i < m_vertexes.size(); i ++) {
		MMD_VERTEX_DESC* pVert = m_pVertexes[i];

		HRESULT hr = S_OK;
		if(pVert->bweight == 0.0f) {
			MMDBone* pBone = pVert->pBones[1];
			if(pBone == NULL) {
				return E_POINTER;
			}
			hr = pBone->ApplySkinning(&(pVert->current), &(pVert->faced));
		}else if(pVert->bweight >= 0.9999f) {
			MMDBone* pBone = pVert->pBones[0];
			if(pBone == NULL) {
				return E_POINTER;
			}
			hr = pBone->ApplySkinning(&(pVert->current), &(pVert->faced));
		}else{
			MMDBone* pBone0 = pVert->pBones[0];
			if(pBone0 == NULL) {
				return E_POINTER;
			}
			MMDBone* pBone1 = pVert->pBones[1];
			if(pBone1 == NULL) {
				return E_POINTER;
			}
			hr = pBone0->ApplySkinning(&(pVert->current), &(pVert->faced), pBone1, pVert->bweight);
		}
		if(FAILED(hr)) {
			return hr;
		}
	}

	return S_OK;
}

// _Zbg܂B
HRESULT MMDVertexList::ResetVertexes()
{
	for(PMD_VERTEX_CHUNK::size_type i = 0; i < m_vertexes.size(); i ++) {
		MMD_VERTEX_DESC* pDesc = m_pVertexes[i];

		pDesc->faced = pDesc->original;
		pDesc->current = pDesc->faced;
	}
	return S_OK;
}

// _obt@𐶐܂B
HRESULT MMDVertexList::CreateVertexDesc()
{
	size_t vsz = sizeof(MMD_VERTEX_DESC*) * m_vertexes.size();
	MMD_VERTEX_DESC** ppNewDescs = new MMD_VERTEX_DESC*[m_vertexes.size()];
	if(ppNewDescs == NULL) {
		return E_OUTOFMEMORY;
	}
	ZeroMemory(ppNewDescs, vsz);
	m_pVertexes = ppNewDescs;
	for(size_t i = 0; i < m_vertexes.size(); i ++) {
		PMD_VERTEX_RECORD vert = m_vertexes[i];
		MMD_VERTEX_DESC* pTargetVert = new MMD_VERTEX_DESC;
		if(pTargetVert == NULL) {
			return E_OUTOFMEMORY;
		}
		ZeroMemory(pTargetVert, sizeof(MMD_VERTEX_DESC));
		pTargetVert->original.point.x = vert.x;
		pTargetVert->original.point.y = vert.y;
		pTargetVert->original.point.z = vert.z;
		pTargetVert->original.normal.x = vert.nx;
		pTargetVert->original.normal.y = vert.ny;
		pTargetVert->original.normal.z = vert.nz;
		pTargetVert->original.uv.x = vert.tx;
		pTargetVert->original.uv.y = vert.ty;
		pTargetVert->faced = pTargetVert->original;
		pTargetVert->current = pTargetVert->faced;
		ppNewDescs[i] = pTargetVert;
	}
	return S_OK;
}
